@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,11 @@
 //  Current versions can be found at www.libavg.de
 //
 
+#ifdef _WIN32
+#include "../avgconfig_win.h"
+#else
 #include "../avgconfig.h"
+#endif
 
 // These defines regularly cause multiple definition warnings and are usually
 // completely unneeded.
