@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,11 +26,12 @@
 
 #define AVG_ERR_UNKNOWN -1
 #define AVG_ERR_XML_PARSE 1
-#define AVG_ERR_XML_EMPTY 2
-#define AVG_ERR_XML_NODE_UNKNOWN 3
-#define AVG_ERR_XML_DUPLICATE_ID 4
-#define AVG_ERR_VIDEO_INIT_FAILED 5
-#define AVG_ERR_VIDEO_GENERAL 6
+#define AVG_ERR_XML_VALID 2
+#define AVG_ERR_XML_EMPTY 3
+#define AVG_ERR_XML_NODE_UNKNOWN 4
+#define AVG_ERR_XML_DUPLICATE_ID 5
+#define AVG_ERR_VIDEO_INIT_FAILED 6
+#define AVG_ERR_VIDEO_GENERAL 7
 #define AVG_ERR_FONT_INIT_FAILED 10
 #define AVG_ERR_VIDEO_LOAD_FAILED 11
 #define AVG_ERR_UNSUPPORTED 12
@@ -44,6 +45,11 @@
 #define AVG_ERR_CANT_PARSE_STRING 20
 #define AVG_ERR_INVALID_CAPTURE 21
 #define AVG_ERR_QUEUE_EMPTY 22
+#define AVG_ERR_NO_NODE 23
+#define AVG_ERR_NO_ARG 24
+#define AVG_ERR_INVALID_ARGS 25
+#define AVG_ERR_NO_BUILDER 26
+#define AVG_ERR_TYPE 27
 
 namespace avg {
  
@@ -56,7 +62,7 @@ class Exception
         virtual int GetCode () const;
         virtual const std::string& GetStr () const;
 
-	private:
+    private:
         int m_Code;
         std::string m_sErr;
 };
