@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,8 @@
 namespace avg {
     
 std::string getPath(const std::string& Filename);
+std::string getFilenamePart(const std::string& Filename);
+bool isAbsPath(const std::string& path);
 
 bool fileExists(const std::string& FileName);
 
