@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,18 +21,18 @@
 
 #include "Logger.h"
 
-#ifndef _WIN32
-#include <sys/time.h>
-#else
+#ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN  /* somewhat limit Win32 pollution */
 #include <Winsock2.h>
 #undef ERROR
 #undef WARNING
 #include <time.h>
+#else
+#include <sys/time.h>
+#include <syslog.h>
 #endif
 #include <fstream>
 #include <iomanip>
-#include <syslog.h>
 #include <boost/thread.hpp>
 using namespace std;
 
@@ -50,6 +50,7 @@ const long Logger::ERROR=128;
 const long Logger::MEMORY=256;
 const long Logger::APP=512;
 const long Logger::LOGGER=1024;
+const long Logger::WATCHDOG=2048;
 
 Logger* Logger::m_pLogger = 0;
 boost::mutex log_Mutex;
@@ -58,10 +59,10 @@ Logger * Logger::get()
 {
 
     if (!m_pLogger) {
-        {
+    {
         boost::mutex::scoped_lock Lock(log_Mutex);
         m_pLogger = new Logger;
-        }
+    }
         m_pLogger->trace(LOGGER, "Logging started ");
     }
     return m_pLogger;
@@ -90,10 +91,12 @@ void Logger::setConsoleDest()
 
 void Logger::setFileDest(const std::string& sFName)
 {
-    boost::mutex::scoped_lock Lock(log_Mutex);
-    closeDest();
-    m_DestType = FILE;
-    m_pDest = new ofstream(sFName.c_str(), ios::out | ios::app);
+    {
+        boost::mutex::scoped_lock Lock(log_Mutex);
+        closeDest();
+        m_DestType = FILE;
+        m_pDest = new ofstream(sFName.c_str(), ios::out | ios::app);
+    }
     if (!*m_pDest) {
         m_pDest = &cerr;
         m_DestType = CONSOLE;
@@ -106,10 +109,15 @@ void Logger::setFileDest(const std::string& sFName)
 
 void Logger::setSyslogDest(int facility, int logopt)
 {
+#ifdef _WIN32
+    AVG_TRACE(ERROR, 
+            "Logging to syslog is not supported under windows. Ignoring request.");
+#else
     boost::mutex::scoped_lock Lock(log_Mutex);
     closeDest();
     m_DestType = SYSLOG;
     openlog("libavg", logopt, facility);
+#endif
 }
     
 void Logger::setCategories(int flags)
@@ -126,11 +134,9 @@ void Logger::trace(int category, const std::string& msg)
             struct tm* pTime;
 #ifdef _WIN32
             __int64 now;
-            struct tm newTime;
             _time64(&now);
-            _localtime64_s(&newTime, &now);
+            pTime = _localtime64(&now);
             unsigned millis = unsigned((now / 1000) % 1000);
-            pTime = &newTime;
 #else
             struct timeval time;
             gettimeofday(&time, NULL);
@@ -145,6 +151,7 @@ void Logger::trace(int category, const std::string& msg)
             (*m_pDest) << msg << endl;
             m_pDest->flush();
         } else {
+#ifndef _WIN32
             int prio;
             switch(category) {
                 case EVENTS:
@@ -171,6 +178,7 @@ void Logger::trace(int category, const std::string& msg)
             }
             syslog(prio, "%s: %s", categoryToString(category),
                     msg.c_str());
+#endif 
         }
     }
 }
@@ -198,6 +206,8 @@ const char * Logger::categoryToString(int category)
             return "APP";
         case LOGGER:
             return "LOGGER";
+        case WATCHDOG:
+            return "WATCHDOG";
         default:
             return "UNKNOWN";
     }
@@ -213,8 +223,10 @@ void Logger::closeDest()
             m_pDest = 0;
             break;
         case SYSLOG:
+#ifndef _WIN32
             closelog();
-            
+#endif
+            break;
     }
 }
 
