@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,11 @@
 #include <iostream>
 #include <sstream>
 
-#ifndef WIN32
+#ifdef WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <Dbghelp.h>
+#else
 #include <cxxabi.h>
 #endif
 
@@ -38,7 +42,7 @@ namespace avg {
 using namespace std;
 
 ObjectCounter* ObjectCounter::m_pObjectCounter = 0;
-boost::mutex CounterMutex;
+boost::mutex * pCounterMutex;
 
 ObjectCounter::ObjectCounter()
 {
@@ -50,31 +54,31 @@ ObjectCounter::~ObjectCounter()
 
 ObjectCounter * ObjectCounter::get()
 {
-#ifdef DEBUG_ALLOC
     if (!m_pObjectCounter) {
-        boost::mutex::scoped_lock Lock(CounterMutex);
         m_pObjectCounter = new ObjectCounter;
+        pCounterMutex = new boost::mutex;
     }
     return m_pObjectCounter;
-#endif
 }
 
 void ObjectCounter::incRef(const std::type_info* pType)
 {
 #ifdef DEBUG_ALLOC
-    boost::mutex::scoped_lock Lock(CounterMutex);
+    boost::mutex::scoped_lock Lock(*pCounterMutex);
     TypeMap::iterator MapEntry = m_TypeMap.find(pType);
     if (MapEntry == m_TypeMap.end()) {
         m_TypeMap[pType] = 1;
     } else {
         (MapEntry->second)++;
     }
+//    cerr << "incRef " << demangle(pType->name()) << ":" << m_TypeMap[pType] << endl;
 #endif
 }
 
 void ObjectCounter::decRef(const std::type_info* pType)
 {
-    boost::mutex::scoped_lock Lock(CounterMutex);
+#ifdef DEBUG_ALLOC
+    boost::mutex::scoped_lock Lock(*pCounterMutex);
     TypeMap::iterator MapEntry = m_TypeMap.find(pType);
     if (MapEntry == m_TypeMap.end()) {
         // Can't decref a type that hasn't been incref'd.
@@ -88,6 +92,8 @@ void ObjectCounter::decRef(const std::type_info* pType)
             assert (false);
         }
     }
+//    cerr << "decRef " << demangle(pType->name()) << ":" << MapEntry->second << endl;
+#endif
 }
     
 int ObjectCounter::getCount(const std::type_info* pType)
@@ -117,9 +123,21 @@ std::string ObjectCounter::dump()
 string ObjectCounter::demangle(string s)
 {
     int rc;
-    char * pszDemangled = abi::__cxa_demangle(s.c_str(), 0, 0, &rc);
     string sResult;
-    if (rc != 0) {
+#ifdef _WIN32
+    char szDemangledName[2048];
+    rc = int(UnDecorateSymbolName(s.c_str(), szDemangledName, sizeof(szDemangledName), 
+            UNDNAME_COMPLETE));
+    if (rc) {
+        sResult = szDemangledName;
+    } else {
+        int error = GetLastError();
+        printf("UnDecorateSymbolName returned error %d\n", error);
+        sResult = s;
+    }
+#else
+    char * pszDemangled = abi::__cxa_demangle(s.c_str(), 0, 0, &rc);
+    if (rc) {
         sResult = s;
     } else {
         sResult = pszDemangled;
@@ -127,6 +145,7 @@ string ObjectCounter::demangle(string s)
     if (pszDemangled) {
         free(pszDemangled);
     }
+#endif
     return sResult;
 }
 
