@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,5 +30,22 @@ bool almostEqual(const DPoint& pt1, const DPoint& pt2)
     return (fabs(pt1.x-pt2.x)+fabs(pt1.y-pt2.y)) < 0.0001;
 }
 
+DPoint rotate(const DPoint& point, double angle, const DPoint& pivot)
+{
+    double cosVal = cos(angle);
+    double sinVal = sin(angle);
+
+    // translate pivot to origin
+    DPoint translated = point - pivot;
+    
+    // calculate rotated coordinates about the origin
+    DPoint rotated(translated.x * cosVal - translated.y * sinVal,
+        translated.x * sinVal + translated.y * cosVal);
+
+    // re-translate pivot to original position
+    rotated += pivot;
+
+    return rotated;
 }
 
+}
