@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -104,14 +104,14 @@ void TimeSource::sleepUntil(long long TargetTime)
         if (TargetTime-now<=2) {
             msleep(0);
          } else {
-            msleep(TargetTime-now-2);
+            msleep(int(TargetTime-now-2));
         }
         now = getCurrentMillisecs();
     }
 #endif
 }
     
-void TimeSource::msleep(int millisecs)
+void msleep(int millisecs)
 {
 #if _WIN32
             Sleep(millisecs);
