@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@
 
 #include <Magick++.h>
 #include <assert.h>
+#include <cstring>
 #include <iostream>
 #include <iomanip>
 #include <stdlib.h>
@@ -47,6 +48,7 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src);
 Bitmap::Bitmap(IntPoint Size, PixelFormat PF, const std::string& sName)
     : m_Size(Size),
       m_PF(PF),
+      m_pBits(0),
       m_bOwnsBits(true),
       m_sName(sName)
 {
@@ -60,6 +62,7 @@ Bitmap::Bitmap(IntPoint Size, PixelFormat PF, unsigned char * pBits,
         int Stride, bool bCopyBits, const std::string& sName)
     : m_Size(Size),
       m_PF(PF),
+      m_pBits(0),
       m_sName(sName)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
@@ -72,6 +75,7 @@ Bitmap::Bitmap(IntPoint Size, PixelFormat PF, unsigned char * pBits,
 Bitmap::Bitmap(const Bitmap& Orig)
     : m_Size(Orig.getSize()),
       m_PF(Orig.getPixelFormat()),
+      m_pBits(0),
       m_bOwnsBits(Orig.m_bOwnsBits),
       m_sName(Orig.getName()+" copy")
 {
@@ -84,6 +88,7 @@ Bitmap::Bitmap(const Bitmap& Orig)
 Bitmap::Bitmap(const Bitmap& Orig, bool bOwnsBits)
     : m_Size(Orig.getSize()),
       m_PF(Orig.getPixelFormat()),
+      m_pBits(0),
       m_bOwnsBits(bOwnsBits),
       m_sName(Orig.getName()+" copy")
 {
@@ -96,8 +101,9 @@ Bitmap::Bitmap(const Bitmap& Orig, bool bOwnsBits)
 // Creates a bitmap that is a rectangle in another bitmap. The pixels are
 // still owned by the original bitmap.
 Bitmap::Bitmap(Bitmap& Orig, const IntRect& Rect)
-    : m_Size(Rect.Width(), Rect.Height()),
+    : m_Size(Rect.size()),
       m_PF(Orig.getPixelFormat()),
+      m_pBits(0),
       m_bOwnsBits(false)
 {
 //    cerr << "Bitmap::Bitmap(Bitmap, " << Rect << "), Name: " << m_sName << endl;
@@ -115,13 +121,22 @@ Bitmap::Bitmap(Bitmap& Orig, const IntRect& Rect)
 }
 
 Bitmap::Bitmap(const std::string& sURI)
-    : m_sName(sURI)
+    : m_pBits(0),
+      m_sName(sURI)
 {
+// TODO: This function loads grayscale images as RGB. That is because Magick++
+// provides no reliable way to determine whether a bitmap is grayscale or not. Maybe
+// the imagemagick C interface is less buggy? 
+// It also returns RGB bitmaps, but I think nearly everywhere in libavg, the bytes
+// are swapped and BGR is used.
 //    cerr << "Bitmap::Bitmap(" << sURI << ")" << endl;
     Image Img;
     try {
         Img.read(sURI);
-    } catch( Magick::WarningCoder &warning ) {
+    } catch( Magick::Warning &e) {
+        cerr << e.what() << endl;
+    } catch( Magick::ErrorConfigure &e) {
+//        cerr << e.what() << endl;
     }
     PixelPacket * pSrcPixels = Img.getPixels(0, 0, Img.columns(), Img.rows());
     m_Size = IntPoint(Img.columns(), Img.rows());
@@ -134,11 +149,20 @@ Bitmap::Bitmap(const std::string& sURI)
     for (int y=0; y<m_Size.y; ++y) {
         Pixel32 * pDestLine = (Pixel32 *)(m_pBits+m_Stride*y);
         PixelPacket * pSrcLine = pSrcPixels+y*Img.columns();
-        for (int x=0; x<m_Size.x; ++x) {
-            *pDestLine = Pixel32(pSrcLine->red, pSrcLine->green, 
-                    pSrcLine->blue, 255-pSrcLine->opacity);
-            pSrcLine++;
-            pDestLine++;
+        if (m_PF == R8G8B8A8) {
+            for (int x=0; x<m_Size.x; ++x) {
+                *pDestLine = Pixel32(pSrcLine->red, pSrcLine->green, 
+                        pSrcLine->blue, 255-pSrcLine->opacity);
+                pSrcLine++;
+                pDestLine++;
+            }
+        } else {
+            for (int x=0; x<m_Size.x; ++x) {
+                *pDestLine = Pixel32(pSrcLine->red, pSrcLine->green, 
+                        pSrcLine->blue, 255);
+                pSrcLine++;
+                pDestLine++;
+            }
         }
     }
     m_bOwnsBits = true;
@@ -151,6 +175,7 @@ Bitmap::~Bitmap()
     ObjectCounter::get()->decRef(&typeid(*this));
     if (m_bOwnsBits) {
         delete[] m_pBits;
+        m_pBits = 0;
     }
 }
 
@@ -160,6 +185,7 @@ Bitmap &Bitmap::operator= (const Bitmap &Orig)
     if (this != &Orig) {
         if (m_bOwnsBits) {
             delete[] m_pBits;
+            m_pBits = 0;
         }
         m_Size = Orig.getSize();
         m_PF = Orig.getPixelFormat();
@@ -195,12 +221,18 @@ void Bitmap::copyPixels(const Bitmap & Orig)
             case YUYV422:
             case YCbCr411:
             case YCbCr420p:
-                if (m_PF == B8G8R8X8) {
-                    YCbCrtoBGR(Orig);
-                } else {
-                    Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
-                    TempBmp.YCbCrtoBGR(Orig);
-                    copyPixels(TempBmp);
+                switch(m_PF) {
+                    case B8G8R8X8:
+                        YCbCrtoBGR(Orig);
+                        break;
+                    case I8:
+                        YCbCrtoI8(Orig);
+                    default: {
+                            Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
+                            TempBmp.YCbCrtoBGR(Orig);
+                            copyPixels(TempBmp);
+                        }
+                        break;
                 }
                 break;
             case I16:
@@ -230,6 +262,34 @@ void Bitmap::copyPixels(const Bitmap & Orig)
                         assert(false);
                 }
                 break;
+            case BAYER8_GBRG:
+                switch(m_PF) {
+                    // Bayer patterns are saved as I8 bitmaps. So simply copy that
+                    case I8:
+                        {
+                            const unsigned char * pSrc = Orig.getPixels();
+                            unsigned char * pDest = m_pBits;
+                            int Height = min(Orig.getSize().y, m_Size.y);
+                            int LineLen = min(Orig.getLineLen(), getLineLen());
+                            int SrcStride = Orig.getStride();
+                            for (int y=0; y<Height; ++y) {
+                                memcpy(pDest, pSrc, LineLen);
+                                pDest += m_Stride;
+                                pSrc += SrcStride;
+                            }
+                        }
+                        break;
+                    case B8G8R8X8:
+                    case B8G8R8A8:
+                    case R8G8B8X8:
+                    case R8G8B8A8:
+                        BY8toRGBBilinear(Orig);
+                        break;
+                    default: 
+                        // Unimplemented conversion.
+                        assert(false);
+                }
+                break;
             default:
                 switch(m_PF) {
                     case B8G8R8A8:
@@ -269,61 +329,44 @@ void Bitmap::save(const std::string& sFilename)
     string sPF;
     BitmapPtr pBmp;
     Magick::StorageType ChannelFormat = Magick::CharPixel;
+    int AlphaOffset = -1;
     switch(m_PF) {
         case B5G6R5:
-            pBmp = BitmapPtr(new Bitmap(m_Size, B8G8R8));
-            pBmp->copyPixels(*this);
-            sPF = "BGR";
-            break;
         case B8G8R8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "BGR";
-            break;
-        case B8G8R8A8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "BGRA";
-            break;
         case B8G8R8X8:
-            pBmp = BitmapPtr(new Bitmap(m_Size, B8G8R8));
-            pBmp->copyPixels(*this);
-            sPF = "BGR";
-            break;
-        case A8B8G8R8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "ABGR";
-            break;
         case X8B8G8R8:
             pBmp = BitmapPtr(new Bitmap(m_Size, B8G8R8));
             pBmp->copyPixels(*this);
             sPF = "BGR";
             break;
         case R5G6B5:
-            pBmp = BitmapPtr(new Bitmap(m_Size, R8G8B8));
-            pBmp->copyPixels(*this);
-            sPF = "RGB";
-            break;
         case R8G8B8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "RGB";
-            break;
-        case R8G8B8A8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "RGBA";
-            break;
         case R8G8B8X8:
-            pBmp = BitmapPtr(new Bitmap(m_Size, R8G8B8, "temp copy"));
-            pBmp->copyPixels(*this);
-            sPF = "RGB";
-            break;
-        case A8R8G8B8:
-            pBmp = BitmapPtr(new Bitmap(*this));
-            sPF = "ARGB";
-            break;
         case X8R8G8B8:
             pBmp = BitmapPtr(new Bitmap(m_Size, R8G8B8));
             pBmp->copyPixels(*this);
             sPF = "RGB";
             break;
+        case B8G8R8A8:
+            pBmp = BitmapPtr(new Bitmap(*this));
+            AlphaOffset = 3;
+            sPF = "BGRA";
+            break;
+        case A8B8G8R8:
+            pBmp = BitmapPtr(new Bitmap(*this));
+            AlphaOffset = 0;
+            sPF = "ABGR";
+            break;
+        case R8G8B8A8:
+            pBmp = BitmapPtr(new Bitmap(*this));
+            AlphaOffset = 3;
+            sPF = "RGBA";
+            break;
+        case A8R8G8B8:
+            pBmp = BitmapPtr(new Bitmap(*this));
+            AlphaOffset = 0;
+            sPF = "ARGB";
+            break;
         case I16:   
             pBmp = BitmapPtr(new Bitmap(*this));
             ChannelFormat = Magick::ShortPixel;
@@ -338,6 +381,18 @@ void Bitmap::save(const std::string& sFilename)
                     << endl;
             assert(false);
     }
+    if (AlphaOffset != -1) {
+        int Stride = pBmp->getStride();
+        unsigned char * pLine = pBmp->getPixels();
+        for (int y=0; y<m_Size.y; ++y) {
+            unsigned char * pPixel = pLine;
+            for (int x=0; x<m_Size.x; ++x) {
+                *(pPixel+AlphaOffset) = 255-*(pPixel+AlphaOffset);
+                pPixel+=4;
+            }
+            pLine += Stride;
+        }
+    }
     Magick::Image Img(m_Size.x, m_Size.y, sPF, ChannelFormat, pBmp->getPixels());
     Img.write(sFilename);
 }
@@ -408,6 +463,8 @@ std::string Bitmap::getPixelFormatString(PixelFormat PF)
             return "YCbCr420p";
         case YCbCrJ420p:
             return "YCbCrJ420p";
+        case BAYER8_GBRG:
+            return "BAYER8_GBRG";
         default:
             return "Unknown";
     }
@@ -474,7 +531,9 @@ int Bitmap::getBytesPerPixel(PixelFormat PF)
         case I16:
             return 2;
         case I8:
+        case BAYER8_GBRG:
             return 1;
+        case YUYV422:
         case YCbCr422:
             return 2;
         default:
@@ -520,6 +579,27 @@ HistogramPtr Bitmap::getHistogram(int Stride) const
     return pHist;
 }
 
+void Bitmap::getMinMax(int Stride, int& min, int& max) const
+{
+    assert (m_PF == I8);
+    const unsigned char * pSrcLine = m_pBits;
+    min = 255;
+    max = 0;
+    for (int y=0; y < m_Size.y; y+=Stride) {
+        const unsigned char * pSrc = pSrcLine;
+        for (int x=0; x<m_Size.x; x+=Stride) {
+            if (*pSrc < min) {
+                min = *pSrc;
+            }
+            if (*pSrc > max) {
+                max = *pSrc;
+            }
+            pSrc+=Stride;
+        }
+        pSrcLine += m_Stride*Stride;
+    }
+}
+
 bool Bitmap::operator ==(const Bitmap & otherBmp)
 {
     // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
@@ -555,93 +635,137 @@ bool Bitmap::operator ==(const Bitmap & otherBmp)
     return true;
 }
 
-template<class Pixel>
-void lineSubtract(const unsigned char * pSrc, unsigned char * pDest, int lineLen)
+Bitmap * Bitmap::subtract(const Bitmap *pOtherBmp)
 {
-    Pixel * pSrcPixel = (Pixel *)pSrc;
-    Pixel * pDestPixel = (Pixel *)pDest;
-    for (int x=0; x<lineLen; ++x) {
-        pDestPixel->setR(abs(pSrcPixel->getR()-pDestPixel->getR()));
-        pDestPixel->setG(abs(pSrcPixel->getG()-pDestPixel->getG()));
-        pDestPixel->setB(abs(pSrcPixel->getB()-pDestPixel->getB()));
-        pSrcPixel++;
-        pDestPixel++;
+    assert(m_PF == pOtherBmp->getPixelFormat() && m_Size == pOtherBmp->getSize());
+    Bitmap * pResultBmp = new Bitmap(m_Size, m_PF);
+    const unsigned char * pSrcLine1 = pOtherBmp->getPixels();
+    const unsigned char * pSrcLine2 = m_pBits;
+    unsigned char * pDestLine = pResultBmp->getPixels();
+    for (int y=0; y<getSize().y; ++y) {
+        switch(m_PF) {
+            case I16: 
+                {
+                    const unsigned short * pSrc1 = (const unsigned short *)pSrcLine1;
+                    const unsigned short * pSrc2 = (const unsigned short *)pSrcLine2;
+                    unsigned short * pDest= (unsigned short *)pDestLine;
+                    for (int x=0; x<m_Size.x; ++x) {
+                        *pDest = abs(*pSrc1-*pSrc2);
+                        pSrc1++;
+                        pSrc2++;
+                        pDest++;
+                    }
+                }
+                break;
+            default:
+                {
+                    const unsigned char * pSrc1 = pSrcLine1;
+                    const unsigned char * pSrc2 = pSrcLine2;
+                    unsigned char * pDest= pDestLine;
+                    for (int x=0; x<getLineLen(); ++x) {
+                        *pDest = abs(*pSrc1-*pSrc2);
+                        pSrc1++;
+                        pSrc2++;
+                        pDest++;
+                    }
+                }
+        }
+        pSrcLine1 += getStride();
+        pSrcLine2 += pOtherBmp->getStride();
+        pDestLine += pResultBmp->getStride();
     }
+    return pResultBmp;
 }
 
-void Bitmap::subtract(const Bitmap *pOtherBmp)
+double Bitmap::getAvg() const
 {
-    const unsigned char * pSrc = pOtherBmp->getPixels();
-    unsigned char * pDest = m_pBits;
+    double sum = 0;
+    unsigned char * pSrc = m_pBits;
+    int componentsPerPixel = getBytesPerPixel();
     for (int y=0; y<getSize().y; ++y) {
         switch(m_PF) {
             case R8G8B8X8:
             case B8G8R8X8:
-                lineSubtract<Pixel32>(pSrc, pDest, m_Size.x);
+                {
+                    Pixel32 * pSrcPixel = (Pixel32 *)pSrc;
+                    for (int x=0; x<m_Size.x; ++x) {
+                        sum += pSrcPixel->getR()+pSrcPixel->getG()+pSrcPixel->getB();
+                        pSrcPixel++;
+                    }
+                    componentsPerPixel = 3;
+                }
                 break;
-            case R8G8B8:
-            case B8G8R8:
-                lineSubtract<Pixel24>(pSrc, pDest, m_Size.x);
+            case I16:
+                {
+                    componentsPerPixel = 1;
+                    unsigned short * pSrcPixel = (unsigned short *)pSrc;
+                    for (int x=0; x<m_Size.x; ++x) {
+                        sum += *pSrcPixel;
+                        pSrcPixel++;
+                    }
+                }
                 break;
             default:
-                // Unimplemented.
-                assert(false);
+                {
+                    unsigned char * pSrcComponent = pSrc;
+                    for (int x=0; x<getLineLen(); ++x) {
+                        sum += *pSrcComponent;
+                        pSrcComponent++;
+                    }
+                }
         }
-        pDest += m_Stride;
-        pSrc += pOtherBmp->getStride();
+        pSrc += m_Stride;
     }
+    sum /= componentsPerPixel;
+    return sum/(getSize().x*getSize().y);
 }
 
-template<class Pixel>
-int lineBrightPixels(const unsigned char * pSrc, int lineLen)
+double Bitmap::getStdDev() const
 {
-    Pixel * pSrcPixel = (Pixel *)pSrc;
-    int Result = 0;
-    for (int x=0; x<lineLen; ++x) {
-        int Val = pSrcPixel->getR()+pSrcPixel->getG()+pSrcPixel->getB();
-        if (Val > 256) {
-            Result++;
-        }
-        pSrcPixel++;
-    }
-    return Result;
-}
+    double average = getAvg();
+    double sum = 0;
 
-int Bitmap::getNumDifferentPixels(const Bitmap & otherBmp)
-{
-    // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
-    // equal value only.
-    if (m_Size != otherBmp.m_Size || m_PF != otherBmp.m_PF) {
-        return m_Size.x*m_Size.y;
-    }
-
-    BitmapPtr pTempBmp(new Bitmap(*this));
-//    pTempBmp->dump(true);
-    pTempBmp->subtract(&otherBmp);
-//    pTempBmp->dump(true);
-    double Matrix[3][3] = {
-            {0.111,0.111,0.111},
-            {0.111,0.111,0.111},
-            {0.111,0.111,0.111}
-    };
-    Filter3x3(Matrix).applyInPlace(pTempBmp);
-
-    int NumBrightPixels = 0;
-    for (int y = 0; y < m_Size.y-2; y++) {
-        const unsigned char * pLine = pTempBmp->getPixels()+y*pTempBmp->getStride();
-        
-        switch (pTempBmp->getBytesPerPixel()) {
-            case 4:
-                NumBrightPixels += lineBrightPixels<Pixel32>(pLine, m_Size.x-2);
+    unsigned char * pSrc = m_pBits;
+    int componentsPerPixel = getBytesPerPixel();
+    for (int y=0; y<getSize().y; ++y) {
+        switch(m_PF) {
+            case R8G8B8X8:
+            case B8G8R8X8:
+                {
+                    componentsPerPixel = 3;
+                    Pixel32 * pSrcPixel = (Pixel32 *)pSrc;
+                    for (int x=0; x<m_Size.x; ++x) {
+                        sum += sqr(pSrcPixel->getR()-average);
+                        sum += sqr(pSrcPixel->getG()-average);
+                        sum += sqr(pSrcPixel->getB()-average);
+                        pSrcPixel++;
+                    }
+                }
                 break;
-            case 3:
-                NumBrightPixels += lineBrightPixels<Pixel24>(pLine, m_Size.x-2);
+            case I16:
+                {
+                    componentsPerPixel = 1;
+                    unsigned short * pSrcPixel = (unsigned short *)pSrc;
+                    for (int x=0; x<m_Size.x; ++x) {
+                        sum += sqr(*pSrcPixel-average);
+                        pSrcPixel++;
+                    }
+                }
                 break;
             default:
-                assert(false);
+                {
+                    unsigned char * pSrcComponent = pSrc;
+                    for (int x=0; x<getLineLen(); ++x) {
+                        sum += sqr(*pSrcComponent-average);
+                        pSrcComponent++;
+                    }
+                }
         }
+        pSrc += m_Stride;
     }
-    return NumBrightPixels;
+    sum /= componentsPerPixel;
+    sum /= (getSize().x*getSize().y);
+    return sqrt(sum);
 }
 
 void Bitmap::dump(bool bDumpPixels) const
@@ -689,8 +813,12 @@ void Bitmap::initWithData(unsigned char * pBits, int Stride, bool bCopyBits)
     }
     if (bCopyBits) {
         allocBits();
-        for (int y=0; y<m_Size.y; ++y) {
-            memcpy(m_pBits+m_Stride*y, pBits+Stride*y, Stride);
+        if (m_Stride == Stride && Stride == (m_Size.x*getBytesPerPixel())) {
+            memcpy(m_pBits, pBits, Stride*m_Size.y);
+        } else {
+            for (int y=0; y<m_Size.y; ++y) {
+                memcpy(m_pBits+m_Stride*y, pBits+Stride*y, Stride);
+            }
         }
         m_bOwnsBits = true;
     } else {
@@ -702,6 +830,7 @@ void Bitmap::initWithData(unsigned char * pBits, int Stride, bool bCopyBits)
 
 void Bitmap::allocBits()
 {
+    assert(!m_pBits);
 //    cerr << "Bitmap::allocBits():" << m_Size <<  endl;
     m_Stride = getLineLen();
     if (m_PF == YCbCr422 || m_PF == YCbCr420p) {
@@ -900,6 +1029,75 @@ void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
     }
 }
     
+void YUYV422toI8Line(const unsigned char* pSrcLine, unsigned char * pDestLine, int Width)
+{
+    const unsigned char * pSrc = pSrcLine;
+    unsigned char * pDest = pDestLine;
+    
+    for (int x = 0; x < Width; x++) {
+        *pDest = *pSrc;
+        pDest++;
+        pSrc+=2;
+    }
+}
+ 
+void YUV411toI8Line(const unsigned char* pSrcLine, unsigned char * pDestLine, int Width)
+{
+    const unsigned char * pSrc = pSrcLine;
+    unsigned char * pDest = pDestLine;
+    
+    for (int x = 0; x < Width/2; x++) {
+        *pDest++ = *pSrc++;
+        *pDest++ = *pSrc++;
+        pSrc++;
+    }
+}
+ 
+void Bitmap::YCbCrtoI8(const Bitmap& Orig)
+{
+    assert(m_PF==I8);
+    const unsigned char * pSrc = Orig.getPixels();
+    unsigned char * pDest = m_pBits;
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    switch(Orig.m_PF) {
+        case YCbCr422:
+            for (int y=0; y<Height; ++y) {
+                // src shifted by one byte to account for UYVY to YUYV 
+                // difference in pixel order.
+                YUYV422toI8Line(pSrc+1, pDest, Width);
+                pDest += m_Stride;
+                pSrc += Orig.getStride();
+            }
+            break;
+        case YUYV422:
+            for (int y=0; y<Height; ++y) {
+                YUYV422toI8Line(pSrc, pDest, Width);
+                pDest += m_Stride;
+                pSrc += Orig.getStride();
+            }
+            break;
+        case YCbCr411:
+            for (int y=0; y<Height; ++y) {
+                YUV411toI8Line(pSrc, pDest, Width);
+                pDest += m_Stride;
+                pSrc += Orig.getStride();
+            }
+            break;
+        case YCbCr420p: 
+            // Just take the first plane.
+            for (int y=0; y<Height; ++y) {
+                memcpy(pDest, pSrc, m_Stride);
+                pDest += m_Stride;
+                pSrc += Orig.getStride();
+            }
+            break;
+        default:
+            // This routine shouldn't be called with other pixel formats.
+            assert(false);
+    }
+}
+
 void Bitmap::I16toI8(const Bitmap& Orig)
 {
     assert(m_PF == I8);
@@ -979,6 +1177,220 @@ void Bitmap::I8toRGB(const Bitmap& Orig)
     }
 }
 
+
+// Nearest Neighbour Bayer Pattern de-mosaicking
+// Code has been taken and adapted from libdc1394 Bayer conversion
+// TODO: adapt it for RGB24, not just for RGB32
+// TODO: add more CFA patterns (now only the GBRG is defined and used)
+void Bitmap::BY8toRGBNearest(const Bitmap& Orig)
+{
+    assert(getBytesPerPixel() == 4);
+    assert(Orig.getPixelFormat() == BAYER8_GBRG);
+
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+
+    const int SrcStride = Width;
+    const int DestStride = 4 * Width;
+    int width = Width;
+    int height = Height;
+
+    // CFA Pattern selection: BGGR: blue=-1, swg=0; GRBG: blue=1, swg=1
+    // Assuming GBRG
+    int blue = 1;
+    int greenFirst = 1;
+
+    const unsigned char *pSrcPixel = Orig.getPixels();
+    unsigned char *pDestPixel = (unsigned char *) getPixels();
+
+    pDestPixel += 1;
+    width -= 1;
+    height -= 1;
+
+    while (--height) {
+
+        const unsigned char *pSrcEndBoundary = pSrcPixel + width;
+
+        if (greenFirst) {
+            pDestPixel[-blue] = pSrcPixel[1];
+            pDestPixel[0] = pSrcPixel[SrcStride + 1];
+            pDestPixel[blue] = pSrcPixel[SrcStride];
+            pDestPixel[2] = 255; // Alpha channel
+            ++pSrcPixel;
+            pDestPixel += 4;
+        }
+
+        if (blue > 0) {
+            while (pSrcPixel <= pSrcEndBoundary - 2) {
+                pDestPixel[-1] = pSrcPixel[0];
+                pDestPixel[0] = pSrcPixel[1];
+                pDestPixel[1] = pSrcPixel[SrcStride + 1];
+                pDestPixel[2] = 255; // Alpha channel
+
+                pDestPixel[3] = pSrcPixel[2];
+                pDestPixel[4] = pSrcPixel[SrcStride + 2];
+                pDestPixel[5] = pSrcPixel[SrcStride + 1];
+                pDestPixel[6] = 255; // Alpha channel
+                
+                pSrcPixel += 2;
+                pDestPixel += 8;
+            }
+        } else {
+            while (pSrcPixel <= pSrcEndBoundary - 2) {
+                pDestPixel[1] = pSrcPixel[0];
+                pDestPixel[0] = pSrcPixel[1];
+                pDestPixel[-1] = pSrcPixel[SrcStride + 1];
+                
+                pDestPixel[6] = 255; // Alpha channel
+                pDestPixel[5] = pSrcPixel[2];
+                pDestPixel[4] = pSrcPixel[SrcStride + 2];
+                pDestPixel[3] = pSrcPixel[SrcStride + 1];
+                pDestPixel[2] = 255; // Alpha channel
+
+                pSrcPixel += 2;
+                pDestPixel += 8;
+            }
+        }
+
+        if (pSrcPixel < pSrcEndBoundary) {
+            pDestPixel[-blue] = pSrcPixel[0];
+            pDestPixel[0] = pSrcPixel[1];
+            pDestPixel[blue] = pSrcPixel[SrcStride + 1];
+            pDestPixel[2] = 255; // Alpha channel
+            ++pSrcPixel;
+            pDestPixel += 4;
+        }
+
+        pSrcPixel -= width;
+        pDestPixel -= width * 4;
+
+        blue = -blue;
+        greenFirst = !greenFirst;
+
+        pSrcPixel += SrcStride;
+        pDestPixel += DestStride;
+    }
+}
+
+// Bilinear Bayer Pattern de-mosaicking
+// Code has been taken and adapted from libdc1394 Bayer conversion
+// Original source is OpenCV Bayer pattern decoding
+// TODO: adapt it for RGB24, not just for RGB32
+// TODO: add more CFA patterns (now only the GBRG is defined and used)
+void Bitmap::BY8toRGBBilinear(const Bitmap& Orig)
+{
+    assert(getBytesPerPixel() == 4);
+    assert(Orig.getPixelFormat() == BAYER8_GBRG);
+
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+
+    const int SrcStride = Width;
+    const int doubleSrcStride = SrcStride * 2;
+    const int DestStride = 4 * Width;
+    int width = Width;
+    int height = Height;
+
+    // CFA Pattern selection: BGGR: blue=-1, greenFirst=0; GRBG: blue=1, greenFirst=1
+    // Assuming GBRG
+    int blue = 1;
+    int greenFirst = 1;
+
+    const unsigned char *pSrcPixel = Orig.getPixels();
+    unsigned char *pDestPixel = (unsigned char *) getPixels();
+
+    pDestPixel += DestStride + 4 + 1;
+    height -= 2;
+    width -= 2;
+
+    while(height--) {
+        int t0, t1;
+        const unsigned char *pSrcEndBoundary = pSrcPixel + width;
+
+        if (greenFirst) {
+            t0 = (pSrcPixel[1] + pSrcPixel[doubleSrcStride + 1] + 1) >> 1;
+            t1 = (pSrcPixel[SrcStride] + pSrcPixel[SrcStride + 2] + 1) >> 1;
+            pDestPixel[-blue] = (unsigned char) t0;
+            pDestPixel[0] = pSrcPixel[SrcStride + 1];
+            pDestPixel[blue] = (unsigned char) t1;
+            pDestPixel[2] = 255; // Alpha channel
+            ++pSrcPixel;
+            pDestPixel += 4;
+        }
+                
+        if (blue > 0) {
+            while(pSrcPixel <= pSrcEndBoundary - 2) {
+                t0 = (pSrcPixel[0] + pSrcPixel[2] + pSrcPixel[doubleSrcStride] +
+                      pSrcPixel[doubleSrcStride + 2] + 2) >> 2;
+                t1 = (pSrcPixel[1] + pSrcPixel[SrcStride] +
+                      pSrcPixel[SrcStride + 2] + pSrcPixel[doubleSrcStride + 1] +
+                      2) >> 2;
+                pDestPixel[-1] = (unsigned char) t0;
+                pDestPixel[0] = (unsigned char) t1;
+                pDestPixel[1] = pSrcPixel[SrcStride + 1];
+                pDestPixel[2] = 255; // Alpha channel
+
+                t0 = (pSrcPixel[2] + pSrcPixel[doubleSrcStride + 2] + 1) >> 1;
+                t1 = (pSrcPixel[SrcStride + 1] + pSrcPixel[SrcStride + 3] +
+                      1) >> 1;
+                pDestPixel[3] = (unsigned char) t0;
+                pDestPixel[4] = pSrcPixel[SrcStride + 2];
+                pDestPixel[5] = (unsigned char) t1;
+                pDestPixel[6] = 255; // Alpha channel
+                
+                pSrcPixel += 2;
+                pDestPixel += 8;
+            }
+        } else {
+            while(pSrcPixel <= pSrcEndBoundary - 2) {
+                t0 = (pSrcPixel[0] + pSrcPixel[2] + pSrcPixel[doubleSrcStride] +
+                      pSrcPixel[doubleSrcStride + 2] + 2) >> 2;
+                t1 = (pSrcPixel[1] + pSrcPixel[SrcStride] +
+                      pSrcPixel[SrcStride + 2] + pSrcPixel[doubleSrcStride + 1] +
+                      2) >> 2;
+                pDestPixel[1] = (unsigned char) t0;
+                pDestPixel[0] = (unsigned char) t1;
+                pDestPixel[-1] = pSrcPixel[SrcStride + 1];
+                pDestPixel[2] = 255; // Alpha channel
+
+                t0 = (pSrcPixel[2] + pSrcPixel[doubleSrcStride + 2] + 1) >> 1;
+                t1 = (pSrcPixel[SrcStride + 1] + pSrcPixel[SrcStride + 3] +
+                      1) >> 1;
+                pDestPixel[5] = (unsigned char) t0;
+                pDestPixel[4] = pSrcPixel[SrcStride + 2];
+                pDestPixel[3] = (unsigned char) t1;
+                pDestPixel[6] = 255; // Alpha channel
+                
+                pSrcPixel += 2;
+                pDestPixel += 8;
+            }
+        }
+
+        if (pSrcPixel < pSrcEndBoundary) {
+            t0 = (pSrcPixel[0] + pSrcPixel[2] + pSrcPixel[doubleSrcStride] +
+                  pSrcPixel[doubleSrcStride + 2] + 2) >> 2;
+            t1 = (pSrcPixel[1] + pSrcPixel[SrcStride] +
+                  pSrcPixel[SrcStride + 2] + pSrcPixel[doubleSrcStride + 1] +
+                  2) >> 2;
+            pDestPixel[-blue] = (unsigned char) t0;
+            pDestPixel[0] = (unsigned char) t1;
+            pDestPixel[blue] = pSrcPixel[SrcStride + 1];
+            pDestPixel[2] = 255; // Alpha channel
+            pSrcPixel++;
+            pDestPixel += 4;
+        }
+
+        pSrcPixel -= width;
+        pDestPixel -= width * 4;
+
+        blue = -blue;
+        greenFirst = !greenFirst;
+        
+        pSrcPixel += SrcStride;
+        pDestPixel += DestStride;
+    }
+}
+
 template<class DestPixel, class SrcPixel>
 void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
 {
@@ -1047,6 +1459,7 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
             createTrueColorCopy<Pixel, Pixel16>(Dest, Src);
             break;
         case I8:
+        case BAYER8_GBRG:
             createTrueColorCopy<Pixel, Pixel8>(Dest, Src);
             break;
         default:
