@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,6 +30,7 @@
 #include <string>
 #include <vector>
 #include <algorithm>
+#include <iostream>
 
 namespace avg {
 
@@ -54,8 +55,10 @@ typedef enum {
                // for Y, Cb and Cr components, with Cb and Cr half as big in 
                // both x and y dimensions. This is mpeg YCbCr, where the 
                // color components have values from 16...235.
-    YCbCrJ420p // Same as YCbCr420p, but this is the jpeg version with component
+    YCbCrJ420p,// Same as YCbCr420p, but this is the jpeg version with component
                // values in the range 0...255
+    BAYER8_GBRG,
+    NO_PIXELFORMAT
 } PixelFormat;
 
 typedef std::vector<int> Histogram;
@@ -98,25 +101,30 @@ public:
     int getMemNeeded() const;
     bool hasAlpha() const;
     HistogramPtr getHistogram(int Stride = 1) const;
+    void getMinMax(int Stride, int& min, int& max) const;
 
     template<class Pixel>
     void setPixel(const IntPoint& p, Pixel Color);
     template<class Pixel>
     void drawLine(IntPoint p0, IntPoint p1, Pixel Color);
 
-    void subtract(const Bitmap* pOtherBmp);
+    Bitmap * subtract(const Bitmap* pOtherBmp);
+    double getAvg() const;
+    double getStdDev() const;
 
     bool operator ==(const Bitmap & otherBmp);
-    int getNumDifferentPixels(const Bitmap & otherBmp);
     void dump(bool bDumpPixels=false) const;
 
 private:
     void initWithData(unsigned char * pBits, int Stride, bool bCopyBits);
     void allocBits();
     void YCbCrtoBGR(const Bitmap& Orig);
+    void YCbCrtoI8(const Bitmap& Orig);
     void I8toI16(const Bitmap& Orig);
     void I8toRGB(const Bitmap& Orig);
     void I16toI8(const Bitmap& Orig);
+    void BY8toRGBNearest(const Bitmap& Orig);
+    void BY8toRGBBilinear(const Bitmap& Orig);
 
     IntPoint m_Size;
     int m_Stride;
@@ -134,10 +142,14 @@ void Bitmap::setPixel(const IntPoint& p, Pixel Color)
     *(Pixel*)(&(m_pBits[p.y*m_Stride+p.x*getBytesPerPixel()])) = Color;
 }
 
-// TODO: This is slow, and it doesn't clip. Replace with external lib?
+// TODO: This is slow, and it clips incorrectly. Replace with external lib?
 template<class Pixel>
 void Bitmap::drawLine(IntPoint p0, IntPoint p1, Pixel Color)
 {
+    IntRect BmpRect(IntPoint(0,0), m_Size);
+    p0 = BmpRect.cropPoint(p0);
+    p1 = BmpRect.cropPoint(p1);
+
     bool bSteep = abs(p1.y - p0.y) > abs(p1.x - p0.x);
     if (bSteep) {
         std::swap(p0.x, p0.y);
