@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,8 @@
 
 #include "Bitmap.h"
 
+#include <boost/shared_ptr.hpp>
+
 namespace avg {
 
 // Base class for filters that operate on bitmaps. Derived classes need
@@ -38,14 +40,16 @@ public:
 
   // In-Place Apply. Applies the filter to pBmp. The base-class
   // version copies the bitmap after calling Apply (pBmp, pTempBmp).
-  virtual void applyInPlace(BitmapPtr pBmp) ;  
+  virtual void applyInPlace(BitmapPtr pBmp);
 
   // Applies the Filter to pBmpSource and returns the result
   // The base-class version copies the bitmap before calling
   // applyInPlace.
-  virtual BitmapPtr apply(BitmapPtr pBmpSource) ;
+  virtual BitmapPtr apply(BitmapPtr pBmpSource);
 };
 
+typedef boost::shared_ptr<Filter> FilterPtr;
+
 } // namespace
 #endif
 
