@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -116,7 +116,8 @@ inline int Pixel8::boxDist (const Pixel8 Pix) const
 
 inline Pixel8 Pixel8::Blend (int Factor, const Pixel8 Pix1, const Pixel8 Pix2)
 {
-  assert(Factor >= 0 && Factor <= 256);
+// The following causes a 'C1055: compiler limit : out of keys' error on MSVC 2003
+//  assert(Factor >= 0 && Factor <= 256);
 
   return Pixel8 ((Pix1.get()*Factor+Pix2.get()*(256-Factor))>>8);
 }
