@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
 //  Current versions can be found at www.libavg.de
 //
 
-
+#include "GraphicsTest.h"
 #include "Bitmap.h"
 #include "Pixel32.h"
 #include "Pixel24.h"
@@ -38,11 +38,22 @@
 #include "FilterGauss.h"
 #include "FilterBlur.h"
 #include "FilterBandpass.h"
+#include "FilterFastDownscale.h"
+#include "FilterMask.h"
+
 #include "../base/TestSuite.h"
 #include "../base/Exception.h"
 
+#ifdef _WIN32
+#pragma warning(push)
+#pragma warning(disable: 4251)
+#endif
 #include <Magick++.h>
+#ifdef _WIN32
+#pragma warning(pop)
+#endif
 
+#include <cstring>
 #include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
@@ -77,10 +88,10 @@ BitmapPtr initBmp(PixelFormat PF)
     return pBmp;
 }
 
-class BitmapTest: public Test {
+class BitmapTest: public GraphicsTest {
 public:
     BitmapTest()
-        : Test("BitmapTest", 2)
+        : GraphicsTest("BitmapTest", 2)
     {
     }
 
@@ -96,12 +107,12 @@ public:
         runLineTest(R8G8B8A8, Pixel32(255,0,0,255));
         runLineTest(R8G8B8, Pixel24(255,0,0));
         runLineTest(I8, Pixel8(255));
-        
+
         cerr << "    Testing OwnsBits." << endl;
         unsigned char pData[4*7*3];
         Bitmap Bmp1 = Bitmap(IntPoint(4,7), R8G8B8, pData, 12, true, "");
         Bitmap Bmp2 = Bitmap(IntPoint(4,7), R8G8B8, pData, 12, false, "");
-        testEqual(Bmp1, Bmp2);
+        testEqual(Bmp1, Bmp2, "BmpOwnsBits");
         {
             cerr << "    Testing copyPixels - R8G8B8X8->R8G8B8->R8G8B8X8." << endl;
             BitmapPtr pBmp = initBmp(R8G8B8X8);
@@ -115,7 +126,7 @@ public:
                             *Bmp2.getBytesPerPixel()+3) = 0x80;
                 }
             }
-            testEqual(*pBmp, Bmp2);
+            testEqual(Bmp2, *pBmp, "BmpCopyPixels_1");
         }
         {
             cerr << "    Testing copyPixels - R8G8B8A8->R8G8B8->R8G8B8A8." << endl;
@@ -130,7 +141,7 @@ public:
                             *pBmp->getBytesPerPixel()+3) = 0x80;
                 }
             }
-            testEqual(*pBmp, *pBaselineBmp);
+            testEqual(*pBmp, *pBaselineBmp, "BmpCopyPixels_2");
         }
         {
             cerr << "    Testing copyPixels - I8->I16->I8." << endl;
@@ -139,7 +150,21 @@ public:
             BitmapPtr pCopyBmp = BitmapPtr(new Bitmap(IntPoint(4,7), I16));
             pCopyBmp->copyPixels(*pBmp);
             pBmp->copyPixels(*pCopyBmp);
-            testEqual(*pBmp, *pBaselineBmp);
+            testEqual(*pBmp, *pBaselineBmp, "BmpCopyPixels_3");
+        }
+        {
+            cerr << "    Testing statistics." << endl;
+            cerr << "      I8" << endl;
+            testStatistics(I8, Pixel8(0), Pixel8(0), Pixel8(2), Pixel8(2));
+            cerr << "      R8G8B8A8" << endl;
+            testStatistics(R8G8B8A8, Pixel32(0,0,0,0), Pixel32(0,0,0,0), 
+                    Pixel32(2,2,2,2), Pixel32(2,2,2,2));
+            cerr << "      R8G8B8X8" << endl;
+            testStatistics(R8G8B8X8, Pixel32(0,0,0,255), Pixel32(0,0,0,255), 
+                    Pixel32(2,2,2,255), Pixel32(2,2,2,255));
+            cerr << "      R8G8B8" << endl;
+            testStatistics(R8G8B8, Pixel24(0,0,0), Pixel24(0,0,0), 
+                    Pixel24(2,2,2), Pixel24(2,2,2));
 
         }
         runSaveTest(R8G8B8A8);
@@ -147,6 +172,19 @@ public:
     }
     
 private:
+    template<class Pixel>
+    void testStatistics(PixelFormat pf, const Pixel& p00, const Pixel& p01,
+            const Pixel& p10, const Pixel& p11)
+    {
+        BitmapPtr pBmp = BitmapPtr(new Bitmap(IntPoint(2,2), pf));
+        pBmp->setPixel(IntPoint(0,0), p00);
+        pBmp->setPixel(IntPoint(0,1), p01);
+        pBmp->setPixel(IntPoint(1,0), p10);
+        pBmp->setPixel(IntPoint(1,1), p11);
+        TEST(pBmp->getAvg() == 1);
+        TEST(pBmp->getStdDev() == 1);
+    }
+
     void runPFTests(PixelFormat PF) {
         cerr << "    Testing " << Bitmap::getPixelFormatString(PF) << endl;
         BitmapPtr pBmp = initBmp(PF);
@@ -161,29 +199,17 @@ private:
         {
             cerr << "      Testing copy constructor." << endl;
             Bitmap BmpCopy1(*pBmp);
-            testEqual(*pBmp, BmpCopy1);
+            testEqual(BmpCopy1, *pBmp, "BmpCopyConstructor");
         }
         {
             cerr << "      Testing assignment operator." << endl;
             Bitmap BmpCopy2 = *pBmp;
-            testEqual(*pBmp, BmpCopy2);
+            testEqual(BmpCopy2, *pBmp, "BmpAssignment");
         }
         {
             cerr << "      Testing sub-bitmap constructor." << endl;
             Bitmap BmpCopy3 (*pBmp, IntRect(IntPoint(0,0), pBmp->getSize()));
-            testEqual(*pBmp, BmpCopy3);
-        }
-        if (PF == R8G8B8X8 || PF == R8G8B8) {
-            cerr << "      Testing getNumDifferentPixels." << endl;
-            Bitmap BmpCopy4 (*pBmp);
-            TEST(pBmp->getNumDifferentPixels(BmpCopy4) == 0);
-            
-            unsigned char * pPixel = BmpCopy4.getPixels();
-            *pPixel += 27;
-            TEST(pBmp->getNumDifferentPixels(BmpCopy4) == 0);
-            *pPixel = 255;
-            *(pPixel+BmpCopy4.getStride()) = 255;
-            TEST(!pBmp->getNumDifferentPixels(BmpCopy4) == 1);
+            testEqual(BmpCopy3, *pBmp, "BmpSubBmpCtor");
         }
         if (PF == I8) {
             cerr << "      Testing getHistogram." << endl;
@@ -208,7 +234,7 @@ private:
         pBmp->save("test.tif");
         Bitmap LoadedBmp("test.tif");
         ::remove("test.tif");
-        testEqual(*pBmp, LoadedBmp);
+        testEqual(LoadedBmp, *pBmp, "BmpSave");
     }
 
     template<class Pixel>
@@ -224,31 +250,19 @@ private:
         Bmp.drawLine(IntPoint(7,7), IntPoint(12,14), Color);
         Bmp.drawLine(IntPoint(7,7), IntPoint(14, 2), Color);
         Bmp.drawLine(IntPoint(7,7), IntPoint(14,12), Color);
-        string sFName = getSrcDir()+"testimages/LineResult"+Bitmap::getPixelFormatString(PF)+".png";
+        string sFName = getSrcDirName()+"baseline/LineResult"+Bitmap::getPixelFormatString(PF)+".png";
 //        Bmp.save(sFName);
         Bitmap BaselineBmp(sFName);
         Bitmap BaselineBmp2(IntPoint(15,15), PF);
         BaselineBmp2.copyPixels(BaselineBmp);
-        testEqual(Bmp, BaselineBmp2);
+        testEqual(Bmp, BaselineBmp2, "BmpLineDraw");
     }
-
-    void testEqual(Bitmap& Bmp1, Bitmap& Bmp2) 
-    {
-        TEST(Bmp1 == Bmp2);
-        if (!(Bmp1 == Bmp2)) {
-            cerr << "Bmp1: " << endl;
-            Bmp1.dump();
-            cerr << "Bmp2: " << endl;
-            Bmp2.dump();
-        }
-    }
-
 };
 
-class FilterColorizeTest: public Test {
+class FilterColorizeTest: public GraphicsTest {
 public:
     FilterColorizeTest()
-        : Test("FilterColorizeTest", 2)
+        : GraphicsTest("FilterColorizeTest", 2)
     {
     }
 
@@ -265,14 +279,13 @@ private:
         FilterColorize(100, 50).applyInPlace(pBmp);
         FilterColorize(50, 100).applyInPlace(pBmp);
 //        pBmp->dump();
-        
     }
 };
 
-class FilterGrayscaleTest: public Test {
+class FilterGrayscaleTest: public GraphicsTest {
 public:
     FilterGrayscaleTest()
-        : Test("FilterGrayscaleTest", 2)
+        : GraphicsTest("FilterGrayscaleTest", 2)
     {
     }
 
@@ -283,7 +296,7 @@ public:
         runPFTests(I8);
     }
 
-private:    
+private:
     void runPFTests(PixelFormat PF) {
         BitmapPtr pBmp = initBmp(PF);
         FilterGrayscale().applyInPlace(pBmp);
@@ -291,10 +304,10 @@ private:
     }
 };
 
-class FilterFillTest: public Test {
+class FilterFillTest: public GraphicsTest {
 public:
     FilterFillTest()
-        : Test("FilterFillTest", 2)
+        : GraphicsTest("FilterFillTest", 2)
     {
     }
 
@@ -313,10 +326,10 @@ private:
     }
 };
 
-class FilterFlipTest: public Test {
+class FilterFlipTest: public GraphicsTest {
 public:
     FilterFlipTest()
-        : Test("FilterFlipTest", 2)
+        : GraphicsTest("FilterFlipTest", 2)
     {
     }
 
@@ -347,10 +360,10 @@ private:
     }
 };
 
-class FilterFlipRGBTest: public Test {
+class FilterFlipRGBTest: public GraphicsTest {
 public:
     FilterFlipRGBTest()
-        : Test("FilterFlipRGBTest", 2)
+        : GraphicsTest("FilterFlipRGBTest", 2)
     {
     }
 
@@ -379,10 +392,10 @@ private:
     }
 };
 
-class FilterFlipUVTest: public Test {
+class FilterFlipUVTest: public GraphicsTest {
 public:
     FilterFlipUVTest()
-        : Test("FilterFlipUVTest", 2)
+        : GraphicsTest("FilterFlipUVTest", 2)
     {
     }
 
@@ -404,10 +417,10 @@ public:
     }
 };
 
-class FilterComboTest: public Test {
+class FilterComboTest: public GraphicsTest {
 public:
     FilterComboTest()
-        : Test("FilterComboTest", 2)
+        : GraphicsTest("FilterComboTest", 2)
     {
     }
 
@@ -442,10 +455,10 @@ private:
     }
 };
 
-class FilterConvolTest: public Test {
+class FilterConvolTest: public GraphicsTest {
 public:
     FilterConvolTest()
-        : Test("FilterConvolTest", 2)
+        : GraphicsTest("FilterConvolTest", 2)
     {
     }
 
@@ -491,10 +504,10 @@ private:
     }
 };
 
-class Filter3x3Test: public Test {
+class Filter3x3Test: public GraphicsTest {
 public:
     Filter3x3Test()
-        : Test("Filter3x3Test", 2)
+        : GraphicsTest("Filter3x3Test", 2)
     {
     }
 
@@ -539,22 +552,12 @@ private:
     }
 };
     
-class HistoryPreProcessorTest: public Test {
+class HistoryPreProcessorTest: public GraphicsTest {
 public:
     HistoryPreProcessorTest()
-        : Test("HistoryPreProcessor", 2)
+        : GraphicsTest("HistoryPreProcessor", 2)
     {
     }
-    void testEqual(Bitmap& Bmp1, Bitmap& Bmp2) 
-    {
-        TEST(Bmp1 == Bmp2);
-        if (!(Bmp1 == Bmp2)) {
-            cerr << "Bmp1: " << endl;
-            Bmp1.dump(true);
-            cerr << "Bmp2: " << endl;
-            Bmp2.dump(true);
-        }
-    }
 
     void runTests() 
     {
@@ -562,21 +565,21 @@ public:
         BitmapPtr pBmp = BitmapPtr(new Bitmap(*pBaseBmp));
         BitmapPtr nullBmp = FilterFill<Pixel8>(0).apply(pBmp);
         pBmp->copyPixels(*pBaseBmp);
-        HistoryPreProcessor filt=HistoryPreProcessor(pBaseBmp->getSize());
+        HistoryPreProcessor filt(pBaseBmp->getSize(), 1, true);
         pBmp = filt.apply(pBaseBmp);
-        testEqual(*pBmp, *nullBmp);
+        testEqual(*pBmp, *nullBmp, "HistoryPreprocessor1");
         for(int i=0;i<1;i++){
             pBmp = filt.apply(pBaseBmp);
-            testEqual(*pBmp, *nullBmp);
+            testEqual(*pBmp, *nullBmp, "HistoryPreprocessor2");
         }
     }
 
 };
     
-class FilterFastBandpassTest: public Test {
+class FilterFastBandpassTest: public GraphicsTest {
 public:
     FilterFastBandpassTest()
-        : Test("FilterFastBandpassTest", 2)
+        : GraphicsTest("FilterFastBandpassTest", 2)
     {
     }
 
@@ -586,19 +589,15 @@ public:
         FilterFill<Pixel8>(0).applyInPlace(pBmp);
         *(pBmp->getPixels()+pBmp->getStride()*7+7) = 255;
         BitmapPtr pDestBmp = FilterFastBandpass().apply(pBmp);
-//        pDestBmp->save("testimages/FastBandpassResult.png");
-        string sFName = getSrcDir()+"testimages/FastBandpassResult.png";
-        BitmapPtr pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "FastBandpassResult", I8);
     }
 };
 
 
-class FilterHighpassTest: public Test {
+class FilterHighpassTest: public GraphicsTest {
 public:
     FilterHighpassTest()
-        : Test("FilterHighpassTest", 2)
+        : GraphicsTest("FilterHighpassTest", 2)
     {
     }
 
@@ -608,19 +607,15 @@ public:
         FilterFill<Pixel8>(0).applyInPlace(pBmp);
         *(pBmp->getPixels()+pBmp->getStride()*7+7) = 255;
         BitmapPtr pDestBmp = FilterHighpass().apply(pBmp);
-//        pDestBmp->save("testimages/HighpassResult.png");
-        string sFName = getSrcDir()+"testimages/HighpassResult.png";
-        BitmapPtr pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "HighpassResult", I8);
     }
 };
 
 
-class FilterGaussTest: public Test {
+class FilterGaussTest: public GraphicsTest {
 public:
     FilterGaussTest()
-        : Test("FilterGaussTest", 2)
+        : GraphicsTest("FilterGaussTest", 2)
     {
     }
 
@@ -636,37 +631,21 @@ public:
 //        FilterGauss(4).dumpKernel();
 //        FilterGauss(5).dumpKernel();
         BitmapPtr pDestBmp = FilterGauss(3).apply(pBmp);
-//        pDestBmp->save("testimages/Gauss3Result.png");
-        string sFName = getSrcDir()+"testimages/Gauss3Result.png";
-        BitmapPtr pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "Gauss3Result", I8);
         pDestBmp = FilterGauss(1).apply(pBmp);
-//        pDestBmp->save("testimages/Gauss1Result.png");
-        sFName = getSrcDir()+"testimages/Gauss1Result.png";
-        pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "Gauss1Result", I8);
         pDestBmp = FilterGauss(1.5).apply(pBmp);
-//        pDestBmp->save("testimages/Gauss15Result.png");
-        sFName = getSrcDir()+"testimages/Gauss15Result.png";
-        pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "Gauss15Result", I8);
         pDestBmp = FilterGauss(5).apply(pBmp);
-//        pDestBmp->save("testimages/Gauss5Result.png");
-        sFName = getSrcDir()+"testimages/Gauss5Result.png";
-        pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "Gauss5Result", I8);
     }
 };
 
 
-class FilterBlurTest: public Test {
+class FilterBlurTest: public GraphicsTest {
 public:
     FilterBlurTest()
-        : Test("FilterBlurTest", 2)
+        : GraphicsTest("FilterBlurTest", 2)
     {
     }
 
@@ -676,18 +655,15 @@ public:
         FilterFill<Pixel8>(0).applyInPlace(pBmp);
         *(pBmp->getPixels()+pBmp->getStride()*7+7) = 255;
         BitmapPtr pDestBmp = FilterBlur().apply(pBmp);
-//        pDestBmp->save("testimages/BlurResult.png");
-        BitmapPtr pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(getSrcDir()+"testimages/BlurResult.png")));
-        TEST(*pDestBmp == *pBaselineBmp);
+        testEqual(*pDestBmp, "BlurResult", I8);
     }
 };
 
 
-class FilterBandpassTest: public Test {
+class FilterBandpassTest: public GraphicsTest {
 public:
     FilterBandpassTest()
-        : Test("FilterBandpassTest", 2)
+        : GraphicsTest("FilterBandpassTest", 2)
     {
     }
 
@@ -698,10 +674,61 @@ public:
         *(pBmp->getPixels()+pBmp->getStride()*7+7) = 255;
         
         BitmapPtr pDestBmp = FilterBandpass(1.9,3).apply(pBmp);
-//        pDestBmp->save("testimages/BandpassResult.png");
-        string sFName = getSrcDir()+"testimages/BandpassResult.png";
-        BitmapPtr pBaselineBmp = FilterGrayscale().apply(
-                BitmapPtr(new Bitmap(sFName)));
+        testEqual(*pDestBmp, "BandpassResult", I8);
+    }
+};
+
+class FilterFastDownscaleTest: public GraphicsTest {
+public:
+    FilterFastDownscaleTest()
+        : GraphicsTest("FilterFastDownscaleTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        BitmapPtr pBmp = BitmapPtr(new Bitmap(IntPoint(4,4), I8));
+        FilterFill<Pixel8>(0).applyInPlace(pBmp);
+        *(pBmp->getPixels()+pBmp->getStride()*3+3) = 252;
+
+        BitmapPtr pDestBmp = FilterFastDownscale(2).apply(pBmp);
+        testEqual(*pDestBmp, "FastDownscaleResult", I8);
+    }
+};
+
+class FilterMaskTest: public GraphicsTest {
+public:
+    FilterMaskTest()
+        : GraphicsTest("FilterMaskTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        runTestsWithBmp(initBmp(I8), "I8");
+        runTestsWithBmp(initBmp(R8G8B8), "R8G8B8");
+        runTestsWithBmp(initBmp(R8G8B8X8), "R8G8B8X8");
+    }
+
+private:
+    void runTestsWithBmp(BitmapPtr pBmp, const string& sName)
+    {
+        BitmapPtr pMaskBmp = BitmapPtr(new Bitmap(pBmp->getSize(), I8));
+        FilterFill<Pixel8>(0).applyInPlace(pMaskBmp);
+        for (int y=0; y<pBmp->getSize().y; y++) {
+            pMaskBmp->setPixel(IntPoint(1, y), Pixel8(128));
+            pMaskBmp->setPixel(IntPoint(2, y), Pixel8(255));
+            pMaskBmp->setPixel(IntPoint(3, y), Pixel8(255));
+        }
+
+        BitmapPtr pDestBmp = FilterMask(pMaskBmp).apply(pBmp);
+        string sFName = string("baseline/MaskResult")+sName+".png";
+//        pDestBmp->save(sFName);
+        sFName = getSrcDirName()+sFName;
+        BitmapPtr pRGBXBaselineBmp = BitmapPtr(new Bitmap(sFName));
+        BitmapPtr pBaselineBmp = BitmapPtr(
+                new Bitmap(pRGBXBaselineBmp->getSize(), pBmp->getPixelFormat()));
+        pBaselineBmp->copyPixels(*pRGBXBaselineBmp);
         TEST(*pDestBmp == *pBaselineBmp);
     }
 };
@@ -728,6 +755,8 @@ public:
         addTest(TestPtr(new FilterBlurTest));
         addTest(TestPtr(new FilterBandpassTest));
         addTest(TestPtr(new FilterFastBandpassTest));
+        addTest(TestPtr(new FilterFastDownscaleTest));
+        addTest(TestPtr(new FilterMaskTest));
     }
 };
 
