@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,7 @@ using namespace std;
 FWCamera::FWCamera(std::string sDevice, IntPoint Size, std::string sPF,
             double FrameRate, bool bColor)
     : m_sDevice(sDevice),
+      m_sPF(sPF),
       m_Size(Size),
       m_FrameRate(FrameRate),
       m_bColor(bColor),
@@ -62,7 +63,6 @@ void FWCamera::open()
 {
 #ifdef AVG_ENABLE_1394
     int CaptureFormat = 0;
-    // TODO: Support other resolutions.
     switch(m_Mode) {
         case MODE_320x240_YUV422:
         case MODE_640x480_MONO:
@@ -132,10 +132,6 @@ void FWCamera::open()
         exit(1);
     }
     int err;
-    err = dc1394_get_camera_feature_set(m_FWHandle,
-                m_Camera.node, &m_FeatureSet);
-    checkDC1394Error(err,
-            "Unable to get firewire camera feature set.");
 //    dumpCameraInfo();
 
     const char * pDeviceFileName = 0;
@@ -161,13 +157,13 @@ void FWCamera::open()
     err = dc1394_start_iso_transmission(m_FWHandle, m_Camera.node);
     checkDC1394Error(err, "Unable to start camera iso transmission");
 #elif AVG_ENABLE_1394_2
-    dc1394camera_t **ppCameras=NULL;
-    uint32_t numCameras;
     m_bCameraAvailable = true;
+    dc1394camera_list_t * pCameraList;
 
-    int err=dc1394_find_cameras(&ppCameras, &numCameras);
+    m_pDC1394 = dc1394_new();
+    int err=dc1394_camera_enumerate(m_pDC1394, &pCameraList);
 
-    if (err!=DC1394_SUCCESS && err != DC1394_NO_CAMERA) {
+    if (err!=DC1394_SUCCESS) {
         AVG_TRACE(Logger::ERROR, "Unable to look for cameras");
 #ifdef linux
         AVG_TRACE(Logger::ERROR, "Please check");
@@ -179,7 +175,7 @@ void FWCamera::open()
         exit(1);
     }
     
-    if (numCameras<1) {
+    if (pCameraList->num == 0) {
         static bool bFirstWarning = true;
         if (bFirstWarning) {
             AVG_TRACE(Logger::WARNING, "No firewire cameras found.");
@@ -189,16 +185,15 @@ void FWCamera::open()
         return;
     }
     // This always uses the first camera on the bus.
-    m_pCamera=ppCameras[0];
+    m_pCamera=dc1394_camera_new(m_pDC1394, pCameraList->ids[0].guid);
+    if (!m_pCamera) {
+        AVG_TRACE(Logger::WARNING, "Could not initialize firewire camera.");
+        m_bCameraAvailable = false;
+        return;
+    }
 
-    // Free the other cameras
-    for (unsigned int i=1;i<numCameras;i++)
-        dc1394_free_camera(ppCameras[i]);
-    free(ppCameras);
+    dc1394_camera_free_list(pCameraList);
 
-    err = dc1394_get_camera_feature_set(m_pCamera, &m_FeatureSet);
-    checkDC1394Error(err,
-            "Unable to get firewire camera feature set.");
     dumpCameraInfo();
 
     err = dc1394_video_set_iso_speed(m_pCamera, DC1394_ISO_SPEED_400);
@@ -221,23 +216,21 @@ void FWCamera::open()
                 << " in the current video mode.");
         dc1394_capture_stop(m_pCamera);
         dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
-        dc1394_free_camera(m_pCamera);
+        dc1394_camera_free(m_pCamera);
         exit(1);
     }
 
     err = dc1394_video_set_framerate(m_pCamera, m_FrameRateConstant);
     checkDC1394Error(err, "Unable to set camera framerate.");
 
-    if (dc1394_capture_setup_dma(m_pCamera,8)!=DC1394_SUCCESS) {
-        AVG_TRACE(Logger::ERROR,
-                "Unable to setup camera. Make sure that");
-        AVG_TRACE(Logger::ERROR,
-                "video mode and framerate (" <<
+    if (dc1394_capture_setup(m_pCamera,8, DC1394_CAPTURE_FLAGS_DEFAULT)!=DC1394_SUCCESS) {
+        AVG_TRACE(Logger::ERROR, "Unable to setup camera. Make sure that");
+        AVG_TRACE(Logger::ERROR, "video mode and framerate (" <<
                 m_FrameRate << ") are");
         AVG_TRACE(Logger::ERROR, "supported by your camera.");
         dc1394_capture_stop(m_pCamera);
         dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
-        dc1394_free_camera(m_pCamera);
+        dc1394_camera_free(m_pCamera);
         exit(1);
     }
     if (dc1394_video_set_transmission(m_pCamera, DC1394_ON) !=DC1394_SUCCESS) {
@@ -273,9 +266,10 @@ void FWCamera::close()
 //        dc1394_dma_release_camera(m_FWHandle, &m_Camera);
         dc1394_destroy_handle(m_FWHandle);
 #elif AVG_ENABLE_1394_2
-        dc1394_capture_stop(m_pCamera);
         dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
-        dc1394_free_camera(m_pCamera);
+        dc1394_capture_stop(m_pCamera);
+        dc1394_camera_free(m_pCamera);
+        dc1394_free(m_pDC1394);
 #endif
         m_bCameraAvailable = false;
         AVG_TRACE(Logger::CONFIG, "Firewire camera closed.");
@@ -297,7 +291,7 @@ BitmapPtr FWCamera::getImage(bool bWait)
 {
 #if defined(AVG_ENABLE_1394) || defined(AVG_ENABLE_1394_2)
     if (!m_bCameraAvailable && bWait) {
-        TimeSource::get()->msleep(1000);
+        msleep(1000);
         open();
     }
     if (!m_bCameraAvailable) {
@@ -305,7 +299,7 @@ BitmapPtr FWCamera::getImage(bool bWait)
         return BitmapPtr();
     }
     BitmapPtr pCurBitmap;
-    if (m_bColor) { 
+    if (m_bColor || m_sPF == "BY8_GBRG") { 
         pCurBitmap = BitmapPtr(new Bitmap(m_Size, B8G8R8X8));
     } else {
         pCurBitmap = BitmapPtr(new Bitmap(m_Size, I8));
@@ -323,12 +317,13 @@ BitmapPtr FWCamera::getImage(bool bWait)
     bGotFrame = (rc == DC1394_SUCCESS);
 #else
     dc1394video_frame_t * pFrame;
+    dc1394error_t err;
     if (bWait) {
-        pFrame = dc1394_capture_dequeue_dma(m_pCamera, DC1394_VIDEO1394_WAIT);
+        err = dc1394_capture_dequeue(m_pCamera, DC1394_CAPTURE_POLICY_WAIT, &pFrame);
     } else {
-        pFrame = dc1394_capture_dequeue_dma(m_pCamera, DC1394_VIDEO1394_POLL);
+        err = dc1394_capture_dequeue(m_pCamera, DC1394_CAPTURE_POLICY_POLL, &pFrame);
     }
-    if (pFrame) {
+    if (err == DC1394_SUCCESS && pFrame) {
         bGotFrame = true;
         pCaptureBuffer = pFrame->image;
     }
@@ -339,14 +334,25 @@ BitmapPtr FWCamera::getImage(bool bWait)
             switch (m_Mode) {
 #ifdef AVG_ENABLE_1394
                 case MODE_640x480_MONO:
+                case MODE_1024x768_MONO:
 #else
                 case DC1394_VIDEO_MODE_640x480_MONO8:
+                case DC1394_VIDEO_MODE_1024x768_MONO8:
 #endif                    
                     {
-                        Bitmap TempBmp(m_Size, I8, 
-                                pCaptureBuffer,
-                                m_Size.x, false, "TempCameraBmp");
-                        pCurBitmap->copyPixels(TempBmp);
+                        // Bayer pattern conversion
+                        if (m_sPF == "BY8_GBRG") {
+                            Bitmap TempBmp(m_Size, BAYER8_GBRG, 
+                                    pCaptureBuffer,
+                                    m_Size.x, false, "TempCameraBmp");
+                            pCurBitmap->copyPixels(TempBmp);
+                        }
+                        else {
+                            Bitmap TempBmp(m_Size, I8, 
+                                    pCaptureBuffer,
+                                    m_Size.x, false, "TempCameraBmp");
+                            pCurBitmap->copyPixels(TempBmp);
+                        }
                     }
                     break;
 #ifdef AVG_ENABLE_1394
@@ -414,7 +420,7 @@ BitmapPtr FWCamera::getImage(bool bWait)
 #ifdef AVG_ENABLE_1394
         dc1394_dma_done_with_buffer(&m_Camera);
 #else
-        dc1394_capture_enqueue_dma (m_pCamera, pFrame);
+        dc1394_capture_enqueue(m_pCamera, pFrame);
 #endif
         return pCurBitmap;
     } else {
@@ -446,10 +452,10 @@ double FWCamera::getFrameRate() const
     return m_FrameRate;
 }
 
-unsigned int FWCamera::getFeature(const std::string& sFeature) const
+unsigned int FWCamera::getFeature(CameraFeature Feature) const
 {
 #if defined(AVG_ENABLE_1394) || defined(AVG_ENABLE_1394_2)
-    dc1394feature_t FeatureID = getFeatureID(sFeature);
+    dc1394feature_t FeatureID = getFeatureID(Feature);
     FeatureMap::const_iterator it = m_Features.find(FeatureID);
     if (it == m_Features.end()) {
         return 0;
@@ -461,13 +467,14 @@ unsigned int FWCamera::getFeature(const std::string& sFeature) const
 #endif
 }
 
-void FWCamera::setFeature(const std::string& sFeature, int Value)
+void FWCamera::setFeature(CameraFeature Feature, int Value)
 {
 #if defined(AVG_ENABLE_1394) || defined(AVG_ENABLE_1394_2)
-    dc1394feature_t FeatureID = getFeatureID(sFeature);
+    dc1394feature_t FeatureID = getFeatureID(Feature);
     m_Features[FeatureID] = Value;
     if (m_bCameraAvailable) {
         setFeature(FeatureID, Value);
+//        dumpCameraInfo();
     }
 #endif
 }
@@ -575,15 +582,21 @@ void FWCamera::dumpCameraInfo()
                 "Unable to get firewire camera info.");
     }
     // TODO: do this using AVG_TRACE
-    dc1394_print_feature_set(&m_FeatureSet);
+    dc1394_feature_set FeatureSet;
+    rc = dc1394_get_camera_feature_set(m_FWHandle, m_Camera.node, &FeatureSet);
+    checkDC1394Error(rc, "Unable to get firewire camera feature set.");
+    dc1394_print_feature_set(&FeatureSet);
 }
 
 #elif AVG_ENABLE_1394_2
 void FWCamera::dumpCameraInfo()
 {
+    dc1394error_t err;
+    dc1394featureset_t FeatureSet;
+    err = dc1394_feature_get_all(m_pCamera, &FeatureSet);
+    checkDC1394Error(err, "Unable to get firewire camera feature set.");
     // TODO: do this using AVG_TRACE
-    dc1394_print_camera_info(m_pCamera);
-    dc1394_print_feature_set(&m_FeatureSet);
+    dc1394_feature_print_all(&FeatureSet, stderr);
 }
 #else
 void FWCamera::dumpCameraInfo()
