@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -37,11 +37,11 @@ namespace avg {
     class FilterDistortion: public Filter 
     {
         public:
-            FilterDistortion(IntPoint srcSize, CoordTransformerPtr coordtrans);
+            FilterDistortion(const IntPoint& SrcSize, CoordTransformerPtr coordtrans);
             virtual ~FilterDistortion();
             BitmapPtr apply (BitmapPtr pBmpSource);
         private:
-            IntRect m_srcRect;
+            IntPoint m_SrcSize;
             CoordTransformerPtr m_pTrafo;
             IntPoint *m_pMap;
     };
