@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,20 +30,10 @@ namespace avg {
 Run::Run(int row, int start_col, int end_col)
 {
     m_Row = row;
-    assert(end_col>=start_col);
+//    assert(end_col>=start_col);
     m_StartCol = start_col;
     m_EndCol = end_col;
+    m_Center = DPoint((m_StartCol + m_EndCol-1)/2., m_Row);
 }
  
-int Run::length()
-{
-    return m_EndCol-m_StartCol;
-}
-
-DPoint Run::center()
-{
-    DPoint d = DPoint((m_StartCol + m_EndCol-1)/2., m_Row);
-    return d;
-}
-
 }
