@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -41,13 +41,14 @@ struct Run
         int m_Row;
         int m_StartCol;
         int m_EndCol;
-        int length();
-        DPoint center();
+        DPoint m_Center;
+        int length() {
+            return m_EndCol-m_StartCol;
+        };
         BlobWeakPtr m_pBlob;
 };
 
-typedef boost::shared_ptr<Run> RunPtr;
-typedef std::vector<RunPtr> RunArray;
+typedef std::vector<Run> RunArray;
 
 }
 
