@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -39,6 +39,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <iostream>
+#include <cstring>
 
 #define CLEAR(x) memset (&(x), 0, sizeof (x))
 
@@ -93,14 +94,14 @@ void V4LCamera::open()
     struct stat St; 
 
     if ( stat(m_sDevice.c_str(), &St) == -1) {
-        AVG_TRACE(Logger::ERROR, "Unable to access v4l device " << 
+        AVG_TRACE(Logger::ERROR, "Unable to access v4l2 device " << 
           m_sDevice);
         // TODO: Disable camera instead of exit(1).
         exit(1);
     }
 
     if (!S_ISCHR (St.st_mode)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice + " is not a v4l device");
+        AVG_TRACE(Logger::ERROR, m_sDevice + " is not a v4l2 device");
         // TODO: Disable camera instead of exit(1).
         exit(1);
     }
@@ -108,14 +109,14 @@ void V4LCamera::open()
     m_Fd = ::open(m_sDevice.c_str(), O_RDWR /* required */ | O_NONBLOCK, 0);
 
     if (m_Fd == -1) {
-        AVG_TRACE(Logger::ERROR, "Unable to open v4l device " << m_sDevice);
+        AVG_TRACE(Logger::ERROR, "Unable to open v4l2 device " << m_sDevice);
         // TODO: Disable camera instead of exit(1).
         exit(1);
     }
     
     initDevice();
     startCapture();
-    AVG_TRACE(Logger::CONFIG, "V4L Camera opened");
+    AVG_TRACE(Logger::CONFIG, "V4L2 Camera opened");
 }
 
 void V4LCamera::close()
@@ -134,9 +135,9 @@ void V4LCamera::close()
         m_vBuffers.clear();
 
         if ( ::close(m_Fd) == -1) {
-            AVG_TRACE(Logger::ERROR, "Error on closing v4l device");
+            AVG_TRACE(Logger::ERROR, "Error on closing v4l2 device");
         }
-        AVG_TRACE(Logger::CONFIG, "V4L Camera closed");
+        AVG_TRACE(Logger::CONFIG, "V4L2 Camera closed");
 
         m_Fd = -1;
         m_bCameraAvailable = false;
@@ -174,8 +175,7 @@ int V4LCamera::getCamPF(const std::string& sPF)
     }
     else {
         AVG_TRACE (Logger::WARNING,
-                std::string("Unsupported or illegal value for camera \
-                pixel format \"") 
+                std::string("Unsupported or illegal value for camera pixel format \"") 
                 + sPF + std::string("\"."));
     }
     
@@ -194,7 +194,6 @@ BitmapPtr V4LCamera::getImage(bool bWait)
     
     // wait for incoming data blocking, timeout 2s
     if (bWait) {
-        cerr << "bWait" << endl;
         fd_set Fds;
         struct timeval Tv;
         int Rc;
@@ -210,13 +209,12 @@ BitmapPtr V4LCamera::getImage(bool bWait)
 
         // caught signal or something else  
         if (Rc == -1) {
-            AVG_TRACE(Logger::WARNING, "V4L: select failed.");
+            AVG_TRACE(Logger::WARNING, "V4L2: select failed.");
             return BitmapPtr();
         }
         // timeout
         if (Rc == 0) {
-            AVG_TRACE(Logger::WARNING, "V4L: Timeout while waiting \
-               for image data");
+            AVG_TRACE(Logger::WARNING, "V4L2: Timeout while waiting for image data");
             return BitmapPtr();
         }
     }
@@ -337,7 +335,7 @@ const std::string& V4LCamera::getDriverName() const
 
 double V4LCamera::getFrameRate() const
 {
-    // TODO: PAL or NTSC have different frame rates
+    // TODO: PAL or NTSC have different frame rates. So do USB cameras!
     return 25;
 }
 
@@ -354,25 +352,26 @@ std::string V4LCamera::getFeatureName(V4LCID_t V4LFeature)
     return sName;
 }
 
-V4LCID_t V4LCamera::getFeatureID(const std::string& sFeature) const
+V4LCID_t V4LCamera::getFeatureID(CameraFeature Feature) const
 {
     V4LCID_t V4LFeature;
-    if (sFeature == "brightness") {
+    if (Feature == CAM_FEATURE_BRIGHTNESS) {
         V4LFeature = V4L2_CID_BRIGHTNESS;
-    } else if (sFeature == "contrast") {
+    } else if (Feature == CAM_FEATURE_CONTRAST) {
         V4LFeature = V4L2_CID_CONTRAST;
-    }  else if (sFeature == "gain") {
+    }  else if (Feature == CAM_FEATURE_GAIN) {
         V4LFeature = V4L2_CID_GAIN;
-    } else if (sFeature == "exposure") {
+    } else if (Feature == CAM_FEATURE_EXPOSURE) {
         V4LFeature = V4L2_CID_EXPOSURE;
-    } else if (sFeature == "whiteness") {
+    } else if (Feature == CAM_FEATURE_WHITE_BALANCE) {
         V4LFeature = V4L2_CID_WHITENESS;
-    } else if (sFeature == "gamma") {
+    } else if (Feature == CAM_FEATURE_GAMMA) {
         V4LFeature = V4L2_CID_GAMMA;
-    } else if (sFeature == "saturation") {
+    } else if (Feature == CAM_FEATURE_SATURATION) {
         V4LFeature = V4L2_CID_SATURATION;
     } else {
-        AVG_TRACE(Logger::WARNING, "Unknown feature " << sFeature);
+        AVG_TRACE(Logger::WARNING, "Feature " << cameraFeatureToString(Feature)
+                << " not supported for V4L2.");
         return -1;
     }
     
@@ -401,9 +400,9 @@ bool V4LCamera::isFeatureSupported(V4LCID_t V4LFeature) const
     }
 }
 
-unsigned int V4LCamera::getFeature(const std::string& sFeature) const
+unsigned int V4LCamera::getFeature(CameraFeature Feature) const
 {
-    V4LCID_t V4LFeature = getFeatureID(sFeature);
+    V4LCID_t V4LFeature = getFeatureID(Feature);
     
     FeatureMap::const_iterator it = m_Features.find(V4LFeature);
     
@@ -417,8 +416,7 @@ unsigned int V4LCamera::getFeature(const std::string& sFeature) const
 void V4LCamera::setFeature(V4LCID_t V4LFeature, int Value)
 {
     if (!m_bCameraAvailable) {
-        AVG_TRACE(Logger::WARNING, "setFeature() called before opening \
-            device: ignored");
+        AVG_TRACE(Logger::WARNING, "setFeature() called before opening device: ignored");
         return;
     }
 
@@ -443,12 +441,12 @@ void V4LCamera::setFeature(V4LCID_t V4LFeature, int Value)
     }
 }
 
-void V4LCamera::setFeature(const std::string& sFeature, int Value)
+void V4LCamera::setFeature(CameraFeature Feature, int Value)
 {
     // ignore -1 coming from default unbiased cameranode parameters
     if (Value < 0) return;
     
-    V4LCID_t V4LFeature = getFeatureID(sFeature);
+    V4LCID_t V4LFeature = getFeatureID(Feature);
 
     m_Features[V4LFeature] = Value;
 
@@ -499,26 +497,18 @@ void V4LCamera::initDevice()
     struct v4l2_crop Crop;
     struct v4l2_format Fmt;
 
-    if (-1 == xioctl (m_Fd, VIDIOC_QUERYCAP, &Cap)) {
-        if (EINVAL == errno) {
-            AVG_TRACE(Logger::ERROR, m_sDevice << " is not a valid \
-                V4L2 device");
-            exit(1);
-        } else {
-            AVG_TRACE(Logger::ERROR, "VIDIOC_QUERYCAP error");
-            exit(1);
-        }
+    if (xioctl(m_Fd, VIDIOC_QUERYCAP, &Cap) == -1) {
+        AVG_TRACE(Logger::ERROR, m_sDevice << " is not a valid V4L2 device");
+        exit(1);
     }
 
     if (!(Cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support \
-            capturing");
+        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support capturing");
         exit(1);
     }
 
     if (!(Cap.capabilities & V4L2_CAP_STREAMING)) {
-        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support \
-            streaming i/os");
+        AVG_TRACE(Logger::ERROR, m_sDevice << " does not support streaming i/os");
         exit(1);
     }
     m_sDriverName = (const char *)Cap.driver;
