@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,9 +34,11 @@
 #include <boost/bind.hpp>
 
 #include <math.h>
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 
 #include <sstream>
 
@@ -122,6 +124,9 @@ class DeDistortTest: public Test {
     }
 };
 
+#ifdef _WIN32
+#pragma warning(disable: 4996)
+#endif
 class SerializeTest: public Test {
 public:
     SerializeTest()
@@ -131,21 +136,27 @@ public:
 
     void runTests() 
     {
-        vector<double> Params;
-        Params.push_back(0);
-        Params.push_back(0);
-        DeDistortPtr pScaler = DeDistortPtr(new DeDistort(DPoint(1,1),
-                Params, 0, 0.0,
-                DPoint(0,0), DPoint(2,2)));
         TrackerConfig Config;
-        Config.m_pTrafo = pScaler;
-        Config.save("temptrackerrc");
-        TrackerConfig LoadedConfig;
-        LoadedConfig.load("temptrackerrc");
-        DeDistortPtr pTrafo = LoadedConfig.m_pTrafo;
-        TEST(almostEqual(pTrafo->transform_point(DPoint(0,0)), DPoint(0,0)));
-        TEST(almostEqual(pTrafo->transformBlobToScreen(DPoint(1,2)), DPoint(2,4)));
-        unlink("temptrackerrc");
+        Config.load(getSrcDirName()+"avgtrackerrc.minimal");
+        
+        Config.setParam("/transform/distortionparams/@p2", "0");
+        Config.setParam("/transform/distortionparams/@p3", "0");
+        Config.setParam("/transform/trapezoid/@value", "0");
+        Config.setParam("/transform/angle/@value", "0");
+        Config.setParam("/transform/displaydisplacement/@x", "0");
+        Config.setParam("/transform/displaydisplacement/@y", "0");
+        Config.setParam("/transform/displayscale/@x", "2");
+        Config.setParam("/transform/displayscale/@y", "2");
+
+        if (getSrcDirName() == "./") {
+            Config.save(getSrcDirName()+"avgtrackerrc.minimal.mod");
+
+            TrackerConfig LoadedConfig;
+            LoadedConfig.load(getSrcDirName()+"avgtrackerrc.minimal.mod");
+            DPoint Scale = LoadedConfig.getPointParam("/transform/displayscale/");
+            TEST(almostEqual(Scale, DPoint(2,2)));
+            unlink((getSrcDirName()+"avgtrackerrc.minimal.mod").c_str());
+        }
     }
 };
 
