@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,9 +29,9 @@ const char * g_pTrackerConfigDTD =
 "    y CDATA #REQUIRED\" >\n"
  
 "<!ELEMENT trackerconfig (camera|tracker|transform)* >\n"
-"<!ELEMENT camera (source|device|size|channel|fps|brightness|exposure|gamma|gain|shutter)* >\n"
-"<!ELEMENT tracker (historyupdateinterval|touch|track)* >\n"
-"<!ELEMENT touch (threshold|similarity|areabounds|eccentricitybounds)* >\n"
+"<!ELEMENT camera (source|device|format|size|channel|fps|brightness|exposure|gamma|gain|shutter)* >\n"
+"<!ELEMENT tracker (mask|prescale|historyupdateinterval|brighterregions|eventonmove|contourprecision|touch|track)* >\n"
+"<!ELEMENT touch (threshold|similarity|areabounds|eccentricitybounds|bandpass|bandpasspostmult)* >\n"
 "<!ELEMENT track (threshold|similarity|areabounds|eccentricitybounds)* >\n"
 "<!ELEMENT transform (cameradisplacement|camerascale|distortionparams|trapezoid|"
 "       angle|displaydisplacement|displayscale)* >\n"
@@ -44,6 +44,10 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST device\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT format EMPTY>\n"
+"<!ATTLIST format\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT size EMPTY>\n"
 "<!ATTLIST size\n"
 "   %posAttrs; >\n"
@@ -76,14 +80,34 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST shutter\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT brighterregions EMPTY>\n"
+"<!ATTLIST brighterregions\n"
+"   value CDATA #REQUIRED >\n"
+
+"<!ELEMENT eventonmove EMPTY>\n"
+"<!ATTLIST eventonmove\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT threshold EMPTY>\n"
 "<!ATTLIST threshold\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT mask EMPTY>\n"
+"<!ATTLIST mask\n"
+"   value CDATA #REQUIRED >\n"
+
+"<!ELEMENT prescale EMPTY>\n"
+"<!ATTLIST prescale\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT historyupdateinterval EMPTY>\n"
 "<!ATTLIST historyupdateinterval\n"
 "   value CDATA #REQUIRED >\n"
 
+"<!ELEMENT contourprecision EMPTY>\n"
+"<!ATTLIST contourprecision\n"
+"   value CDATA #REQUIRED >\n"
+
 "<!ELEMENT similarity EMPTY>\n"
 "<!ATTLIST similarity\n"
 "   value CDATA #REQUIRED >\n"
@@ -96,18 +120,19 @@ const char * g_pTrackerConfigDTD =
 "<!ATTLIST eccentricitybounds\n"
 "   %minmaxAttrs; >\n"
 
-"<!ELEMENT cameradisplacement EMPTY>\n"
-"<!ATTLIST cameradisplacement\n"
-"   %posAttrs; >\n"
+"<!ELEMENT bandpass EMPTY>\n"
+"<!ATTLIST bandpass\n"
+"   %minmaxAttrs; >\n"
 
-"<!ELEMENT camerascale EMPTY>\n"
-"<!ATTLIST camerascale\n"
-"   %posAttrs; >\n"
+"<!ELEMENT bandpasspostmult EMPTY>\n"
+"<!ATTLIST bandpasspostmult\n"
+"   value CDATA #REQUIRED >\n"
 
 "<!ELEMENT distortionparams EMPTY>\n"
 "<!ATTLIST distortionparams\n"
 "   p2 CDATA #REQUIRED\n"
-"   p3 CDATA #REQUIRED >\n"
+"   p3 CDATA #REQUIRED\n"
+"   p4 CDATA #REQUIRED >\n"
 
 "<!ELEMENT trapezoid EMPTY>\n"
 "<!ATTLIST trapezoid\n"
