@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "../avgconfigwrapper.h"
 #include "../base/Logger.h"
 #include "../base/Exception.h"
-#include "../player/MathHelper.h"
+#include "../base/MathHelper.h"
 
 #ifdef AVG_ENABLE_PARPORT 
 #include <linux/ppdev.h>
@@ -36,6 +36,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
+#include <cstring>
 #include <iostream>
 #include <sstream>
 #include <cerrno>
