@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,12 +31,16 @@ namespace avg {
 
 class AVGNode : public DivNode
 {
-	public:
-        AVGNode (const xmlNodePtr xmlNode, Player * pPlayer);
+    public:
+        static NodeDefinition getNodeDefinition();
+        
+        AVGNode (const ArgList& Args, Player * pPlayer, bool bFromXML);
         virtual ~AVGNode ();
 
         virtual std::string getTypeStr ();
+        virtual std::string getEffectiveMediaDir();
         bool getCropSetting();
+        void setAngle(double angle);
         
     private:
         bool m_bEnableCrop;
@@ -46,4 +50,3 @@ class AVGNode : public DivNode
 }
 
 #endif //_AVGNode_H_
-
