@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,7 @@
 
 namespace avg{
 
-CursorEvent::CursorEvent(int id, Type eventType, IntPoint Position, Source source)
+CursorEvent::CursorEvent(int id, Type eventType, const IntPoint& Position, Source source)
             :Event(eventType, source),
             m_Position(Position),
             m_ID(id)
@@ -37,9 +37,15 @@ CursorEvent::~CursorEvent()
 {
 }
 
-Event *CursorEvent::cloneAs(Type EventType)
+CursorEventPtr CursorEvent::cloneAs(Type EventType) const
 {
     assert(false);
+    return CursorEventPtr();
+}
+
+DPoint CursorEvent::getPos() const
+{
+    return DPoint(m_Position);
 }
 
 int CursorEvent::getXPosition() const
@@ -57,5 +63,16 @@ int CursorEvent::getCursorID() const
     return m_ID;
 }
 
+DPoint CursorEvent::getLastDownPos() const
+{
+    return DPoint(m_LastDownPos);
+}
+
+void CursorEvent::setLastDownPos(const IntPoint& pos)
+{
+    m_LastDownPos = pos;
+}
+
+
 }
 
