@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,19 +32,27 @@ namespace avg {
 
 const int MOUSECURSORID=-1;
 
+class CursorEvent;
+typedef boost::shared_ptr<class CursorEvent> CursorEventPtr;
+
 class CursorEvent: public Event 
 {
     public:
-        CursorEvent(int id, Type eventType, IntPoint Position, Source source);
+        CursorEvent(int id, Type eventType, const IntPoint& Position, Source source);
         virtual ~CursorEvent();
-        virtual Event* cloneAs(Type EventType);
+        virtual CursorEventPtr cloneAs(Type EventType) const;
+        DPoint getPos() const;
         int getXPosition() const;
         int getYPosition() const;
         int getCursorID() const;
 
+        DPoint getLastDownPos() const;
+        void setLastDownPos(const IntPoint& pos);
+
     protected:
         IntPoint m_Position;
         int m_ID;
+        IntPoint m_LastDownPos;
 };
 
 }
