@@ -1,6 +1,6 @@
 
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,7 @@ class ISurface;
 class Region;
 
 class DisplayEngine
-{	
+{   
     public:
         typedef enum BlendMode {BLEND_BLEND, BLEND_ADD, BLEND_MIN, BLEND_MAX};
 
@@ -60,20 +60,19 @@ class DisplayEngine
         virtual void setGamma(double Red, double Green, double Blue) = 0;
 
 
-        virtual void render(AVGNodePtr pRootNode, bool bRenderEverything) = 0;
+        virtual void render(AVGNodePtr pRootNode) = 0;
         void frameWait();
         long long getDisplayTime();
         
-        virtual void setClipRect() = 0;
-        virtual bool pushClipRect(const DRect& rc, bool bClip) = 0;
+        virtual bool pushClipRect(const DRect& rc) = 0;
         virtual void popClipRect() = 0;
         virtual const DRect& getClipRect() = 0;
-        virtual void blt32(ISurface * pSurface, const DRect* pDestRect, 
-                double opacity, double angle, const DPoint& pivot,
-                BlendMode Mode) = 0;
-        virtual void blta8(ISurface * pSurface, const DRect* pDestRect, 
-                double opacity, const Pixel32& color, double angle,
-                const DPoint& pivot, BlendMode Mode) = 0;
+        virtual void pushTransform(const DPoint& translate, double angle, const DPoint& pivot) = 0;
+        virtual void popTransform() = 0;
+        virtual void blt32(ISurface * pSurface, const DPoint& DestSize, 
+                double opacity, BlendMode Mode) = 0;
+        virtual void blta8(ISurface * pSurface, const DPoint& DestSize, 
+                double opacity, const Pixel32& color, BlendMode Mode) = 0;
         virtual ISurface * createSurface() = 0;
         virtual void surfaceChanged(ISurface * pSurface) {};
 
@@ -89,7 +88,9 @@ class DisplayEngine
         virtual void showCursor (bool bShow) = 0;
 
         virtual BitmapPtr screenshot () = 0;
-    
+        virtual long long getGPUMemoryUsage() = 0;
+        virtual void deregisterSurface(OGLSurface *) = 0;
+
     protected:
         void checkJitter();
         
@@ -119,4 +120,3 @@ class DisplayEngine
 }
 
 #endif //_DisplayEngine_H_
-
