@@ -1,5 +1,5 @@
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,9 @@
 namespace avg {
 
 DisplayParams::DisplayParams()
-    : m_Width(0),
+    : m_x(-1),
+      m_y(-1),
+      m_Width(0),
       m_Height(0),
       m_bFullscreen(false),
       m_BPP(24),
