@@ -1,6 +1,6 @@
 
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,8 @@ struct DisplayParams {
     DisplayParams();
     virtual ~DisplayParams();
 
+    int m_x;
+    int m_y;
     int m_Width;
     int m_Height;
     bool m_bFullscreen;
