@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -43,7 +43,7 @@ class Event {
             RESIZE,
             QUIT 
         };
-        enum Source {MOUSE, TOUCH, TRACK, NONE};
+        enum Source {MOUSE=1, TOUCH=2, TRACK=4, NONE=8};
     
         Event(Type type, Source source=NONE, int when=-1);
         Event(const Event& e);
@@ -51,7 +51,7 @@ class Event {
         
         virtual void trace();
 
-        int getWhen() const;
+        long long getWhen() const;
         Type getType() const;
         Event::Source getSource() const;
         NodePtr getElement() const;
@@ -62,7 +62,7 @@ class Event {
     protected:
         Type m_Type;
         NodePtr m_pNode;
-        int m_When;
+        long long m_When;
 
     private:
         int m_Counter;
@@ -72,7 +72,7 @@ class Event {
 };
 
 // Functor to compare two EventPtrs chronologically
-typedef Event * EventPtr;
+typedef boost::shared_ptr<class Event> EventPtr;
 struct isEventAfter:std::binary_function<EventPtr, EventPtr, bool> {
     bool operator()(const EventPtr & x, const EventPtr & y) const {
         if (x->getWhen() == y->getWhen()) {
