@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@ namespace avg {
 class IEventSink {
     public:
         virtual ~IEventSink() {};
-        virtual bool handleEvent(Event * pEvent) =0;
+        virtual bool handleEvent(EventPtr pEvent) =0;
 };
 
 }
