@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@ class IEventSource {
     public:
         virtual ~IEventSource() {};
         virtual void initEventSource() {};
-        virtual std::vector<Event *> pollEvents()=0;
+        virtual std::vector<EventPtr> pollEvents()=0;
 };
 
 }
