@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -39,7 +39,7 @@ class ISurface {
         virtual PixelFormat getPixelFormat() = 0;
         virtual IntPoint getSize() = 0;
 
-        virtual bool wouldTile(IntPoint Size) { return false; }
+        virtual bool isOneTexture(IntPoint Size) { return false; }
 };
 
 }
