@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,12 +30,13 @@ using namespace std;
 namespace avg {
 
 KeyEvent::KeyEvent(Type eventType, unsigned char scanCode, int keyCode, 
-                const string& keyString, int modifiers)
+                const string& keyString, int unicode, int modifiers)
     : Event(eventType)
 {
     m_ScanCode = scanCode;
     m_KeyCode = keyCode;
     m_KeyString = keyString;
+    m_Unicode = unicode;
     m_Modifiers = modifiers;
 }
 
@@ -58,6 +59,11 @@ const std::string& KeyEvent::getKeyString() const
     return m_KeyString;
 }
 
+int KeyEvent::getUnicode() const
+{
+    return m_Unicode;
+}
+
 int KeyEvent::getModifiers() const
 {
     return m_Modifiers;
