@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,15 +20,18 @@
 //
 
 #include "Node.h"
-#include "DivNode.h"
 
+#include "NodeDefinition.h"
 #include "Event.h"
 #include "CursorEvent.h"
 #include "MouseEvent.h"
 #include "DivNode.h"
 #include "Player.h"
 #include "DisplayEngine.h"
+#include "Arg.h"
+#include "WrapPython.h" 
 
+#include "../base/MathHelper.h"
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/XMLHelper.h"
@@ -37,7 +40,7 @@
 #include <object.h>
 #include <compile.h>
 #include <eval.h>
-#include <boost/python.hpp>
+#include "BoostPython.h"
 
 #include <iostream>
 
@@ -47,29 +50,38 @@ using namespace std;
 
 namespace avg {
 
-Node::Node (const xmlNodePtr xmlNode, Player * pPlayer)
+
+NodeDefinition Node::getNodeDefinition()
+{
+    return NodeDefinition("node")
+        .addArg(Arg<string>("id", "", false, offsetof(Node, m_ID)))
+        .addArg(Arg<string>("oncursormove", ""))
+        .addArg(Arg<string>("oncursorup", ""))
+        .addArg(Arg<string>("oncursordown", ""))
+        .addArg(Arg<string>("oncursorover", ""))
+        .addArg(Arg<string>("oncursorout", ""))
+        .addArg(Arg<double>("x", 0.0, false, offsetof(Node, m_RelViewport.tl.x)))
+        .addArg(Arg<double>("y", 0.0, false, offsetof(Node, m_RelViewport.tl.y)))
+        .addArg(Arg<double>("width", 0.0, false, offsetof(Node, m_WantedSize.x)))
+        .addArg(Arg<double>("height", 0.0, false, offsetof(Node, m_WantedSize.y)))
+        .addArg(Arg<double>("angle", 0.0, false, offsetof(Node, m_Angle)))
+        .addArg(Arg<double>("pivotx", -32767, false, offsetof(Node, m_Pivot.x)))
+        .addArg(Arg<double>("pivoty", -32767, false, offsetof(Node, m_Pivot.y)))
+        .addArg(Arg<double>("opacity", 1.0, false, offsetof(Node, m_Opacity)))
+        .addArg(Arg<bool>("active", true, false, offsetof(Node, m_bActive)))
+        .addArg(Arg<bool>("sensitive", true, false, offsetof(Node, m_bSensitive)));
+}
+
+Node::Node (Player * pPlayer)
     : m_pParent(),
       m_This(),
-      m_pEngine(0),
+      m_pDisplayEngine(0),
+      m_pAudioEngine(0),
       m_pPlayer(pPlayer),
       m_RelViewport(0,0,0,0),
-      m_AbsViewport(0,0,0,0)
+      m_State(NS_UNCONNECTED)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
-    m_ID = getDefaultedStringAttr (xmlNode, "id", "");
-    addEventHandlers(Event::CURSORMOTION, getDefaultedStringAttr (xmlNode, "oncursormove", ""));
-    addEventHandlers(Event::CURSORUP, getDefaultedStringAttr (xmlNode, "oncursorup", ""));
-    addEventHandlers(Event::CURSORDOWN, getDefaultedStringAttr (xmlNode, "oncursordown", ""));
-    addEventHandlers(Event::CURSOROVER, getDefaultedStringAttr (xmlNode, "oncursorover", ""));
-    addEventHandlers(Event::CURSOROUT, getDefaultedStringAttr (xmlNode, "oncursorout", ""));
-    m_RelViewport.tl.x = getDefaultedDoubleAttr (xmlNode, "x", 0.0);
-    m_RelViewport.tl.y = getDefaultedDoubleAttr (xmlNode, "y", 0.0);
-    m_WantedSize.x = getDefaultedDoubleAttr (xmlNode, "width", 0.0);
-    m_WantedSize.y = getDefaultedDoubleAttr (xmlNode, "height", 0.0);
-    m_Opacity = getDefaultedDoubleAttr (xmlNode, "opacity", 1.0);
-    m_bActive = getDefaultedBoolAttr (xmlNode, "active", true);
-    m_bSensitive = getDefaultedBoolAttr (xmlNode, "sensitive", true);
-    setState(NS_UNCONNECTED);
 }
 
 Node::~Node()
@@ -86,92 +98,184 @@ void Node::setThis(NodeWeakPtr This)
     m_This = This;
 }
 
-void Node::setParent(DivNodeWeakPtr pParent)
+void Node::setArgs(const ArgList& Args)
 {
+    addEventHandlers(Event::CURSORMOTION, Args.getArgVal<string> ("oncursormove"));
+    addEventHandlers(Event::CURSORUP, Args.getArgVal<string> ("oncursorup"));
+    addEventHandlers(Event::CURSORDOWN, Args.getArgVal<string> ("oncursordown"));
+    addEventHandlers(Event::CURSOROVER, Args.getArgVal<string> ("oncursorover"));
+    addEventHandlers(Event::CURSOROUT, Args.getArgVal<string> ("oncursorout"));
+    m_RelViewport.setWidth(m_WantedSize.x);
+    m_RelViewport.setHeight(m_WantedSize.y);
+}
+
+void Node::setParent(DivNodeWeakPtr pParent, NodeState parentState)
+{
+    assert(getState() == NS_UNCONNECTED);
     if (getParent() && !!(pParent.lock())) {
         throw(Exception(AVG_ERR_UNSUPPORTED, 
                 string("Can't change parent of node (") + m_ID + ")."));
     }
     m_pParent = pParent;
-    setState(NS_CONNECTED);
+    if (parentState != NS_UNCONNECTED) {
+        connect();
+    }
 }
 
-void Node::setDisplayEngine(DisplayEngine * pEngine)
+void Node::removeParent()
 {
-    DPoint PreferredSize = getPreferredMediaSize();
+    m_pParent = DivNodePtr();
+    if (getState() != NS_UNCONNECTED) {
+        disconnect();
+    }
+}
+
+void Node::setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine)
+{
+    assert(getState() == NS_CONNECTED);
+    m_bHasCustomPivot = ((m_Pivot.x != -32767) && (m_Pivot.y != -32767));
+    IntPoint PreferredSize = getMediaSize();
     if (m_WantedSize.x == 0.0) {
-        m_RelViewport.SetWidth(PreferredSize.x);
+        m_RelViewport.setWidth(PreferredSize.x);
     } else {
-        m_RelViewport.SetWidth(m_WantedSize.x);
+        m_RelViewport.setWidth(m_WantedSize.x);
     }
     if (m_WantedSize.y == 0.0) {
-        m_RelViewport.SetHeight(PreferredSize.y);
+        m_RelViewport.setHeight(PreferredSize.y);
     } else {
-        m_RelViewport.SetHeight(m_WantedSize.y);
+        m_RelViewport.setHeight(m_WantedSize.y);
     } 
-    m_pEngine = pEngine;
-    DPoint pos(0,0);
-    if (getParent()) {
-        pos = getParent()->getAbsViewport().tl;
-    }
-    m_AbsViewport = DRect (pos+getRelViewport().tl, pos+getRelViewport().br);
+    m_pDisplayEngine = pDisplayEngine;
+    m_pAudioEngine = pAudioEngine;
+    setState(NS_CANRENDER);
+}
+
+void Node::connect()
+{
+    setState(NS_CONNECTED);
 }
 
 void Node::disconnect()
 {
-    m_pEngine = 0;
+    assert(getState() != NS_UNCONNECTED);
+    if (getState() == NS_CANRENDER) {
+        m_pDisplayEngine = 0;
+        m_pAudioEngine = 0;
+    }
     getPlayer()->removeNodeID(m_ID);
     setState(NS_UNCONNECTED);
 }
 
-const string& Node::getID () const
+const string& Node::getID() const
 {
     return m_ID;
 }
 
 void Node::setID(const std::string& ID)
 {
-    if (getState() == NS_CONNECTED) {
+    if (getState() != NS_UNCONNECTED) {
         throw(Exception(AVG_ERR_UNSUPPORTED, "Node with ID "+m_ID
                 +" is connected. setID invalid."));
     }
     m_ID = ID;
 }
 
-double Node::getX() const {
+double Node::getX() const 
+{
     return m_RelViewport.tl.x;
 }
 
-void Node::setX(double x) {
+void Node::setX(double x) 
+{
     setViewport(x, -32767, -32767, -32767);
 }
 
-double Node::getY() const {
+double Node::getY() const 
+{
     return m_RelViewport.tl.y;
 }
 
-void Node::setY(double y) {
+void Node::setY(double y) 
+{
     setViewport(-32767, y, -32767, -32767);
 }
 
-double Node::getWidth() const {
-    return getRelViewport().Width();
+const DPoint& Node::getPos() const
+{
+    return m_RelViewport.tl;
 }
 
-void Node::setWidth(double width) {
+void Node::setPos(const DPoint& pt)
+{
+    setViewport(pt.x, pt.y, -32767, -32767);
+}
+
+double Node::getWidth() 
+{
+    return getRelViewport().width();
+}
+
+void Node::setWidth(double width) 
+{
     m_WantedSize.x = width;
     setViewport(-32767, -32767, width, -32767);
 }
 
-double Node::getHeight() const {
-    return getRelViewport().Height();
+double Node::getHeight()
+{
+    return getRelViewport().height();
 }
 
-void Node::setHeight(double height) {
+void Node::setHeight(double height) 
+{
     m_WantedSize.y = height;
     setViewport(-32767, -32767, -32767, height);
 }
 
+DPoint Node::getSize() const
+{
+    return getRelViewport().size();
+}
+
+void Node::setSize(const DPoint& pt)
+{
+    setViewport(-32767, -32767, pt.x, pt.y);
+}
+
+double Node::getAngle() const
+{
+    return m_Angle;
+}
+
+void Node::setAngle(double Angle)
+{
+    m_Angle = fmod(Angle, 2*PI);
+}
+
+double Node::getPivotX() const
+{
+    return m_Pivot.x;
+}
+
+void Node::setPivotX(double Pivotx)
+{
+    m_Pivot = getPivot();
+    m_Pivot.x = Pivotx;
+    m_bHasCustomPivot = true;
+}
+
+double Node::getPivotY() const
+{
+    return m_Pivot.y;
+}
+
+void Node::setPivotY(double Pivoty)
+{
+    m_Pivot = getPivot();
+    m_Pivot.y = Pivoty;
+    m_bHasCustomPivot = true;
+}
+
 double Node::getOpacity() const {
     return m_Opacity;
 }
@@ -214,10 +318,37 @@ DivNodePtr Node::getParent() const
     }
 }
 
+void Node::unlink()
+{
+    if (m_pParent.expired()) {
+        throw(Exception(AVG_ERR_UNSUPPORTED, "Node with ID "+m_ID
+                +" has no parent. unlink invalid."));
+    }
+    DivNodePtr pParent = m_pParent.lock();
+    pParent->removeChild(pParent->indexOf(getThis()));
+}
+
 DPoint Node::getRelPos(const DPoint& AbsPos) const 
 {
-    DRect VP = getAbsViewport();
-    return AbsPos-VP.tl;
+    DPoint parentPos;
+    if (m_pParent.expired()) {
+        parentPos = AbsPos;
+    } else {
+        parentPos = m_pParent.lock()->getRelPos(AbsPos);
+    }
+    return toLocal(parentPos);
+}
+
+DPoint Node::getAbsPos(const DPoint& RelPos) const 
+{
+    DPoint thisPos = toGlobal(RelPos);
+    DPoint parentPos;
+    if (m_pParent.expired()) {
+        parentPos = thisPos;
+    } else {
+        parentPos = m_pParent.lock()->getAbsPos(thisPos);
+    }
+    return parentPos;
 }
 
 void Node::setMouseEventCapture()
@@ -235,20 +366,25 @@ void Node::setEventCapture(int cursorID) {
 }
 
 void Node::releaseEventCapture(int cursorID) {
-    m_pPlayer->releaseEventCapture(m_This, cursorID);
+    m_pPlayer->releaseEventCapture(cursorID);
 }
 
-void Node::setEventHandler(Event::Type Type, Event::Source Source, PyObject * pFunc)
+void Node::setEventHandler(Event::Type Type, int Sources, PyObject * pFunc)
 {
-    EventHandlerID ID(Type, Source);
-    EventHandlerMap::iterator it = m_EventHandlerMap.find(ID);
-    if (it != m_EventHandlerMap.end()) {
-        Py_DECREF(it->second);
-        m_EventHandlerMap.erase(it);
-    }
-    if (pFunc != Py_None) {
-        Py_INCREF(pFunc);
-        m_EventHandlerMap[ID] = pFunc;
+    for (int i=0; i<4; ++i) {
+        int source = int(pow(2.,i));
+        if (source & Sources) {
+            EventHandlerID ID(Type, (Event::Source)source);
+            EventHandlerMap::iterator it = m_EventHandlerMap.find(ID);
+            if (it != m_EventHandlerMap.end()) {
+                Py_DECREF(it->second);
+                m_EventHandlerMap.erase(it);
+            }
+            if (pFunc != Py_None) {
+                Py_INCREF(pFunc);
+                m_EventHandlerMap[ID] = pFunc;
+            }
+        }
     }
 }
 
@@ -264,42 +400,32 @@ bool Node::reactsToMouseEvents()
 
 NodePtr Node::getElementByPos (const DPoint & pos)
 {
-    if (getVisibleRect().Contains(pos) && reactsToMouseEvents()) {
+    DPoint relPos = toLocal(pos);
+    if (relPos.x >= 0 && relPos.y >= 0 && 
+            relPos.x < getSize().x && relPos.y < getSize().y &&
+            reactsToMouseEvents())
+    {
         return m_This.lock();
     } else {
         return NodePtr();
     }
 }
 
-void Node::prepareRender (int time, const DRect& parent)
-{
-    calcAbsViewport();
-}
-
 void Node::maybeRender (const DRect& Rect)
 {
+    assert(getState() == NS_CANRENDER);
     if (m_bActive) {
-        bool bVisible;
-        // FIXME
-        if (dynamic_cast<DivNode*>(this) != 0) {
-            bVisible = getEngine()->pushClipRect(getVisibleRect(), true);
-        } else {
-            bVisible = getEngine()->pushClipRect(getVisibleRect(), false);
-        }
-        if (bVisible && getEffectiveOpacity() > 0.01) {
-            if (!getParent() || !getParent()->obscures(getEngine()->getClipRect(), 
-                    getParent()->indexOf(m_This.lock())))
-            {
-                if (m_ID != "") {
-                    AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr() << 
-                            " with ID " << m_ID);
-                } else {
-                    AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr()); 
-                }
-                render(Rect);
+        if (getEffectiveOpacity() > 0.01) {
+            if (m_ID != "") {
+                AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr() << 
+                        " with ID " << m_ID);
+            } else {
+                AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr()); 
             }
+            getDisplayEngine()->pushTransform(getRelViewport().tl, getAngle(), getPivot());
+            render(Rect);
+            getDisplayEngine()->popTransform();
         }
-        getEngine()->popClipRect();
     }
 }
 
@@ -307,21 +433,34 @@ void Node::render (const DRect& Rect)
 {
 }
 
-bool Node::obscures (const DRect& Rect, int Child)  
-{
-    return false;
-}
-
 Node::NodeState Node::getState() const
 {
     return m_State;
 }
 
-bool Node::isDisplayAvailable() const
+bool Node::operator ==(const Node& other) const
 {
-    return (getState() == NS_CONNECTED) && m_pEngine;
+    return m_This.lock() == other.m_This.lock();
 }
 
+bool Node::operator !=(const Node& other) const
+{
+    return m_This.lock() != other.m_This.lock();
+}
+
+long Node::getHash() const
+{
+    return long(&*m_This.lock());
+}
+
+DPoint Node::getPivot() const
+{
+    if (m_bHasCustomPivot) {
+        return m_Pivot;
+    } else {
+        return getSize()/2;
+    }
+}
 
 Player * Node::getPlayer() const
 {
@@ -336,7 +475,7 @@ void Node::setViewport (double x, double y, double width, double height)
     if (y == -32767) {
         y = getRelViewport().tl.y;
     }
-    DPoint MediaSize = getPreferredMediaSize();
+    IntPoint MediaSize = getMediaSize();
     if (width == -32767) {
         if (m_WantedSize.x == 0.0) {
             width = MediaSize.x;
@@ -352,7 +491,6 @@ void Node::setViewport (double x, double y, double width, double height)
         } 
     }
     m_RelViewport = DRect (x, y, x+width, y+height);
-    calcAbsViewport();
 }
 
 const DRect& Node::getRelViewport () const
@@ -362,37 +500,6 @@ const DRect& Node::getRelViewport () const
     return m_RelViewport;
 }
 
-const DRect& Node::getAbsViewport () const
-{
-    return m_AbsViewport;
-}
-
-DRect Node::getVisibleRect() const
-{
-    DRect visRect = getAbsViewport();
-    NodePtr pParent = getParent();
-    if (pParent) {
-        DRect parent = pParent->getVisibleRect();
-        visRect.Intersect(parent);
-    }
-    return visRect;
-}
-
-void Node::calcAbsViewport()
-{
-    NodePtr pParent = getParent();
-    if (pParent) {
-        DRect parent = pParent->getAbsViewport();
-        m_AbsViewport = DRect(parent.tl+getRelViewport().tl, 
-                parent.tl+getRelViewport().br);
-    } else {
-        m_AbsViewport = getRelViewport();
-    }
-    if (m_AbsViewport.Width() < 0 || m_AbsViewport.Height() < 0) {
-        m_AbsViewport.br=m_AbsViewport.tl;
-    }
-}
-
 double Node::getEffectiveOpacity()
 {
     if (getParent()) {
@@ -402,9 +509,14 @@ double Node::getEffectiveOpacity()
     }
 }
 
-DisplayEngine * Node::getEngine() const
+DisplayEngine * Node::getDisplayEngine() const
 {
-    return m_pEngine;
+    return m_pDisplayEngine;
+}
+
+AudioEngine * Node::getAudioEngine() const
+{
+    return m_pAudioEngine;
 }
 
 NodePtr Node::getThis() const
@@ -418,11 +530,7 @@ string Node::dump (int indent)
     char sz[256];
     sprintf (sz, ", x=%.1f, y=%.1f, width=%.1f, height=%.1f, opacity=%.2f\n",
             m_RelViewport.tl.x, m_RelViewport.tl.y,
-            m_RelViewport.Width(), m_RelViewport.Height(), m_Opacity);
-    dumpStr += sz;
-    sprintf (sz, "        Abs: (x=%.1f, y=%.1f, width=%.1f, height=%.1f)\n",
-            m_AbsViewport.tl.x, m_AbsViewport.tl.y,  
-            m_AbsViewport.Width(), m_AbsViewport.Height());
+            m_RelViewport.width(), m_RelViewport.height(), m_Opacity);
     dumpStr += sz;
 
     return dumpStr; 
@@ -447,22 +555,18 @@ bool Node::EventHandlerID::operator < (const EventHandlerID& other) const {
     }
 }
 
-void Node::handleEvent (Event* pEvent)
+void Node::handleEvent (EventPtr pEvent)
 {
     EventHandlerID ID(pEvent->getType(), pEvent->getSource());
     EventHandlerMap::iterator it = m_EventHandlerMap.find(ID);
     if (it!=m_EventHandlerMap.end()) {
-        pEvent->setElement(m_This.lock());
         callPython(it->second, pEvent);
     }
-    if (getParent()) {
-        getParent()->handleEvent (pEvent);
-    }
 }
 
-void Node::callPython (PyObject * pFunc, Event *pEvent)
+void Node::callPython (PyObject * pFunc, EventPtr pEvent)
 {
-    boost::python::call<void>(pFunc, boost::python::ptr(pEvent));
+    boost::python::call<void>(pFunc, pEvent);
 }
 
 PyObject * Node::findPythonFunc(const string& Code)
@@ -506,15 +610,25 @@ void Node::addEventHandler(Event::Type EventType, Event::Source Source,
 
 void Node::initFilename(Player * pPlayer, string& sFilename)
 {
-    if (sFilename[0] != '/') {
-        sFilename = pPlayer->getCurDirName() + sFilename;
+    bool bAbsDir = sFilename[0] == '/';
+#ifdef _WIN32
+    if (!bAbsDir) {
+        bAbsDir = (sFilename[0] == '\\' || sFilename[1] == ':');
+    }
+#endif
+    if (!bAbsDir) {
+        if (m_pParent.expired()) {
+            sFilename = pPlayer->getRootMediaDir()+sFilename;
+        } else {
+            sFilename = m_pParent.lock()->getEffectiveMediaDir()+sFilename;
+        }
     }
 }
 
 void Node::setState(Node::NodeState State)
 {
-/*
-    cerr << m_ID << "state: ";
+/*    
+    cerr << m_ID << " state: ";
     switch(State) {
         case NS_UNCONNECTED:
             cerr << "unconnected" << endl;
@@ -522,13 +636,31 @@ void Node::setState(Node::NodeState State)
         case NS_CONNECTED:
             cerr << "connected" << endl;
             break;
-        case NS_DISABLED:
-            cerr << "disabled" << endl;
+        case NS_CANRENDER:
+            cerr << "canrender" << endl;
             break;
     }
 */
+    if (m_State == NS_UNCONNECTED) {
+        assert(State != NS_CANRENDER);
+    }
+    if (m_State == NS_CANRENDER) {
+        assert(State != NS_CONNECTED);
+    }
+
     m_State = State;
 }
 
+DPoint Node::toLocal(const DPoint& globalPos) const
+{
+    DPoint localPos = globalPos-m_RelViewport.tl;
+    return rotate(localPos, -m_Angle, getPivot());
 }
 
+DPoint Node::toGlobal(const DPoint& localPos) const
+{
+    DPoint globalPos = rotate(localPos, m_Angle, getPivot());
+    return globalPos+m_RelViewport.tl;
+}
+
+}
