@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include "Region.h"
 #include "Event.h"
 #include "ISurface.h"
+#include "ArgList.h"
 
 #include "../base/Point.h"
 #include "../base/Rect.h"
@@ -34,8 +35,8 @@
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
 
-#include <Python.h>
-#undef HAVE_STAT
+// Python docs say python.h should be included before any standard headers (!)
+#include "WrapPython.h" 
 
 #include <vector>
 #include <string>
@@ -50,6 +51,8 @@ class Region;
 class DisplayEngine;
 class Player;
 class OGLSurface;
+class NodeDefinition;
+class AudioEngine;
 
 typedef boost::shared_ptr<Node> NodePtr;
 typedef boost::weak_ptr<Node> NodeWeakPtr;
@@ -61,15 +64,25 @@ typedef boost::weak_ptr<AVGNode> AVGNodeWeakPtr;
 class Node
 {
     public:
-        enum NodeState {NS_UNCONNECTED, NS_CONNECTED};
+        enum NodeState {NS_UNCONNECTED, NS_CONNECTED, NS_CANRENDER};
         
-        virtual ~Node () = 0;
+        template<class NodeType>
+        static NodePtr buildNode(const ArgList& Args, Player* pPlayer, bool bFromXML)
+        {
+            return NodePtr(new NodeType(Args, pPlayer, bFromXML));
+        }
+        static NodeDefinition getNodeDefinition();
+        
+        virtual ~Node() = 0;
         virtual void setThis(NodeWeakPtr This);
-        void setParent(DivNodeWeakPtr pParent);
-        virtual void setDisplayEngine(DisplayEngine * pEngine);
+        virtual void setArgs(const ArgList& Args);
+        virtual void setParent(DivNodeWeakPtr pParent, NodeState parentState);
+        void removeParent();
+        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, AudioEngine * pAudioEngine);
+        virtual void connect();
         virtual void disconnect();
         
-        virtual const std::string& getID () const;
+        virtual const std::string& getID() const;
         void setID(const std::string& ID);
 
         double getX() const;
@@ -77,12 +90,27 @@ class Node
         
         double getY() const;
         void setY(double Y);
-        
-        double getWidth() const;
+
+        const DPoint& getPos() const;
+        void setPos(const DPoint& pt);
+
+        virtual double getWidth();
         void setWidth(double width);
         
-        double getHeight() const;
+        virtual double getHeight();
         void setHeight(double height);
+       
+        DPoint getSize() const;
+        void setSize(const DPoint& pt);
+
+        double getAngle() const;
+        void setAngle(double Angle);
+        
+        double getPivotX() const;
+        void setPivotX(double Pivotx);
+        
+        double getPivotY() const;
+        void setPivotY(double Pivoty);
         
         double getOpacity() const;
         void setOpacity(double opacity);
@@ -94,62 +122,67 @@ class Node
         void setSensitive(bool bSensitive);
 
         virtual DivNodePtr getParent() const;
+        void unlink();
+
         DPoint getRelPos(const DPoint& AbsPos) const;
+        DPoint getAbsPos(const DPoint& RelPos) const;
 
         void setMouseEventCapture();
         void releaseMouseEventCapture();
         void setEventCapture(int cursorID);
         void releaseEventCapture(int cursorID);
-        void setEventHandler(Event::Type Type, Event::Source Source, PyObject * pFunc);
+        void setEventHandler(Event::Type Type, int Sources, PyObject * pFunc);
 
         bool isActive();
         bool reactsToMouseEvents();
         virtual NodePtr getElementByPos (const DPoint & pos);
-        virtual void prepareRender (int time, const DRect& parent);
+        virtual void preRender() {};
         virtual void maybeRender (const DRect& Rect);
         virtual void render (const DRect& Rect);
-        virtual bool obscures (const DRect& Rect, int Child);
         virtual void setViewport (double x, double y, double width, 
                 double height);
         virtual const DRect& getRelViewport () const;
-        virtual const DRect& getAbsViewport () const;
-        DRect getVisibleRect() const;
         virtual double getEffectiveOpacity();
 
         virtual std::string dump (int indent = 0);
         virtual std::string getTypeStr () const;
         
-        virtual void handleEvent (Event* pEvent); 
+        virtual void handleEvent (EventPtr pEvent); 
         NodeState getState() const;
-        bool isDisplayAvailable() const;
-        
-        // TODO: Do we still need this? Isn't rtti good enough?
-        enum {NT_UNKNOWN, NT_IMAGE, NT_AVG, NT_VIDEO, NT_TEXT, NT_EXCL, 
-                NT_CAMERA, NT_DIV, NT_PANOIMAGE};
+        virtual void checkReload() {};
+
+        bool operator ==(const Node& other) const;
+        bool operator !=(const Node& other) const;
+
+        long getHash() const;
+        virtual IntPoint getMediaSize() 
+            { return IntPoint(0,0); };
 
     protected:
-        Node (const xmlNodePtr xmlNode, Player * pPlayer);
-        virtual DPoint getPreferredMediaSize() 
-            { return DPoint(0,0); };
+        Node (Player * pPlayer);
+        DPoint getPivot() const;
         Player * getPlayer() const;
-        DisplayEngine * getEngine() const;
+        DisplayEngine * getDisplayEngine() const;
+        AudioEngine * getAudioEngine() const;
         NodePtr getThis() const;
 
-        void callPython (PyObject * pFunc, avg::Event* pEvent);
+        void callPython (PyObject * pFunc, avg::EventPtr pEvent);
         void addEventHandlers(Event::Type EventType, const std::string& Code);
         void addEventHandler(Event::Type EventType, Event::Source Source, 
                 const std::string& Code);
             
         void initFilename (Player * pPlayer, std::string& sFilename);
         void setState(NodeState State);
+        DPoint toLocal(const DPoint& pos) const;
+        DPoint toGlobal(const DPoint& pos) const;
  
     private:
-        void calcAbsViewport();
         PyObject * findPythonFunc(const std::string& Code);
 
         DivNodeWeakPtr m_pParent;
         NodeWeakPtr m_This;
-        DisplayEngine * m_pEngine;
+        DisplayEngine * m_pDisplayEngine;
+        AudioEngine * m_pAudioEngine;
         Player * m_pPlayer;
 
         std::string m_ID;
@@ -166,10 +199,12 @@ class Node
         EventHandlerMap m_EventHandlerMap;
 
         DRect m_RelViewport;      // In coordinates relative to the parent.
-        DRect m_AbsViewport;      // In window coordinates.
         double m_Opacity;
         bool m_bActive;
         bool m_bSensitive;
+        double m_Angle;
+        DPoint m_Pivot;
+        bool m_bHasCustomPivot;
         
         // Size specified by user.
         DPoint m_WantedSize;
@@ -180,4 +215,3 @@ class Node
 }
 
 #endif //_Node_H_
-
