@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,10 +20,10 @@
 //
 
 #include "OGLSurface.h"
-#include "OGLHelper.h"
 #include "Player.h"
-#include "MathHelper.h"
+#include "SDLDisplayEngine.h"
 
+#include "../base/MathHelper.h"
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/ScopeTimer.h"
@@ -39,46 +39,36 @@ namespace avg {
 
 OGLSurface::OGLSurface(SDLDisplayEngine * pEngine)
     : m_pEngine(pEngine),
+      m_bCreated(false),
       m_bBound(false),
       m_Size(-1,-1),
-      m_MaxTileSize(-1,-1),
-      m_NumHorizTextures(-1),
-      m_NumVertTextures(-1)
+      m_NumTextures(-1, -1),
+      m_MaxTileSize(-1,-1)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
-    // Do an NVIDIA texture support query if it hasn't happened already.
-//    getTextureMode();
 }
 
 OGLSurface::~OGLSurface()
 {
-    unbind();
-    switch(m_MemoryMode) {
-        case PBO:
-            if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
-                for (int i=0; i<3; i++) {
-                    glproc::DeleteBuffers(1, &m_hPixelBuffers[i]);
-                }
-            } else {
-                glproc::DeleteBuffers(1, &m_hPixelBuffers[0]);
-            }
-            OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-                    "OGLSurface::~OGLSurface: glDeleteBuffers()");
-            break;
-        default:
-            break;
+    if (m_bCreated) {
+        unbind();
+        deleteBuffers();
     }
+    m_pEngine->deregisterSurface(this);
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void OGLSurface::create(const IntPoint& Size, PixelFormat pf, bool bFastDownload)
 {
 //    cerr << "create: " << Size << ", " << Bitmap::getPixelFormatString(pf) << endl;
-    
     if (m_bBound && m_Size == Size && m_pf == pf) {
         // If nothing's changed, we can ignore everything.
         return;
     }
+    if (m_bCreated) {
+        unbind();
+        deleteBuffers();
+    }
     m_Size = Size;
     m_pf = pf;
     m_MemoryMode = OGL;
@@ -94,21 +84,9 @@ void OGLSurface::create(const IntPoint& Size, PixelFormat pf, bool bFastDownload
         createBitmap(Size, m_pf, 0);
     }
         
-    unbind();
-    setupTiles();
+    calcTileSizes();
     initTileVertices(m_TileVertices);
-}
-
-void OGLSurface::createFromBits(IntPoint Size, PixelFormat pf,
-        unsigned char * pBits, int Stride)
-{
-    unbind();
-    m_MemoryMode = OGL;
-    m_Size = Size;
-    m_pf = pf;
-    m_pBmps[0] = BitmapPtr(new Bitmap(Size, pf, pBits, Stride, false, ""));
-    
-    setupTiles();
+    m_bCreated = true;
 }
 
 BitmapPtr OGLSurface::lockBmp(int i)
@@ -139,6 +117,18 @@ BitmapPtr OGLSurface::lockBmp(int i)
                 } else {
                     pf = m_pf;
                 }
+#ifdef __APPLE__
+                if ( m_pf == I8 || m_pf == YCbCr420p || m_pf == YCbCrJ420p || 
+                        m_pf == YCbCr422) 
+                {
+                    // Under Mac OS X 10.5.0 and 10.5.1, the combination of 
+                    // glTexSubImage2D, GL_ALPHA and PBO is broken if pStartPos is
+                    // 0. So we use an offset. There's corresponding code in
+                    // OGLTile that undoes this... bleagh.
+                    pBuffer += 4;
+                }
+#endif
+
                 m_pBmps[i] = BitmapPtr(new Bitmap(Size, pf, pBuffer, 
                         Size.x*Bitmap::getBytesPerPixel(pf), false));
             }
@@ -175,7 +165,7 @@ void OGLSurface::setMaxTileSize(const IntPoint& MaxTileSize)
         m_MaxTileSize.y = nextpow2(m_MaxTileSize.y/2+1);
     }
     if (m_pBmps[0]) {
-        setupTiles();
+        calcTileSizes();
         initTileVertices(m_TileVertices);
     }
 }
@@ -204,11 +194,11 @@ void OGLSurface::setWarpedVertexCoords(const VertexGrid& Grid)
         bind();
     }
     bool bGridOK = true;
-    if (Grid.size() != (unsigned)(m_NumVertTextures+1)) {
+    if (Grid.size() != (unsigned)(m_NumTiles.y+1)) {
         bGridOK = false;
     }
     for (unsigned i = 0; i< Grid.size(); ++i) {
-        if (Grid[i].size() != (unsigned)(m_NumHorizTextures+1)) {
+        if (Grid[i].size() != (unsigned)(m_NumTiles.x+1)) {
             bGridOK = false;
         }
     }
@@ -232,8 +222,6 @@ string getGlModeString(int Mode)
             return "GL_BGR";
         case GL_BGRA:
             return "GL_BGRA";
-        case GL_YCBCR_MESA:
-            return "GL_YCBCR_MESA";
         case GL_YCBCR_422_APPLE:
             return "GL_YCBCR_422_APPLE";
         default:
@@ -248,32 +236,60 @@ void OGLSurface::bind()
     } else {
         int Width = m_Size.x;
         int Height = m_Size.y;
-        m_pTiles.clear();
-        vector<OGLTilePtr> v;
+        vector<vector<OGLTexturePtr> > pOldTextures = m_pTextures;
+        m_pTextures.clear();
+        vector<OGLTexturePtr> v;
         glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                 "OGLSurface::bind: glPixelStorei(GL_UNPACK_ALIGNMENT)");
         
-        for (int y=0; y<m_NumVertTextures; y++) {
-            m_pTiles.push_back(v);
-            for (int x=0; x<m_NumHorizTextures; x++) {
-                IntPoint CurSize = m_TileSize;
-                if (y == m_NumVertTextures-1) {
-                    CurSize.y = Height-y*m_TileSize.y;
+        for (int y=0; y<m_NumTextures.y; y++) {
+            m_pTextures.push_back(v);
+            for (int x=0; x<m_NumTextures.x; x++) {
+                IntPoint CurSize = m_TextureSize;
+                if (y == m_NumTextures.y-1) {
+                    CurSize.y = Height-y*m_TextureSize.y;
                 }
-                if (x == m_NumHorizTextures-1) {
-                    CurSize.x = Width-x*m_TileSize.x;
+                if (x == m_NumTextures.x-1) {
+                    CurSize.x = Width-x*m_TextureSize.x;
                 }
-                Rect<int> CurExtent(x*m_TileSize.x, y*m_TileSize.y,
-                        x*m_TileSize.x+CurSize.x, y*m_TileSize.y+CurSize.y);
+                Rect<int> CurExtent(x*m_TextureSize.x, y*m_TextureSize.y,
+                        x*m_TextureSize.x+CurSize.x, y*m_TextureSize.y+CurSize.y);
+                IntRect TileIndexExtents(
+                        safeCeil(double(CurExtent.tl.x)/m_TileSize.x), 
+                        safeCeil(double(CurExtent.tl.y)/m_TileSize.y),
+                        safeCeil(double(CurExtent.br.x)/m_TileSize.x), 
+                        safeCeil(double(CurExtent.br.y)/m_TileSize.y));
                 if (m_pEngine->getTextureMode() == GL_TEXTURE_2D) {
                     CurSize.x = nextpow2(CurSize.x);
                     CurSize.y = nextpow2(CurSize.y);
                 }
-                
-                OGLTilePtr pTile = OGLTilePtr(new OGLTile(CurExtent, CurSize,
-                        m_Size.x, m_pf, m_pEngine));
-                m_pTiles[y].push_back(pTile);
+/*
+                cerr << "Texture: " << x << ", " << y << endl;
+                cerr << "CurExtent: " << CurExtent << endl;
+                cerr << "m_TileSize: " << m_TileSize << endl;
+                cerr << "TileIndexExtents: " << TileIndexExtents << endl;
+*/                
+                OGLTexturePtr pTexture;
+                bool bReuseTexture;
+                if (int(pOldTextures.size()) != m_NumTextures.y || 
+                        int(pOldTextures[0].size()) != m_NumTextures.x) 
+                {
+                    bReuseTexture = false;
+                } else {
+                    bReuseTexture = 
+                            ((pOldTextures[y][x])->getTileIndexExtent() == TileIndexExtents &&
+                             (pOldTextures[y][x]->getPixelFormat() == m_pf));
+                }
+                if (bReuseTexture) {
+                    pTexture = pOldTextures[y][x];
+                    pTexture->resize(CurExtent, CurSize, m_TileSize, m_Size.x);
+                } else {
+                    pTexture = OGLTexturePtr(new OGLTexture(CurExtent, 
+                                CurSize, m_TileSize, TileIndexExtents, m_Size.x, m_pf,
+                                m_pEngine));
+                }
+                m_pTextures[y].push_back(pTexture);
                 if (m_MemoryMode == PBO) {
                     if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                         for (int i=0; i<3; i++) {
@@ -281,17 +297,17 @@ void OGLSurface::bind()
                                     m_hPixelBuffers[i]);
                             OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                                     "OGLSurface::bind: glBindBuffer()");
-                            pTile->downloadTexture(i, m_pBmps[i], m_Size.x, m_MemoryMode);
+                            pTexture->downloadTexture(i, m_pBmps[i], m_Size.x, m_MemoryMode);
                         }
                     } else {
                         glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_hPixelBuffers[0]);
                         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                                 "OGLSurface::bind: glBindBuffer()");
-                        pTile->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
+                        pTexture->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
                     }
                     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 0);
                 } else {
-                    pTile->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
+                    pTexture->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
                 }
             }
         }
@@ -307,9 +323,6 @@ void OGLSurface::bind()
 void OGLSurface::unbind() 
 {
 //    cerr << "OGLSurface::unbind()" << endl;
-    if (m_bBound) {
-        m_pTiles.clear();
-    }
     m_bBound = false;
 }
 
@@ -319,26 +332,26 @@ void OGLSurface::rebind()
     glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
             "AVGOGLSurface::rebind: glPixelStorei(GL_UNPACK_ALIGNMENT)");
-    for (unsigned int y=0; y<m_pTiles.size(); y++) {
-        for (unsigned int x=0; x<m_pTiles[y].size(); x++) {
-            OGLTilePtr pTile = m_pTiles[y][x];
+    for (unsigned int y=0; y<m_pTextures.size(); y++) {
+        for (unsigned int x=0; x<m_pTextures[y].size(); x++) {
+            OGLTexturePtr pTexture = m_pTextures[y][x];
             if (m_MemoryMode == PBO) {
                 if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                     for (int i=0; i<3; i++) {
                         glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_hPixelBuffers[i]);
                         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                                 "OGLSurface::rebind: glBindBuffer()");
-                        pTile->downloadTexture(i, m_pBmps[i], m_Size.x, m_MemoryMode);
+                        pTexture->downloadTexture(i, m_pBmps[i], m_Size.x, m_MemoryMode);
                     }
                 } else {
                     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_hPixelBuffers[0]);
                     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                             "OGLSurface::rebind: glBindBuffer()");
-                    pTile->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
+                    pTexture->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
                 }
                 glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 0);
             } else {
-                pTile->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
+                pTexture->downloadTexture(0, m_pBmps[0], m_Size.x, m_MemoryMode);
             }
         }
     }
@@ -359,80 +372,87 @@ IntPoint OGLSurface::getSize()
     return m_Size;
 }
 
-void OGLSurface::blt(const DRect* pDestRect, 
-        double angle, const DPoint& pivot, 
+void OGLSurface::blt(const DPoint& DestSize, 
         DisplayEngine::BlendMode Mode)
 {
 //    cerr << "OGLSurface::blt()" << endl;
     if (!m_bBound) {
         bind();
     }
-    bltTexture(pDestRect, angle, pivot, Mode);
+    bltTexture(DestSize, Mode);
 }
 
-bool OGLSurface::wouldTile(IntPoint Size)
+bool OGLSurface::isOneTexture(IntPoint Size)
 {
-    if (m_MaxTileSize.x != -1 || m_MaxTileSize.y != -1 ||
-        Size.x > m_pEngine->getMaxTexSize() || 
+    if (Size.x > m_pEngine->getMaxTexSize() || 
         Size.y > m_pEngine->getMaxTexSize() ||
         m_pEngine->getTextureMode() == GL_TEXTURE_2D)
     {
-        return true;
-    } else {
         return false;
+    } else {
+        return true;
     }
 }
 
-void OGLSurface::setupTiles()
+void OGLSurface::calcTileSizes()
 {
-    if (m_Size.x > m_pEngine->getMaxTexSize() || 
-        m_Size.y > m_pEngine->getMaxTexSize()) 
-    {
-        m_TileSize = IntPoint(m_pEngine->getMaxTexSize(), m_pEngine->getMaxTexSize());
-    } else {
-        if (m_pEngine->getTextureMode() == GL_TEXTURE_2D) {
-            if ((m_Size.x > 256 && nextpow2(m_Size.x) > m_Size.x*1.3) ||
-                    (m_Size.y > 256 && nextpow2(m_Size.y) > m_Size.y*1.3)) 
-            {
-                m_TileSize = IntPoint(nextpow2(m_Size.x)/2, nextpow2(m_Size.y)/2);
-            } else {
-                m_TileSize = IntPoint(nextpow2(m_Size.x), nextpow2(m_Size.y));
-            }
+    if (m_pEngine->getTextureMode() == GL_TEXTURE_2D) {
+        if ((m_Size.x > 256 && nextpow2(m_Size.x) > m_Size.x*1.3) ||
+                (m_Size.y > 256 && nextpow2(m_Size.y) > m_Size.y*1.3)) 
+        {
+            m_TextureSize = IntPoint(nextpow2(m_Size.x)/2, nextpow2(m_Size.y)/2);
         } else {
-            m_TileSize = m_Size;
+            m_TextureSize = IntPoint(nextpow2(m_Size.x), nextpow2(m_Size.y));
         }
+    } else {
+        m_TextureSize = m_Size;
     }
-    if (m_MaxTileSize.x != -1 && m_MaxTileSize.x < m_TileSize.x) {
+    if (m_Size.x > m_pEngine->getMaxTexSize()) {
+        m_TextureSize.x = m_pEngine->getMaxTexSize();
+    }
+    if (m_Size.y > m_pEngine->getMaxTexSize()) {
+        m_TextureSize.y = m_pEngine->getMaxTexSize();
+    }
+    m_NumTextures.x = safeCeil(double(m_Size.x)/m_TextureSize.x);
+    m_NumTextures.y = safeCeil(double(m_Size.y)/m_TextureSize.y);
+
+    m_TileSize = m_TextureSize;
+    if (m_MaxTileSize.x != -1 && m_MaxTileSize.x < m_TextureSize.x) {
         m_TileSize.x = m_MaxTileSize.x;
     }
-    if (m_MaxTileSize.y != -1 && m_MaxTileSize.y < m_TileSize.y) {
+    if (m_MaxTileSize.y != -1 && m_MaxTileSize.y < m_TextureSize.y) {
         m_TileSize.y = m_MaxTileSize.y;
     }
-    m_NumHorizTextures = int(ceil(float(m_Size.x)/m_TileSize.x));
-    m_NumVertTextures = int(ceil(float(m_Size.y)/m_TileSize.y));
-
+    m_NumTiles.x = safeCeil(double(m_Size.x)/m_TileSize.x);
+    m_NumTiles.y = safeCeil(double(m_Size.y)/m_TileSize.y);
+/*    
+    cerr << "----calcTileSizes: " << endl;
+    cerr << "TextureSize: " << m_TextureSize << ", NumTextures: " << m_NumTextures << endl;
+    cerr << "TileSize: " << m_TileSize << ", NumTiles: " << m_NumTiles << endl;
+*/    
 }
 
 void OGLSurface::initTileVertices(VertexGrid& Grid)
 {
-    std::vector<DPoint> TileVerticesLine(m_NumHorizTextures+1);
+    std::vector<DPoint> TileVerticesLine(m_NumTiles.x+1);
     Grid = std::vector<std::vector<DPoint> >
-                (m_NumVertTextures+1, TileVerticesLine);
+                (m_NumTiles.y+1, TileVerticesLine);
     for (unsigned int y=0; y<Grid.size(); y++) {
         for (unsigned int x=0; x<Grid[y].size(); x++) {
             initTileVertex(x, y, Grid[y][x]);
         }
     }
+    m_FinalVertices = std::vector<std::vector<DPoint> >(m_NumTiles.y+1, TileVerticesLine);
 }
 
 void OGLSurface::initTileVertex (int x, int y, DPoint& Vertex) 
 {
-    if (x < m_NumHorizTextures) {
+    if (x < m_NumTiles.x) {
         Vertex.x = double(m_TileSize.x*x) / m_Size.x;
     } else {
         Vertex.x = 1;
     }
-    if (y < m_NumVertTextures) {
+    if (y < m_NumTiles.y) {
         Vertex.y = double(m_TileSize.y*y) / m_Size.y;
     } else {
         Vertex.y = 1;
@@ -470,6 +490,26 @@ void OGLSurface::createBitmap(const IntPoint& Size, PixelFormat pf, int i)
     }
 }
 
+void OGLSurface::deleteBuffers()
+{
+    switch(m_MemoryMode) {
+        case PBO:
+            if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
+                for (int i=0; i<3; i++) {
+                    glproc::DeleteBuffers(1, &m_hPixelBuffers[i]);
+                }
+            } else {
+                glproc::DeleteBuffers(1, &m_hPixelBuffers[0]);
+            }
+            OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
+                    "OGLSurface::~OGLSurface: glDeleteBuffers()");
+            break;
+        default:
+            break;
+    }
+    m_bCreated = false;
+}
+
 void OGLSurface::unlockBmp(int i) 
 {
 //    cerr << "unlockBmp" << endl;
@@ -491,23 +531,9 @@ void OGLSurface::unlockBmp(int i)
     }
 }
 
-void OGLSurface::bltTexture(const DRect* pDestRect, 
-                double angle, const DPoint& pivot, 
-                DisplayEngine::BlendMode Mode)
+void OGLSurface::bltTexture(const DPoint& DestSize, 
+        DisplayEngine::BlendMode Mode)
 {
-    if (fabs(angle) > 0.001) {
-        DPoint center(pDestRect->tl.x+pivot.x,
-                pDestRect->tl.y+pivot.y);
-
-        glPushMatrix();
-        glTranslated(center.x, center.y, 0);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "bltTexture: glTranslated");
-        glRotated(angle*180.0/PI, 0, 0, 1);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "bltTexture: glRotated");
-        glTranslated(-center.x, -center.y, 0);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "bltTexture: glTranslated");
-    }
-
     switch(Mode) {
         case DisplayEngine::BLEND_BLEND:
             glproc::BlendEquation(GL_FUNC_ADD);
@@ -531,56 +557,58 @@ void OGLSurface::bltTexture(const DRect* pDestRect,
             break;
     }
 
-    for (unsigned int y=0; y<m_pTiles.size(); y++) {
-        for (unsigned int x=0; x<m_pTiles[y].size(); x++) {
-            DPoint TLPoint = 
-                    calcFinalVertex(pDestRect, m_TileVertices[y][x]);
-            DPoint TRPoint = 
-                    calcFinalVertex(pDestRect, m_TileVertices[y][x+1]);
-            DPoint BLPoint = 
-                    calcFinalVertex(pDestRect, m_TileVertices[y+1][x]);
-            DPoint BRPoint = 
-                    calcFinalVertex(pDestRect, m_TileVertices[y+1][x+1]);
-            OGLTilePtr pCurTile = m_pTiles[y][x];
-            pCurTile->blt(TLPoint, TRPoint, BLPoint, BRPoint); 
+    for (unsigned int y=0; y<m_FinalVertices.size(); y++) {
+        for (unsigned int x=0; x<m_FinalVertices[y].size(); x++) {
+            m_FinalVertices[y][x] = calcFinalVertex(DestSize, m_TileVertices[y][x]);
         }
     }
 
-    AVG_TRACE(Logger::BLTS, "(" << pDestRect->tl.x << ", " 
-            << pDestRect->tl.y << ")" << ", width:" << pDestRect->Width() 
-            << ", height: " << pDestRect->Height() << ", m_pf: " 
+    for (unsigned int y=0; y<m_pTextures.size(); y++) {
+        for (unsigned int x=0; x<m_pTextures[y].size(); x++) {
+            m_pTextures[y][x]->blt(&m_FinalVertices); 
+        }
+    }
+
+    AVG_TRACE(Logger::BLTS, "(" << DestSize.x << ", " 
+            << DestSize.y << ")" << ", m_pf: " 
             << Bitmap::getPixelFormatString(m_pf) << ", " 
             << getGlModeString(m_pEngine->getOGLSrcMode(m_pf)) << "-->" 
-            << getGlModeString(m_pEngine->getOGLDestMode(m_pf)) << endl);
-    if (fabs(angle) > 0.001) {
-        glPopMatrix();
-    }
+            << getGlModeString(m_pEngine->getOGLDestMode(m_pf)));
 }
 
-DPoint OGLSurface::calcFinalVertex(const DRect* pDestRect,
+DPoint OGLSurface::calcFinalVertex(const DPoint& Size,
         const DPoint & NormalizedVertex)
 {
     DPoint Point;
-    Point.x = pDestRect->tl.x+
-        (pDestRect->Width()*NormalizedVertex.x);
-    Point.y = pDestRect->tl.y+
-        (pDestRect->Height()*NormalizedVertex.y);
+    Point.x = Size.x*NormalizedVertex.x;
+    Point.y = Size.y*NormalizedVertex.y;
     return Point;
 }
 
-
-
-void OGLSurface::checkBlendModeError(string sMode) 
+void OGLSurface::checkBlendModeError(const char *mode) 
 {    
     GLenum err = glGetError();
     if (err != GL_NO_ERROR) {
         static bool bErrorReported = false;
         if (!bErrorReported) {
-            AVG_TRACE(Logger::WARNING, "Blendmode "+sMode+
+            AVG_TRACE(Logger::WARNING, "Blendmode "<<mode<<
                     " not supported by OpenGL implementation.");
             bErrorReported = true;
         }
     }
 }
 
+int OGLSurface::getTotalTexMemory()
+{
+    int iAmount = 0;
+    if (m_bBound) {
+        for (int y=0; y<m_NumTextures.y; y++) {
+            for (int x=0; x<m_NumTextures.x; x++) {
+                iAmount += m_pTextures[y][x]->getTexMemDim();
+            }
+        }
+    }
+    return iAmount;
+}
+
 }
