@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,20 +23,20 @@
 #define _OGLSurface_H_
 
 #include "ISurface.h"
-#include "OGLTile.h"
-#include "OGLHelper.h"
-#include "OGLShader.h"
-#include "SDLDisplayEngine.h"
+#include "DisplayEngine.h"
+#include "OGLTexture.h"
 
 #include "../base/Point.h"
 #include "../base/Rect.h"
 
+#include "../graphics/OGLHelper.h"
+
 #include <vector>
 #include <string>
 
 namespace avg {
 
-typedef std::vector<std::vector<DPoint> > VertexGrid;
+class SDLDisplayEngine;
 
 class OGLSurface: public ISurface {
     public:
@@ -49,10 +49,6 @@ class OGLSurface: public ISurface {
         virtual BitmapPtr lockBmp(int index=0);
         virtual void unlockBmps();
 
-        // Methods specific to OGLSurface
-        void createFromBits(Point<int> Size, PixelFormat pf,
-                unsigned char * pBits, int Stride);
-
         void bind();
         void unbind();
         void rebind();
@@ -60,57 +56,53 @@ class OGLSurface: public ISurface {
         PixelFormat getPixelFormat();
         IntPoint getSize();
 
-        void blt(const DRect* pDestRect,
-                double angle, const DPoint& pivot, 
-                DisplayEngine::BlendMode Mode);
+        void blt(const DPoint& DestSize, DisplayEngine::BlendMode Mode);
 
-        void setMaxTileSize(const Point<int>& MaxTileSize);
+        void setMaxTileSize(const IntPoint& MaxTileSize);
         VertexGrid getOrigVertexCoords();
         VertexGrid getWarpedVertexCoords();
         void setWarpedVertexCoords(const VertexGrid& Grid);
  
-        bool wouldTile(IntPoint Size);
+        bool isOneTexture(IntPoint Size);
+        
+        int getTotalTexMemory();
 
     private:
-        void setupTiles();
+        void calcTileSizes();
         void initTileVertices(VertexGrid& Grid);
         void initTileVertex (int x, int y, DPoint& Vertex);
 
         void createBitmap(const IntPoint& Size, PixelFormat pf, int index);
+        void deleteBuffers();
         void unlockBmp(int i);
-        void bindOneTexture(OGLTile& Tile);
-        void bltTexture(const DRect* pDestRect, double angle, const DPoint& pivot, 
-                DisplayEngine::BlendMode Mode);
-        DPoint calcFinalVertex(const DRect* pDestRect,
+        void bindOneTexture(OGLTexture& Texture);
+        void bltTexture(const DPoint& DestSize, DisplayEngine::BlendMode Mode);
+        DPoint calcFinalVertex(const DPoint& Size,
                 const DPoint & NormalizedVertex);
-        void checkBlendModeError(std::string sMode);
+        void checkBlendModeError(const char * mode);
 
         SDLDisplayEngine * m_pEngine;
-        
+       
+        bool m_bCreated;
         bool m_bBound;
 
         BitmapPtr m_pBmps[3];
         IntPoint m_Size;
         PixelFormat m_pf;
 
-        Point<int> m_MaxTileSize;
-        Point<int> m_TileSize;
-        int m_NumHorizTextures;
-        int m_NumVertTextures;
-        std::vector<std::vector<OGLTilePtr> > m_pTiles;
+        IntPoint m_TextureSize;
+        IntPoint m_NumTextures;
+        IntPoint m_MaxTileSize;
+        IntPoint m_TileSize;
+        IntPoint m_NumTiles;
+        std::vector<std::vector<OGLTexturePtr> > m_pTextures;
         VertexGrid m_TileVertices;
+        VertexGrid m_FinalVertices;
 
         OGLMemoryMode m_MemoryMode;
 
         // PBO memory mode
         GLuint m_hPixelBuffers[3];
-/*
-#ifndef __APPLE__
-        void * m_pMESABuffer;
-        static PFNGLXALLOCATEMEMORYMESAPROC s_AllocMemMESAProc;
-        static PFNGLXFREEMEMORYMESAPROC s_FreeMemMESAProc;
-#endif
-*/
 };
 
 }
