@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,7 @@
 
 #include "Player.h"
 
-#include "../avgconfig.h"
-#include "avgdtd.h"
+#include "../avgconfigwrapper.h"
 #include "AVGNode.h"
 #include "DivNode.h"
 #include "Words.h"
@@ -30,6 +29,8 @@
 #include "CameraNode.h"
 #include "Image.h"
 #include "PanoImage.h"
+#include "Sound.h"
+#include "NodeDefinition.h"
 
 #include "TrackerEventSource.h"
 #include "Event.h"
@@ -39,6 +40,8 @@
 #include "SDLDisplayEngine.h"
 
 #include "../base/FileHelper.h"
+#include "../base/StringHelper.h"
+#include "../base/OSHelper.h"
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 #include "../base/ConfigMgr.h"
@@ -46,15 +49,19 @@
 #include "../base/Profiler.h"
 #include "../base/ScopeTimer.h"
 #include "../base/TimeSource.h"
-#include "../base/MemHelper.h"
-#include "../base/Rect.h"
 
 #include "../imaging/FWCamera.h"
 #ifdef AVG_ENABLE_V4L2
 #include "../imaging/V4LCamera.h"
 #endif
+#ifdef _WIN32
+#include "../imaging/DSCamera.h"
+#endif
 #include "../imaging/FakeCamera.h"
 
+#include "../audio/SDLAudioEngine.h"
+
+#undef HAVE_TEMPNAM
 #include <Magick++.h>
 
 #include <libxml/xmlmemory.h>
@@ -74,34 +81,59 @@ using namespace std;
 
 namespace avg {
 
+Player * Player::s_pPlayer=0;
+
 Player::Player()
     : m_pRootNode(),
       m_pDisplayEngine(0),
-      m_TestHelper(this),
+      m_pAudioEngine(0),
       m_pTracker(0),
       m_bInHandleTimers(false),
-      m_bCurrentTimeoutDeleted(false), 
-      m_pLastMouseNode(),
+      m_bCurrentTimeoutDeleted(false),
       m_pEventCaptureNode(),
       m_bUseFakeCamera(false),
+      m_MaxGPUMemUsed(0),
+      m_bStopOnEscape(true),
       m_bIsPlaying(false),
       m_bFakeFPS(false),
       m_FakeFPS(0),
       m_FrameTime(0),
-      m_PlayStartTime(0)
+      m_PlayStartTime(0),
+      m_Volume(1),
+      m_bPythonAvailable(true)
 {
+    if (s_pPlayer) {
+        throw Exception(AVG_ERR_UNKNOWN, "Player has already been instantiated.");
+    }
     ThreadProfilerPtr pThreadProfiler = ThreadProfilerPtr(new ThreadProfiler("Main"));
     Profiler::get().registerThreadProfiler(pThreadProfiler);
     initConfig();
+
+    // Register all node types
+    registerNodeType(AVGNode::getNodeDefinition());
+    registerNodeType(DivNode::getNodeDefinition());
+    registerNodeType(Image::getNodeDefinition());
+    registerNodeType(Words::getNodeDefinition());
+    registerNodeType(Video::getNodeDefinition());
+    registerNodeType(CameraNode::getNodeDefinition());
+    registerNodeType(PanoImage::getNodeDefinition());
+    registerNodeType(Sound::getNodeDefinition());
+
     // Find and parse dtd.
-    registerDTDEntityLoader("avg.dtd", g_pAVGDTD);
+    registerDTDEntityLoader("avg.dtd", m_NodeFactory.getDTD().c_str());
     string sDTDFName = "avg.dtd";
     m_dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
     if (!m_dtd) {
         AVG_TRACE(Logger::WARNING, 
                 "DTD not found at " << sDTDFName << ". Not validating xml files.");
     }
-    m_pLastMouseNode[MOUSECURSORID]=NodePtr();
+    m_pTestHelper = new TestHelper(this);
+
+#ifdef _WIN32
+    Magick::InitializeMagick((getAvgLibPath()+"magick\\").c_str());
+#endif
+
+    s_pPlayer = this;
 }
 
 Player::~Player()
@@ -109,16 +141,24 @@ Player::~Player()
     if (m_pDisplayEngine) {
         delete m_pDisplayEngine;
     }
-    if (m_pTracker) {
-        delete m_pTracker;
+    if (m_pAudioEngine) {
+        delete m_pAudioEngine;
     }
     if (m_dtd) {
         xmlFreeDtd(m_dtd);
     }
+    delete m_pTestHelper;
 }
 
-void Player::setResolution(bool bFullscreen, 
-                int width, int height, int bpp)
+Player* Player::get()
+{
+    if (!s_pPlayer) {
+        s_pPlayer = new Player();
+    }
+    return s_pPlayer;
+}
+
+void Player::setResolution(bool bFullscreen, int width, int height, int bpp)
 {
     m_DP.m_bFullscreen = bFullscreen;
     if (bpp) {
@@ -131,7 +171,13 @@ void Player::setResolution(bool bFullscreen,
         m_DP.m_WindowHeight = height;
     }
 }
-        
+
+void Player::setWindowPos(int x, int y)
+{
+    m_DP.m_x = x;
+    m_DP.m_y = y;
+}
+
 void Player::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
                 bool bUsePixelBuffers, int MultiSampleSamples)
 {
@@ -141,104 +187,94 @@ void Player::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode,
     m_MultiSampleSamples = MultiSampleSamples;
 }
 
-void Player::loadFile (const std::string& filename)
+void Player::setMultiSampleSamples(int MultiSampleSamples)
 {
+    m_MultiSampleSamples = MultiSampleSamples;
+}
+
+void Player::setAudioOptions(int samplerate, int channels)
+{
+    m_AP.m_SampleRate = samplerate;
+    m_AP.m_Channels = channels;
+}
+
+void Player::loadFile (const std::string& sFilename)
+{
+    string RealFilename;
     try {
-        m_pEventDispatcher = EventDispatcherPtr(new EventDispatcher);
         AVG_TRACE(Logger::MEMORY, 
-                std::string("Player::LoadFile(") + filename + ")");
-        if (m_pRootNode) {
-            cleanup();
-        }
-        assert (!m_pRootNode);
+                std::string("Player::loadFile(") + sFilename + ")");
 
         // When loading an avg file, assets are loaded from a directory relative
         // to the file.
         char szBuf[1024];
-        string RealFilename;
-        if (filename[0] == '/') {
-            RealFilename = filename; 
+        if (sFilename[0] == '/') {
+            RealFilename = sFilename; 
         } else {
             getcwd(szBuf, 1024);
             m_CurDirName = string(szBuf)+"/";
-            RealFilename = m_CurDirName+filename;
+            RealFilename = m_CurDirName+sFilename;
         }
         m_CurDirName = RealFilename.substr(0, RealFilename.rfind('/')+1);
-        
-        xmlPedanticParserDefault(1);
-        xmlDoValidityCheckingDefaultValue =0;
 
-        xmlDocPtr doc;
-        if (!fileExists(RealFilename)) {
-            throw (Exception(AVG_ERR_FILEIO, 
-                        string("File ")+RealFilename+" not found."));
-        }
-        doc = xmlParseFile(RealFilename.c_str());
-        if (!doc) {
-            throw (Exception(AVG_ERR_XML_PARSE, 
-                        string("Error parsing xml document ")+RealFilename));
-        }
-
-        xmlValidCtxtPtr cvp = xmlNewValidCtxt();
-        cvp->error = xmlParserValidityError;
-        cvp->warning = xmlParserValidityWarning;
-        int valid=xmlValidateDtd(cvp, doc, m_dtd);  
-        xmlFreeValidCtxt(cvp);
-        if (!valid) {
-            throw (Exception(AVG_ERR_XML_PARSE, 
-                    filename + " does not validate."));
-        }
-        xmlNodePtr xmlNode = xmlDocGetRootElement(doc);
-        m_pRootNode = boost::dynamic_pointer_cast<AVGNode>
-            (createNodeFromXml(doc, xmlNode, DivNodePtr()));
-        m_pRootNode->setParent(DivNodeWeakPtr());
-        m_DP.m_Height = getDefaultedIntAttr (xmlNode, "height", 0);
-        m_DP.m_Width = getDefaultedIntAttr (xmlNode, "width", 0);
-        registerNode(m_pRootNode);
+        string sAVG;
+        readWholeFile(RealFilename, sAVG);
+        internalLoad(sAVG);
 
         // Reset the directory to load assets from to the current dir.
         getcwd(szBuf, 1024);
         m_CurDirName = string(szBuf)+"/";
-        
-        xmlFreeDoc(doc);
     } catch (Exception& ex) {
-        AVG_TRACE(Logger::ERROR, ex.GetStr());
-        throw;
-    } catch (Magick::Exception & ex) {
-        AVG_TRACE(Logger::ERROR, ex.what());
-        throw;
+        switch (ex.GetCode()) {
+            case AVG_ERR_XML_PARSE:
+                throw (Exception(AVG_ERR_XML_PARSE, 
+                        string("Error parsing xml document ")+RealFilename));
+                break;
+            case AVG_ERR_XML_VALID:
+                throw (Exception(AVG_ERR_XML_VALID, 
+                        RealFilename + " does not validate."));
+                break;
+            default:
+                throw;
+        }
+    }
+}
+
+void Player::loadString(const std::string& sAVG)
+{
+    try {
+        AVG_TRACE(Logger::MEMORY, "Player::loadString()");
+        char szBuf[1024];
+        getcwd(szBuf, 1024);
+        m_CurDirName = string(szBuf)+"/";
+
+        string sEffectiveDoc = removeStartEndSpaces(sAVG);
+        internalLoad(sEffectiveDoc);
+    } catch (Exception& ex) {
+        switch (ex.GetCode()) {
+            case AVG_ERR_XML_PARSE:
+                throw Exception(AVG_ERR_XML_PARSE, "Error parsing xml string.");
+                break;
+            case AVG_ERR_XML_VALID:
+                throw Exception(AVG_ERR_XML_VALID, "Error validating xml string.");
+                break;
+            default:
+                throw;
+        }
     }
 }
 
 void Player::play()
 {
-    m_bIsPlaying = true;
     try {
-        if (!m_pRootNode) {
-            AVG_TRACE(Logger::ERROR, "play called, but no xml file loaded.");
-        }
-        assert(m_pRootNode);
-        initGraphics();
-        m_pRootNode->setDisplayEngine(m_pDisplayEngine);
-       
-        m_pEventDispatcher->addSource(m_pEventSource);
-        m_pEventDispatcher->addSource(&m_TestHelper);
-        m_pEventDispatcher->addSink(&m_EventDumper);
-        m_pEventDispatcher->addSink(this);
-        
-        m_pDisplayEngine->initRender();
-        m_bStopping = false;
-
-        m_PlayStartTime = TimeSource::get()->getCurrentMillisecs();
-        m_FrameTime = 0;
-        m_NumFrames = 0;
-        m_pDisplayEngine->render(m_pRootNode, true);
-        if (m_pDisplayEngine->wasFrameLate()) {
-            ThreadProfiler::get()->dumpFrame();
-        }
-        
-        ThreadProfiler::get()->start();
+        initPlayback();
         try {
+            m_pDisplayEngine->render(m_pRootNode);
+            if (m_pDisplayEngine->wasFrameLate()) {
+                ThreadProfiler::get()->dumpFrame();
+            }
+            ThreadProfiler::get()->start();
+
             while (!m_bStopping) {
                 doFrame();
             }
@@ -253,7 +289,6 @@ void Player::play()
         AVG_TRACE(Logger::ERROR, ex.GetStr());
         throw;
     }
-    m_bIsPlaying = false;
 }
 
 void Player::stop ()
@@ -264,6 +299,34 @@ void Player::stop ()
         cleanup();
     }
 }
+        
+void Player::initPlayback()
+{
+    m_bIsPlaying = true;
+    if (!m_pRootNode) {
+        throw Exception(AVG_ERR_NO_NODE, "Play called, but no xml file loaded.");
+    }
+    
+    initGraphics();
+    initAudio();
+    
+    m_pRootNode->setRenderingEngines(m_pDisplayEngine, m_pAudioEngine);
+
+    m_pEventDispatcher->addSource(m_pEventSource);
+    m_pEventDispatcher->addSource(m_pTestHelper);
+    m_pEventDispatcher->addSink(this);
+
+    m_pDisplayEngine->initRender();
+    m_bStopping = false;
+    if (m_pTracker) {
+        m_pTracker->start();
+    }
+
+    m_PlayStartTime = TimeSource::get()->getCurrentMillisecs();
+    m_FrameTime = 0;
+    m_NumFrames = 0;
+    m_MaxGPUMemUsed = 0;
+}
 
 bool Player::isPlaying()
 {
@@ -298,14 +361,9 @@ double Player::getEffectiveFramerate() {
     }
 }
 
-unsigned Player::getMemUsed()
-{
-    return avg::getMemUsed();
-}
-
 TestHelper * Player::getTestHelper()
 {
-    return &m_TestHelper;
+    return m_pTestHelper;
 }
 
 void Player::setFakeFPS(double fps)
@@ -317,6 +375,10 @@ void Player::setFakeFPS(double fps)
         m_bFakeFPS = true;
         m_FakeFPS = fps;
     }
+
+    if (m_pAudioEngine) {
+        m_pAudioEngine->setAudioEnabled(!m_bFakeFPS);
+    }
 }
 
 long long Player::getFrameTime()
@@ -324,37 +386,62 @@ long long Player::getFrameTime()
     return m_FrameTime;
 }
 
-TrackerEventSource * Player::addTracker()
+TrackerEventSource * Player::addTracker(const string& sConfigFilename)
 {
     TrackerConfig Config;
-    Config.load();
+    Config.load(sConfigFilename);
     CameraPtr pCamera;
-    
-    if (Config.m_sSource == "v4l") {
+
+    string sSource = Config.getParam("/camera/source/@value");
+    string sDevice = Config.getParam("/camera/device/@value");
+    IntPoint Size(Config.getPointParam("/camera/size/"));
+    string sPixFmt = Config.getParam("/camera/format/@value");
+    double FPS = Config.getDoubleParam("/camera/fps/@value");
+
+    if (!m_pEventDispatcher) {
+        throw Exception(AVG_ERR_UNSUPPORTED, "You must use loadFile() before addTracker().");
+    }
+
+    if (sSource == "v4l") {
 #ifdef AVG_ENABLE_V4L2
-        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for V4L camera " <<
-                Config.m_sDevice << " width=" << Config.m_Size.x << " height=" <<
-                Config.m_Size.y << " channel=" << Config.m_Channel);
+        int Channel = Config.getIntParam("/camera/channel/@value");
+        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for V4L camera " << sDevice
+                << " size=" << Size << " channel=" << Channel << " format=" 
+                << sPixFmt);
         
-        // is there a need to get a configurable pixel format for tracker?
-        pCamera = CameraPtr(new V4LCamera(Config.m_sDevice, Config.m_Channel, 
-                Config.m_Size, "MONO8", false));
+        pCamera = CameraPtr(new V4LCamera(sDevice, Channel, Size, sPixFmt, false));
 #else
         AVG_TRACE(Logger::ERROR, "Video4Linux camera tracker requested, but " 
                 "Video4Linux support not compiled in.");
         exit(1);
 #endif
-    } else {
+    } else if (sSource == "fw") {
         AVG_TRACE(Logger::CONFIG, "Adding a Tracker for FW camera " << 
-                Config.m_sDevice << " width=" << Config.m_Size.x << 
-                " height=" << Config.m_Size.y);
-        pCamera = CameraPtr(new FWCamera(Config.m_sDevice, Config.m_Size, 
-                "MONO8", Config.m_FPS, false));
+                sDevice << " size=" << Size << " format=" << sPixFmt);
+        pCamera = CameraPtr(new FWCamera(sDevice, Size, sPixFmt, FPS, false));
+    } else if (sSource == "ds") {
+#ifdef _WIN32        
+        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for DS camera " << 
+                sDevice << " size=" << Size << " format=" << sPixFmt);
+        pCamera = CameraPtr(new DSCamera(sDevice, Size, sPixFmt, FPS, false));
+#else
+        AVG_TRACE(Logger::ERROR, "DS camera tracker requested, but " 
+                "DS support not compiled in.");
+        exit(1);
+#endif        
+    } else {
+        throw Exception(AVG_ERR_INVALID_CAPTURE,
+                string("Invalid camera source value '")+sSource+
+                        "'. Can't initialize tracker.");
     }
     
     m_pTracker = new TrackerEventSource(pCamera, Config, 
             IntPoint(m_DP.m_Width, m_DP.m_Height), true);
     m_pEventDispatcher->addSource(m_pTracker);
+    if (m_bIsPlaying) {
+        m_pTracker->start();
+    }
+
     return m_pTracker;
 }
 
@@ -408,9 +495,9 @@ bool Player::clearInterval(int id)
     return false;
 }
 
-const MouseEvent& Player::getMouseState() const
+MouseEventPtr Player::getMouseState() const
 {
-    return m_pEventDispatcher->getLastMouseEvent();
+    return m_MouseState.getLastEvent();
 }
 
 Bitmap * Player::screenshot()
@@ -431,14 +518,15 @@ void Player::showCursor(bool bShow)
 void Player::setEventCapture(NodeWeakPtr pNode, int cursorID) {
     std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
     if (it!=m_pEventCaptureNode.end()&&!it->second.expired()) {
-        throw Exception(AVG_ERR_INVALID_CAPTURE,
-                "setEventCapture called, but cursor already captured.");
+        throw Exception(AVG_ERR_INVALID_CAPTURE, "setEventCapture called for '"
+                + pNode.lock()->getID() + "', but cursor already captured by '"
+                + it->second.lock()->getID() + "'.");
     } else {
         m_pEventCaptureNode[cursorID] = pNode;
     }
 }
 
-void Player::releaseEventCapture(NodeWeakPtr pNode, int cursorID) {
+void Player::releaseEventCapture(int cursorID) {
     std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
     if(it==m_pEventCaptureNode.end()||(it->second.expired()) ) {
         throw Exception(AVG_ERR_INVALID_CAPTURE,
@@ -481,9 +569,9 @@ void Player::removeNodeID(const std::string& id)
         if (it != m_IDMap.end()) {
             m_IDMap.erase(it);
         } else {
-            AVG_TRACE(Logger::ERROR, "removeNodeID(\""+id+"\") failed.");
-            exit(1);
-      }
+            cerr << "removeNodeID(\"" << id << "\") failed." << endl;
+            assert(false);
+        }
     }
 }
 
@@ -512,6 +600,22 @@ string Player::getCurDirName()
 {
     return m_CurDirName;
 }
+        
+std::string Player::getRootMediaDir()
+{
+    string sMediaDir;
+    if (m_pRootNode) {
+        sMediaDir = m_pRootNode->getEffectiveMediaDir();
+    } else {
+        sMediaDir = m_CurDirName;
+    }
+    return sMediaDir;
+}
+
+void Player::disablePython()
+{
+    m_bPythonAvailable = false;
+}
 
 static ProfilingZone MainProfilingZone("Player - Total frame time");
 static ProfilingZone TimersProfilingZone("Player - handleTimers");
@@ -537,12 +641,20 @@ void Player::doFrame ()
         {
             ScopeTimer Timer(EventsProfilingZone);
             m_pEventDispatcher->dispatch();
+            sendFakeEvents();
         }
         if (!m_bStopping) {
-            Py_BEGIN_ALLOW_THREADS;
             ScopeTimer Timer(RenderProfilingZone);
-            m_pDisplayEngine->render(m_pRootNode, false);
-            Py_END_ALLOW_THREADS;
+            if (m_bPythonAvailable) {
+                Py_BEGIN_ALLOW_THREADS;
+                m_pDisplayEngine->render(m_pRootNode);
+                Py_END_ALLOW_THREADS;
+            } else {
+                m_pDisplayEngine->render(m_pRootNode);
+            }
+            if (getGPUMemoryUsage() > m_MaxGPUMemUsed) {
+                m_MaxGPUMemUsed = getGPUMemoryUsage();
+            }
         }
         {
             ScopeTimer Timer(ListenerProfilingZone);
@@ -622,6 +734,10 @@ void Player::initConfig() {
         exit(-1);
     }
 
+    m_AP.m_Channels = atoi(pMgr->getOption("aud", "channels")->c_str());
+    m_AP.m_SampleRate = atoi(pMgr->getOption("aud", "samplerate")->c_str());
+    m_AP.m_OutputBufferSamples = atoi(pMgr->getOption("aud", "outputbuffersamples")->c_str());
+
     m_bUsePOW2Textures = pMgr->getBoolOption("scr", "usepow2textures", false);
 
     const string * psYCbCrMode =pMgr->getOption("scr", "ycbcrmode");
@@ -717,9 +833,65 @@ void Player::initGraphics()
     m_pDisplayEngine->init(m_DP);
 }
 
+void Player::initAudio()
+{
+    if (!m_pAudioEngine) {
+        m_pAudioEngine = new SDLAudioEngine();
+    }
+    m_pAudioEngine->init(m_AP, m_Volume);
+    m_pAudioEngine->setAudioEnabled(!m_bFakeFPS);
+    m_pAudioEngine->play();
+}
+
+void Player::internalLoad(const string& sAVG)
+{
+    try {
+        if (m_pRootNode) {
+            cleanup();
+        }
+        assert (!m_pRootNode);
+        m_pEventDispatcher = EventDispatcherPtr(new EventDispatcher);
+
+        char szBuf[1024];
+        getcwd(szBuf, 1024);
+        m_CurDirName = string(szBuf)+"/";
+        
+        xmlPedanticParserDefault(1);
+        xmlDoValidityCheckingDefaultValue =0;
+
+        xmlDocPtr doc;
+        doc = xmlParseMemory(sAVG.c_str(), sAVG.length());
+        if (!doc) {
+            throw (Exception(AVG_ERR_XML_PARSE, ""));
+        }
+
+        xmlValidCtxtPtr cvp = xmlNewValidCtxt();
+        cvp->error = xmlParserValidityError;
+        cvp->warning = xmlParserValidityWarning;
+        int valid=xmlValidateDtd(cvp, doc, m_dtd);  
+        xmlFreeValidCtxt(cvp);
+        if (!valid) {
+            throw (Exception(AVG_ERR_XML_VALID, ""));
+        }
+        xmlNodePtr xmlNode = xmlDocGetRootElement(doc);
+        createNodeFromXml(doc, xmlNode, DivNodePtr());
+        registerNode(m_pRootNode);
+        m_DP.m_Height = int(m_pRootNode->getHeight());
+        m_DP.m_Width = int(m_pRootNode->getWidth());
+        
+        xmlFreeDoc(doc);
+    } catch (Exception& ex) {
+        AVG_TRACE(Logger::ERROR, ex.GetStr());
+        throw;
+    } catch (Magick::Exception & ex) {
+        AVG_TRACE(Logger::ERROR, ex.what());
+        throw;
+    }
+}
+
 void Player::registerNode(NodePtr pNode)
 {
-    addNodeID(pNode);
+    addNodeID(pNode);    
     DivNodePtr pDivNode = boost::dynamic_pointer_cast<DivNode>(pNode);
     if (pDivNode) {
         for (int i=0; i<pDivNode->getNumChildren(); i++) {
@@ -728,75 +900,72 @@ void Player::registerNode(NodePtr pNode)
     }
 }
 
+void Player::registerNodeType(NodeDefinition Def)
+{
+    m_NodeFactory.registerNodeType(Def);
+}
+
+NodePtr Player::createNode(const string& sType, const boost::python::dict& PyDict)
+{
+    NodePtr pNode = m_NodeFactory.createNode(sType, PyDict, this);
+    pNode->setThis(pNode);
+    return pNode;
+}
+
 NodePtr Player::createNodeFromXmlString (const string& sXML)
 {
-    try {
-        xmlPedanticParserDefault(1);
-        xmlDoValidityCheckingDefaultValue =0;
-        
-        xmlDocPtr doc;
-        doc = xmlParseMemory(sXML.c_str(), sXML.length());
-        if (!doc) {
-            throw (Exception(AVG_ERR_XML_PARSE, 
-                        string("Error parsing xml:\n  ")+sXML));
-        }
-        NodePtr pNode = createNodeFromXml(doc, xmlDocGetRootElement(doc), DivNodePtr());
+    xmlPedanticParserDefault(1);
+    xmlDoValidityCheckingDefaultValue =0;
 
-        xmlValidCtxtPtr cvp = xmlNewValidCtxt();
-        cvp->error = xmlParserValidityError;
-        cvp->warning = xmlParserValidityWarning;
-        int valid=xmlValidateDtd(cvp, doc, m_dtd);  
-        xmlFreeValidCtxt(cvp);
-        if (!valid) {
-            throw (Exception(AVG_ERR_XML_PARSE, 
-                    "Could not validate '"+sXML+"'"));
-        }
-
-        xmlFreeDoc(doc);
-        return pNode;
-    } catch (Exception& ex) {
-        AVG_TRACE(Logger::ERROR, ex.GetStr());
-        return NodePtr();
-    } catch (Magick::Exception& ex) {
-        AVG_TRACE(Logger::ERROR, ex.what());
-        return NodePtr();
+    xmlDocPtr doc;
+    doc = xmlParseMemory(sXML.c_str(), int(sXML.length()));
+    if (!doc) {
+        throw (Exception(AVG_ERR_XML_PARSE, 
+                    string("Error parsing xml:\n  ")+sXML));
     }
+    NodePtr pNode = createNodeFromXml(doc, xmlDocGetRootElement(doc), DivNodePtr());
+
+    xmlValidCtxtPtr cvp = xmlNewValidCtxt();
+    cvp->error = xmlParserValidityError;
+    cvp->warning = xmlParserValidityWarning;
+    int valid=xmlValidateDtd(cvp, doc, m_dtd);  
+    xmlFreeValidCtxt(cvp);
+    if (!valid) {
+        throw (Exception(AVG_ERR_XML_PARSE, 
+                    "Could not validate '"+sXML+"'"));
+    }
+
+    xmlFreeDoc(doc);
+    return pNode;
 }
 
 NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc, 
         const xmlNodePtr xmlNode, DivNodeWeakPtr pParent)
 {
-    const char * nodeType = (const char *)xmlNode->name;
     NodePtr curNode;
+    const char * nodeType = (const char *)xmlNode->name;
     
-    string id = getDefaultedStringAttr (xmlNode, "id", "");
-    if (!strcmp (nodeType, "avg")) {
-        curNode = NodePtr(new AVGNode(xmlNode, this));
-    } else if (!strcmp (nodeType, "div")) {
-        curNode = NodePtr(new DivNode(xmlNode, this));
-    } else if (!strcmp (nodeType, "image")) {
-        curNode = NodePtr(new Image(xmlNode, this));
-    } else if (!strcmp (nodeType, "words")) {
-        curNode = NodePtr(new Words(xmlNode, this));
-        string s = getXmlChildrenAsString(xmlDoc, xmlNode);
-        boost::dynamic_pointer_cast<Words>(curNode)->initText(s);
-    } else if (!strcmp (nodeType, "video")) {
-        curNode = NodePtr(new Video(xmlNode, this));
-    } else if (!strcmp (nodeType, "camera")) {
-        curNode = NodePtr(new CameraNode(xmlNode, this));
-    }
-    else if (!strcmp (nodeType, "panoimage")) {
-        curNode = NodePtr(new PanoImage(xmlNode, this));
-    } 
-    else if (!strcmp (nodeType, "text") || 
-               !strcmp (nodeType, "comment")) {
+    if (!strcmp (nodeType, "text") || 
+        !strcmp (nodeType, "comment")) {
         // Ignore whitespace & comments
         return NodePtr();
-    } else {
-        throw (Exception (AVG_ERR_XML_NODE_UNKNOWN, 
-            string("Unknown node type ")+(const char *)nodeType+" encountered."));
+    }
+    curNode = m_NodeFactory.createNode(nodeType, xmlNode, this);
+    if (!strcmp (nodeType, "words")) {
+        // TODO: This is an end-run around the generic serialization mechanism
+        // that will probably break at some point.
+        string s = getXmlChildrenAsString(xmlDoc, xmlNode);
+        boost::dynamic_pointer_cast<Words>(curNode)->setTextFromNodeValue(s);
     }
     curNode->setThis(curNode);
+
+    // If this is the root node, remember it.
+    AVGNodePtr pRootNode = boost::dynamic_pointer_cast<AVGNode>(curNode);
+    if (pRootNode) {
+        m_pRootNode = pRootNode;
+        m_pRootNode->setParent(DivNodeWeakPtr(), Node::NS_CONNECTED);
+    }
+
     // If this is a container, recurse into children
     DivNodePtr curDivNode = boost::dynamic_pointer_cast<DivNode>(curNode);
     if (curDivNode) {
@@ -818,12 +987,14 @@ void Player::handleTimers()
     vector<Timeout *>::iterator it;
     m_bInHandleTimers = true;
     vector<Timeout *> IntervalsFired;
-    
+   
     it = m_PendingTimeouts.begin();
     while (it != m_PendingTimeouts.end() && (*it)->IsReady(getFrameTime()) && !m_bStopping)
     {
         (*it)->Fire(getFrameTime());
-        if (!m_bCurrentTimeoutDeleted) {
+        if (m_bCurrentTimeoutDeleted) {
+            it = m_PendingTimeouts.begin();
+        } else {
             if ((*it)->IsInterval()) {
                 Timeout* pTempTimeout = *it;
                 it = m_PendingTimeouts.erase(it);
@@ -844,60 +1015,36 @@ void Player::handleTimers()
 }
 
 
-bool Player::handleEvent(Event * pEvent)
+bool Player::handleEvent(EventPtr pEvent)
 {
-    assert(pEvent); 
-    if(CursorEvent * pCursorEvent = dynamic_cast<CursorEvent*>(pEvent)) {
-        DPoint pos(pCursorEvent->getXPosition(), 
-                pCursorEvent->getYPosition());
-        int cursorID = pCursorEvent->getCursorID();
-        NodePtr pNode;
-        if (m_pEventCaptureNode.find(cursorID) != m_pEventCaptureNode.end()) {
-            NodeWeakPtr pEventCaptureNode = m_pEventCaptureNode[cursorID];
-            if (pEventCaptureNode.expired()) {
-                m_pEventCaptureNode.erase(cursorID);
-            } else {
-                pNode = m_pEventCaptureNode[cursorID].lock();
-            }
-        } 
-        if (!pNode) {
-            if (pEvent->getType() == Event::CURSOROVER ||
-                    pEvent->getType() == Event::CURSOROUT)
-            {
-                pNode = pCursorEvent->getElement();
-            } else {
-                pNode = m_pRootNode->getElementByPos(pos);
-            }
-        }
-        if (pNode != m_pLastMouseNode[cursorID] && 
-                pEvent->getType() != Event::CURSOROVER &&
-                pEvent->getType() != Event::CURSOROUT)
+    assert(pEvent);
+   
+    if (MouseEventPtr pMouseEvent = boost::dynamic_pointer_cast<MouseEvent>(pEvent)) {
+        m_MouseState.setEvent(pMouseEvent);
+        pMouseEvent->setLastDownPos(m_MouseState.getLastDownPos());
+    }
+    
+    if (CursorEventPtr pCursorEvent = boost::dynamic_pointer_cast<CursorEvent>(pEvent)) {
+        if (pEvent->getType() == Event::CURSOROUT || 
+                pEvent->getType() == Event::CURSOROVER)
         {
-            if (m_pLastMouseNode[cursorID] && m_pLastMouseNode[cursorID]->getSensitive()) {
-                sendOver(pCursorEvent, Event::CURSOROUT, 
-                        m_pLastMouseNode[cursorID]);
-            }
-            if (pNode && pNode->getSensitive()) {
-                sendOver(pCursorEvent, Event::CURSOROVER, pNode);
-            }
-            m_pLastMouseNode[cursorID] = pNode;
+            pEvent->trace();
+            pEvent->getElement()->handleEvent(pEvent);
+        } else {
+            handleCursorEvent(pCursorEvent);
         }
-        if (pCursorEvent->getType() == Event::CURSORUP && 
-            pCursorEvent->getSource() != CursorEvent::MOUSE) 
-        {
-            m_pLastMouseNode.erase(cursorID);
-        }
-        if (pNode && pNode->getSensitive()) {
-            pNode->handleEvent(pCursorEvent);
-        }
-    } else if ( KeyEvent * pKeyEvent = dynamic_cast<KeyEvent*>(pEvent)){
+    }
+    else if (KeyEventPtr pKeyEvent = boost::dynamic_pointer_cast<KeyEvent>(pEvent))
+    {
+        pEvent->trace();
         m_pRootNode->handleEvent(pKeyEvent);
-        if (pEvent->getType() == Event::KEYDOWN &&
-            pKeyEvent->getKeyCode() == 27) 
+        if (m_bStopOnEscape
+                && pEvent->getType() == Event::KEYDOWN
+                && pKeyEvent->getKeyCode() == avg::key::KEY_ESCAPE)
         {
             m_bStopping = true;
         }
-    }else {
+    } else {
         switch(pEvent->getType()){
             case Event::QUIT:
                 m_bStopping = true;
@@ -911,6 +1058,130 @@ bool Player::handleEvent(Event * pEvent)
     return true; 
 }
 
+void Player::sendFakeEvents()
+{
+    std::map<int, CursorStatePtr>::iterator it;
+    for (it=m_pLastCursorStates.begin(); it != m_pLastCursorStates.end(); ++it) {
+        CursorStatePtr state = it->second;
+        handleCursorEvent(state->getLastEvent(), true);
+    }
+}
+
+void Player::handleCursorEvent(CursorEventPtr pEvent, bool bOnlyCheckCursorOver)
+{
+    DPoint pos(pEvent->getXPosition(), pEvent->getYPosition());
+    int cursorID = pEvent->getCursorID();
+    // Find all nodes under the cursor.
+    vector<NodeWeakPtr> pCursorNodes = getElementsByPos(pos);
+
+    // Determine the nodes the event should be sent to.
+    vector<NodeWeakPtr> pDestNodes = pCursorNodes;
+    bool bIsCapturing = false;
+    if (m_pEventCaptureNode.find(cursorID) != m_pEventCaptureNode.end()) {
+        NodeWeakPtr pEventCaptureNode = m_pEventCaptureNode[cursorID];
+        if (pEventCaptureNode.expired()) {
+            m_pEventCaptureNode.erase(cursorID);
+        } else {
+            pDestNodes = vector<NodeWeakPtr>();
+            pDestNodes.push_back(pEventCaptureNode);
+            bIsCapturing = true;
+        }
+    } 
+
+    vector<NodeWeakPtr> pLastCursorNodes;
+    {
+        map<int, CursorStatePtr>::iterator it;
+        it = m_pLastCursorStates.find(cursorID);
+        if (it != m_pLastCursorStates.end()) {
+            pLastCursorNodes = it->second->getNodes();
+        }
+    }
+
+    // Send out events.
+    vector<NodeWeakPtr>::const_iterator itLast;
+    vector<NodeWeakPtr>::iterator itCur;
+    for (itLast = pLastCursorNodes.begin(); itLast != pLastCursorNodes.end(); ++itLast) {
+        NodePtr pLastNode = itLast->lock();
+        for (itCur = pCursorNodes.begin(); itCur != pCursorNodes.end(); ++itCur) {
+            if (itCur->lock() == pLastNode) {
+                break;
+            }
+        }
+        if (itCur == pCursorNodes.end()) {
+            if (!bIsCapturing || pLastNode == pDestNodes.begin()->lock()) {
+                sendOver(pEvent, Event::CURSOROUT, pLastNode);
+            }
+        }
+    } 
+
+    // Send over events.
+    for (itCur = pCursorNodes.begin(); itCur != pCursorNodes.end(); ++itCur) {
+        NodePtr pCurNode = itCur->lock();
+        for (itLast = pLastCursorNodes.begin(); itLast != pLastCursorNodes.end(); 
+                ++itLast) 
+        {
+            if (itLast->lock() == pCurNode) {
+                break;
+            }
+        }
+        if (itLast == pLastCursorNodes.end()) {
+            if (!bIsCapturing || pCurNode == pDestNodes.begin()->lock()) {
+                sendOver(pEvent, Event::CURSOROVER, pCurNode);
+            }
+        }
+    } 
+
+    if (!bOnlyCheckCursorOver) {
+        // Iterate through the nodes and send the event to all of them.
+        vector<NodeWeakPtr>::iterator it;
+        for (it = pDestNodes.begin(); it != pDestNodes.end(); ++it) {
+            NodePtr pNode = (*it).lock();
+            if (pNode) {
+                CursorEventPtr pNodeEvent =
+                        boost::dynamic_pointer_cast<CursorEvent>(pEvent->cloneAs(pEvent->getType()));
+                pNodeEvent->setElement(pNode);
+                if (pNodeEvent->getType() != Event::CURSORMOTION) {
+                    pNodeEvent->trace();
+                }
+                pNode->handleEvent(pNodeEvent);
+            }
+        }
+    }
+    if (pEvent->getType() == Event::CURSORUP && pEvent->getSource() != Event::MOUSE) {
+        // Cursor has disappeared: send out events.
+        if (bIsCapturing) {
+            NodePtr pNode = pDestNodes.begin()->lock();
+            sendOver(pEvent, Event::CURSOROUT, pNode);
+        } else {
+            vector<NodeWeakPtr>::iterator it;
+            for (it = pCursorNodes.begin(); it != pCursorNodes.end(); ++it) {
+                NodePtr pNode = it->lock();
+                sendOver(pEvent, Event::CURSOROUT, pNode);
+            } 
+        }
+        m_pLastCursorStates.erase(cursorID);
+    } else {
+        // Update list of nodes under cursor
+        if (m_pLastCursorStates.find(cursorID) != m_pLastCursorStates.end()) {
+            m_pLastCursorStates[cursorID]->setInfo(pEvent, pCursorNodes);
+        } else {
+            m_pLastCursorStates[cursorID] =
+                    CursorStatePtr(new CursorState(pEvent, pCursorNodes));
+        }
+    }
+}
+
+vector<NodeWeakPtr> Player::getElementsByPos(const DPoint& pos) const
+{
+    vector<NodeWeakPtr> Elements;
+    NodePtr pNode = m_pRootNode->getElementByPos(pos);
+    while (pNode) {
+        Elements.push_back(pNode);
+        pNode = pNode->getParent();
+    }
+    return Elements;
+}
+
 DisplayEngine * Player::getDisplayEngine() const 
 {
     return m_pDisplayEngine;
@@ -921,12 +1192,32 @@ void Player::useFakeCamera(bool bFake)
     m_bUseFakeCamera = bFake;
 }
 
-void Player::sendOver(CursorEvent * pOtherEvent, Event::Type Type, 
+void Player::stopOnEscape(bool bStop)
+{
+    m_bStopOnEscape = bStop;
+}
+
+void Player::setVolume(double volume)
+{
+    m_Volume = volume;
+    if (m_pAudioEngine) {
+        m_pAudioEngine->setVolume(m_Volume);
+    }
+}
+
+double Player::getVolume() const
+{
+    return m_Volume;
+}
+
+void Player::sendOver(const CursorEventPtr pOtherEvent, Event::Type Type, 
                 NodePtr pNode)
 {
-    Event * pNewEvent = pOtherEvent->cloneAs(Type);
-    pNewEvent->setElement(pNode);
-    m_pEventDispatcher->sendEvent(pNewEvent);
+    if (pNode) {
+        EventPtr pNewEvent = pOtherEvent->cloneAs(Type);
+        pNewEvent->setElement(pNode);
+        m_pEventDispatcher->sendEvent(pNewEvent);
+    }
 }
 
 void Player::cleanup() 
@@ -938,17 +1229,31 @@ void Player::cleanup()
     }
     m_PendingTimeouts.clear();
     Profiler::get().dumpStatistics();
-    m_pRootNode->disconnect();
+    if (m_pRootNode) {
+        m_pRootNode->disconnect();
+    }
     m_pRootNode = AVGNodePtr();
+
+    if (m_pTracker) {
+        delete m_pTracker;
+        m_pTracker = 0;
+    }
+
     if (m_pDisplayEngine) {
         m_pDisplayEngine->deinitRender();
         m_pDisplayEngine->teardown();
     }
-    m_pLastMouseNode.clear();
+    if (m_pAudioEngine) {
+        m_pAudioEngine->teardown();
+    }
+    AVG_TRACE(Logger::PROFILE, "Max. GPU memory used: " << m_MaxGPUMemUsed/1024 << "k");
+    
     m_IDMap.clear();
     m_pEventDispatcher = EventDispatcherPtr();
+    m_MouseState = MouseState();
     initConfig();
     m_FrameTime = 0;
+    m_bIsPlaying = false;
 }
 
 int Player::addTimeout(Timeout* pTimeout)
@@ -972,4 +1277,15 @@ void Player::removeTimeout(Timeout* pTimeout)
     m_PendingTimeouts.erase(it);
 }
 
+long long Player::getGPUMemoryUsage()
+{
+    if (m_pDisplayEngine) {
+        return m_pDisplayEngine->getGPUMemoryUsage();
+    } else {
+        return 0;
+    }
+}
+
+
+
 }
