@@ -1,6 +1,6 @@
 
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,12 +34,13 @@
 #include "KeyEvent.h"
 
 #include "OGLSurface.h"
-#include "OGLHelper.h"
 
+#include "../base/MathHelper.h"
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 #include "../base/ScopeTimer.h"
 #include "../base/Profiler.h"
+#include "../base/OSHelper.h"
 
 #include "../graphics/Filterflip.h"
 #include "../graphics/Filterfliprgb.h"
@@ -52,10 +53,6 @@
 #include <X11/extensions/xf86vmode.h>
 #endif
 
-//#define XMD_H 1
-#include "OGLHelper.h"
-
-
 #ifdef __APPLE__
 #include <OpenGL/OpenGL.h>
 #endif
@@ -68,7 +65,14 @@
 #include <signal.h>
 #include <iostream>
 #include <sstream>
+
+#ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#else
 #include <unistd.h>
+#endif
+
 #include <assert.h>
 
 using namespace std;
@@ -158,12 +162,15 @@ void dumpSDLGLParams() {
 }
 
 SDLDisplayEngine::SDLDisplayEngine()
-    : m_bEnableCrop(false),
+    : m_WindowWidth(0),
+      m_WindowHeight(0),
+      m_bEnableCrop(false),
       m_pScreen(0),
       m_VBMethod(VB_NONE),
       m_VBMod(0),
       m_dri_fd(0),
       m_bMouseOverApp(true),
+      m_LastMousePos(-1, -1),
       m_TextureMode(0),
       m_MaxTexSize(0),
       m_bCheckedMemoryMode(false)
@@ -180,15 +187,26 @@ SDLDisplayEngine::SDLDisplayEngine()
         exit(-1);
     }
     initTranslationTable();
+    SDL_EnableUNICODE(true);
 }
 
 SDLDisplayEngine::~SDLDisplayEngine()
 {
     SDL_QuitSubSystem(SDL_INIT_VIDEO);
+    assert(m_pSurfaces.size() == 0);
 }
 
 void SDLDisplayEngine::init(const DisplayParams& DP) 
 {
+    stringstream ss;
+    if (DP.m_x != -1) {
+        ss << DP.m_x << "," << DP.m_y;
+        setEnv("SDL_VIDEO_WINDOW_POS", ss.str().c_str());
+    }
+#ifdef linux
+    int oldWindowWidth = m_WindowWidth;
+    int oldWindowHeight = m_WindowHeight;
+#endif
     double AspectRatio = double(DP.m_Width)/double(DP.m_Height);
     if (DP.m_WindowWidth == 0 && DP.m_WindowHeight == 0) {
         m_WindowWidth = DP.m_Width;
@@ -200,7 +218,19 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
         m_WindowWidth = DP.m_WindowWidth;
         m_WindowHeight = int(DP.m_WindowWidth/AspectRatio);
     }
-    
+#ifdef linux
+    // Workaround for what appears to be an SDL or X11 bug:
+    // Stencil buffers stop working after a window resize, so we reinit
+    // everything.
+    if (oldWindowWidth != m_WindowWidth || oldWindowHeight != m_WindowHeight) {
+        SDL_QuitSubSystem(SDL_INIT_VIDEO);
+        if (SDL_InitSubSystem(SDL_INIT_VIDEO)==-1) {
+            AVG_TRACE(Logger::ERROR, "Can't init SDL display subsystem.");
+            exit(-1);
+        }
+    }
+#endif
+
     switch (DP.m_BPP) {
         case 32:
             safeSetAttribute( SDL_GL_RED_SIZE, 8 );
@@ -232,6 +262,7 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
             exit(-1);
     }
     safeSetAttribute(SDL_GL_DEPTH_SIZE, 0);
+    safeSetAttribute(SDL_GL_STENCIL_SIZE, 8);
     safeSetAttribute(SDL_GL_DOUBLEBUFFER, 1);
     if (m_MultiSampleSamples > 1) {
         safeSetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
@@ -251,18 +282,20 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
                 << m_WindowHeight << ", bpp=" << DP.m_BPP << ", multisamplesamples="
                 << m_MultiSampleSamples << ").");
         exit(-1);
-    }   
+    }
     glproc::init();
-    
+
     SDL_WM_SetCaption("AVG Renderer", 0);
     calcRefreshRate();
 
     glEnable(GL_BLEND);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_BLEND)");
     glShadeModel(GL_FLAT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glShadeModel()");
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glShadeModel(GL_FLAT)");
     glDisable(GL_DEPTH_TEST);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_DEPTH_TEST)");
+    glEnable(GL_STENCIL_TEST);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_STENCIL_TEST)");
     int TexMode = getTextureMode();
     glEnable(TexMode);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(TexMode);");
@@ -273,6 +306,8 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
         glDisable(GL_MULTISAMPLE);
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_MULTISAMPLE);");
     }
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     setGamma(DP.m_Gamma[0], DP.m_Gamma[1], DP.m_Gamma[2]);
     showCursor(DP.m_bShowCursor);
     if (DP.m_Framerate == 0) {
@@ -289,11 +324,14 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
     // SDL sets up a signal handler we really don't want.
     signal(SIGSEGV, SIG_DFL);
     logConfig();
-    
+
 //    dumpSDLGLParams();
     m_bEnableCrop = false;
 }
 
+#ifdef _WIN32
+#pragma warning(disable: 4996)
+#endif
 void SDLDisplayEngine::teardown()
 {
     if (m_VBMethod == VB_DRI) {
@@ -314,7 +352,7 @@ void SDLDisplayEngine::setGamma(double Red, double Green, double Blue)
 {
     if (Red > 0) {
         AVG_TRACE(Logger::CONFIG, "Setting gamma to " << Red << ", " << Green << ", " << Blue);
-        int err = SDL_SetGamma(Red, Green, Blue);
+        int err = SDL_SetGamma(float(Red), float(Green), float(Blue));
         if (err == -1) {
             AVG_TRACE(Logger::WARNING, "Unable to set display gamma.");
         }
@@ -369,29 +407,21 @@ void SDLDisplayEngine::logConfig()
                 "  Max. texture size is " << getMaxTexSize());
 }
 
-static ProfilingZone PrepareRenderProfilingZone("Root node: prepareRender");
 static ProfilingZone RootRenderProfilingZone("Root node: render");
 
-void SDLDisplayEngine::render(AVGNodePtr pRootNode, bool bRenderEverything)
+void SDLDisplayEngine::render(AVGNodePtr pRootNode)
 {
-    if (!m_bEnableCrop && pRootNode->getCropSetting()) {
-        m_bEnableCrop = true;
-        glEnable (GL_CLIP_PLANE0);
-        glEnable (GL_CLIP_PLANE1);
-        glEnable (GL_CLIP_PLANE2);
-        glEnable (GL_CLIP_PLANE3);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-                "setClipPlane: glEnable()");
-    }
-    {
-        ScopeTimer Timer(PrepareRenderProfilingZone);
-        pRootNode->prepareRender(0, pRootNode->getAbsViewport());
-    }
+    pRootNode->preRender();
+    m_bEnableCrop = pRootNode->getCropSetting();
+    
     glClearColor(0.0, 0.0, 0.0, 0.0); 
     glClear(GL_COLOR_BUFFER_BIT);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-            "SDLDisplayEngine::render::glClear()");
-
+            "SDLDisplayEngine::render::glClear(GL_COLOR_BUFFER_BIT)");
+    glClearStencil(0);
+    glClear(GL_STENCIL_BUFFER_BIT);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
+            "SDLDisplayEngine::render::glClear(GL_STENCIL_BUFFER_BIT)");
     glViewport(0, 0, m_WindowWidth, m_WindowHeight);
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
             "SDLDisplayEngine::render: glViewport()");
@@ -421,29 +451,16 @@ void SDLDisplayEngine::render(AVGNodePtr pRootNode, bool bRenderEverything)
     swapBuffers();
     checkJitter();
 }
- 
-void SDLDisplayEngine::setClipRect()
-{
-    m_ClipRects.clear();
-    m_ClipRects.push_back(DRect(0, 0, m_Width, m_Height));
-}
 
 static ProfilingZone PushClipRectProfilingZone("pushClipRect");
 
-bool SDLDisplayEngine::pushClipRect(const DRect& rc, bool bClip)
+bool SDLDisplayEngine::pushClipRect(const DRect& rc)
 {
     ScopeTimer Timer(PushClipRectProfilingZone);
 
     m_ClipRects.push_back(rc);
+    clip(true);
 
-    glPushMatrix();
-    AVG_TRACE(Logger::BLTS, "Clip set to " << 
-            rc.tl.x << "x" << rc.tl.y << 
-            ", width: " << rc.Width() << ", height: " << 
-            rc.Height());
-    if (bClip) {
-        clip();
-    }
     return true;
 }
 
@@ -452,9 +469,9 @@ static ProfilingZone PopClipRectProfilingZone("popClipRect");
 void SDLDisplayEngine::popClipRect()
 {
     ScopeTimer Timer(PopClipRectProfilingZone);
-    glPopMatrix();
+    
+    clip(false);
     m_ClipRects.pop_back();
-    clip();
 }
 
 const DRect& SDLDisplayEngine::getClipRect() 
@@ -462,57 +479,83 @@ const DRect& SDLDisplayEngine::getClipRect()
     return m_ClipRects.back();
 }
 
-void SDLDisplayEngine::blt32(ISurface * pSurface, const DRect* pDestRect, 
-        double opacity, double angle, const DPoint& pivot, BlendMode Mode)
+void SDLDisplayEngine::pushTransform(const DPoint& translate, double angle, const DPoint& pivot)
+{
+    glPushMatrix();
+    glTranslated(translate.x, translate.y, 0);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "pushTransform: glTranslated");
+    glTranslated(pivot.x, pivot.y, 0);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "pushTransform: glTranslated");
+    glRotated(angle*180.0/PI, 0, 0, 1);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "pushTransform: glRotated");
+    glTranslated(-pivot.x, -pivot.y, 0);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "pushTransform: glTranslated");
+}
+
+void SDLDisplayEngine::popTransform()
+{
+    glPopMatrix();
+}
+
+void SDLDisplayEngine::blt32(ISurface * pSurface, const DPoint& DestSize, 
+        double opacity, BlendMode Mode)
 {
     OGLSurface * pOGLSurface = dynamic_cast<OGLSurface*>(pSurface);
-    glColor4f(1.0f, 1.0f, 1.0f, opacity);
-    pOGLSurface->blt(pDestRect, angle, pivot, Mode);
+    glColor4d(1.0, 1.0, 1.0, opacity);
+    pOGLSurface->blt(DestSize, Mode);
 }
 
 void SDLDisplayEngine::blta8(ISurface * pSurface, 
-        const DRect* pDestRect, double opacity, 
-        const Pixel32& color, double angle, 
-        const DPoint& pivot, BlendMode Mode)
+        const DPoint& DestSize, double opacity, 
+        const Pixel32& color, BlendMode Mode)
 {
     OGLSurface * pOGLSurface = dynamic_cast<OGLSurface*>(pSurface);
-    glColor4f(float(color.getR())/256, float(color.getG())/256, 
-            float(color.getB())/256, opacity);
-    pOGLSurface->blt(pDestRect, angle, pivot, Mode);
+    glColor4d(double(color.getR())/256, double(color.getG())/256, 
+            double(color.getB())/256, opacity);
+    pOGLSurface->blt(DestSize, Mode);
 }
 
-
-void SDLDisplayEngine::setClipPlane(double Eqn[4], int WhichPlane)
+void SDLDisplayEngine::clip(bool forward)
 {
-    glClipPlane(WhichPlane, Eqn);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-            "setClipPlane: glClipPlane()");
-}
-
-void SDLDisplayEngine::clip()
-{
-    
-    if (m_bEnableCrop && !m_ClipRects.empty()) {       
+    if (m_bEnableCrop && !m_ClipRects.empty()) {
+        GLenum stencilOp;
+        int level;
+        if(forward) {
+            stencilOp = GL_INCR;
+            level = int(m_ClipRects.size());
+        } else {
+            stencilOp = GL_DECR;
+            level = int(m_ClipRects.size()) - 1;
+        }
+        
         DRect rc = m_ClipRects.back();
-        double yEqn[4] = {0.0, 1.0, 0.0, -rc.tl.y};
-        //    yEqn[3] = -rc.tl.y;
-        setClipPlane(yEqn, GL_CLIP_PLANE0);
+        
+        // Disable drawing to color buffer
+        glColorMask(0, 0, 0, 0);
+        
+        // Enable drawing to stencil buffer
+        glStencilMask(~0);
 
-        yEqn[0] = 0;
-        yEqn[1] = -1.0;
-        yEqn[2] = 0;
-        yEqn[3] = rc.br.y;
-        setClipPlane(yEqn, GL_CLIP_PLANE1);
+        // Draw clip rectangle into stencil buffer
+        glStencilFunc(GL_ALWAYS, 0, 0);
+        glStencilOp(stencilOp, stencilOp, stencilOp);
 
-        double xEqn[4] = {1.0, 0.0, 0.0, 0.0};
-        xEqn[3] = -rc.tl.x;
-        setClipPlane(xEqn, GL_CLIP_PLANE2);
+        glBegin(GL_QUADS);
+            glVertex2d(rc.tl.x, rc.tl.y);
+            glVertex2d(rc.br.x, rc.tl.y);
+            glVertex2d(rc.br.x, rc.br.y);
+            glVertex2d(rc.tl.x, rc.br.y);
+        glEnd();
 
-        xEqn[0] = -1;
-        xEqn[1] = 0.0;
-        xEqn[2] = 0.0;
-        xEqn[3] = rc.br.x;
-        setClipPlane(xEqn, GL_CLIP_PLANE3);
+        // Set stencil test to only let
+        glStencilFunc(GL_LEQUAL, level, ~0);
+        glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
+        
+        // Disable drawing to stencil buffer
+        glStencilMask(0);
+        
+        // Enable drawing to color buffer
+        glColorMask(~0, ~0, ~0, ~0);
     }
 }
 
@@ -528,7 +571,10 @@ void SDLDisplayEngine::swapBuffers()
 
 ISurface * SDLDisplayEngine::createSurface()
 {
-    return new OGLSurface(this);
+    OGLSurface * pOGLSurface = new OGLSurface(this);
+    m_pSurfaces.push_back(pOGLSurface);
+
+    return pOGLSurface;
 }
 
 void SDLDisplayEngine::surfaceChanged(ISurface * pSurface) 
@@ -564,19 +610,25 @@ OGLShaderPtr SDLDisplayEngine::getYCbCrJ420pShader()
 
 void SDLDisplayEngine::showCursor (bool bShow)
 {
+#ifdef _WIN32
+    ShowCursor(bShow);
+#else
     if (bShow) {
         SDL_ShowCursor(SDL_ENABLE);
     } else {
         SDL_ShowCursor(SDL_DISABLE);
     }
+#endif
 }
 
 BitmapPtr SDLDisplayEngine::screenshot ()
 {
     BitmapPtr pBmp (new Bitmap(IntPoint(m_Width, m_Height), R8G8B8X8, "screenshot"));
-    glReadBuffer(GL_FRONT);
+    glReadBuffer(GL_BACK);
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "SDLDisplayEngine::screenshot:glReadBuffer()");
     glReadPixels(0, 0, m_Width, m_Height, GL_RGBA, GL_UNSIGNED_BYTE, 
             pBmp->getPixels());
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "SDLDisplayEngine::screenshot:glReadPixels()");
 //    FilterFlipRGB().applyInPlace(pBmp);
     FilterFlip().applyInPlace(pBmp);
     return pBmp;
@@ -602,12 +654,10 @@ void SDLDisplayEngine::checkYCbCrSupport()
     m_YCbCrMode = OGL_NONE;
     if (queryOGLExtension("GL_ARB_fragment_shader") &&
         queryOGLExtension("GL_ARB_texture_rectangle") &&
-        (queryOGLExtension("GL_ARB_pixel_buffer_object") || 
-         queryOGLExtension("GL_EXT_pixel_buffer_object")) &&
+        getMemoryModeSupported() == PBO &&
         m_DesiredYCbCrMode == OGL_SHADER && 
-        m_bShouldUsePixelBuffers &&
         getTextureMode() != GL_TEXTURE_2D
-        )
+       )
     {
         m_YCbCrMode = OGL_SHADER;
         string sProgramInit =
@@ -675,7 +725,11 @@ bool SDLDisplayEngine::initVBlank(int rate) {
             AVG_TRACE(Logger::WARNING,
                     "VBlank rate set to " << rate << " but Mac OS X only supports 1. Assuming 1.");
         }
+#if MAC_OS_X_VERSION_10_5
+        const GLint l = 1;
+#else
         const long l = 1;
+#endif
         CGLError err = CGLSetParameter(Context, kCGLCPSwapInterval, &l);
         m_VBMethod = VB_APPLE;
         if (err) {
@@ -684,7 +738,16 @@ bool SDLDisplayEngine::initVBlank(int rate) {
                     CGLErrorString(err) << "(" << err << ").");
             m_VBMethod = VB_NONE;
         }
-#elif defined linux
+#elif defined _WIN32
+        if (queryOGLExtension("WGL_EXT_swap_control")) {
+            glproc::SwapIntervalEXT(rate);
+            m_VBMethod = VB_WIN;
+        } else {
+            AVG_TRACE(Logger::WARNING,
+                    "Windows VBlank setup failed: OpenGL Extension not supported.");
+            m_VBMethod = VB_NONE;
+        }
+#else
         if (getenv("__GL_SYNC_TO_VBLANK") != 0) 
         {
             AVG_TRACE(Logger::WARNING, 
@@ -726,9 +789,14 @@ bool SDLDisplayEngine::initVBlank(int rate) {
         case VB_APPLE:
             AVG_TRACE(Logger::CONFIG, "  Using Apple GL vertical blank support.");
             break;
+        case VB_WIN:
+            AVG_TRACE(Logger::CONFIG, "  Using Windows GL vertical blank support.");
+            break;
         case VB_NONE:
             AVG_TRACE(Logger::CONFIG, "  Vertical blank support disabled.");
             break;
+        default:
+            AVG_TRACE(Logger::WARNING, "  Illegal vblank enum value.");
     }
     return m_VBMethod != VB_NONE;
 }
@@ -786,6 +854,7 @@ bool SDLDisplayEngine::vbWait(int rate) {
             }
 #endif
         case VB_APPLE:
+        case VB_WIN:
             return true;
         case VB_NONE:
         default:
@@ -817,7 +886,15 @@ void SDLDisplayEngine::calcRefreshRate() {
         AVG_TRACE(Logger::WARNING, 
                 "Apple refresh rate calculation (CGDisplayCurrentMode) failed");
     }
-#elif defined linux 
+#elif defined _WIN32
+     // This isn't correct for multi-monitor systems.
+    HDC hDC = CreateDC("DISPLAY", NULL,NULL,NULL);
+    s_RefreshRate = GetDeviceCaps(hDC, VREFRESH);
+    if (s_RefreshRate < 2) {
+        s_RefreshRate = 60;
+    }
+    DeleteDC(hDC);
+#else 
     Display * display = XOpenDisplay(0);
     int PixelClock;
     XF86VidModeModeLine mode_line;
@@ -886,27 +963,27 @@ const char * getEventTypeName(unsigned char Type)
     }
 }
 
-vector<Event *> SDLDisplayEngine::pollEvents()
+vector<EventPtr> SDLDisplayEngine::pollEvents()
 {
     SDL_Event sdlEvent;
-    vector<Event *> Events;
+    vector<EventPtr> Events;
 
-    while(SDL_PollEvent(&sdlEvent)){
+    while(SDL_PollEvent(&sdlEvent)) {
         switch(sdlEvent.type) {
             case SDL_ACTIVEEVENT:
                 if (sdlEvent.active.state & SDL_APPMOUSEFOCUS) {
-                    m_bMouseOverApp = sdlEvent.active.gain;
+                    m_bMouseOverApp = sdlEvent.active.gain != 0;
                     if (!sdlEvent.active.gain) {
-                        Events.push_back(
+                        Events.push_back(EventPtr(
                                 new MouseEvent (Event::CURSORMOTION, false,
                                         false, false, IntPoint(-1, -1), 
-                                        MouseEvent::NO_BUTTON));
+                                        MouseEvent::NO_BUTTON, DPoint(0,0))));
                     }
                 }
             case SDL_MOUSEMOTION:
                 if (m_bMouseOverApp) {
                     Events.push_back
-                            (createMouseMotionEvent(Event::CURSORMOTION, sdlEvent));
+                            (createMouseEvent(Event::CURSORMOTION, sdlEvent, MouseEvent::NO_BUTTON));
                 }
                 break;
             case SDL_MOUSEBUTTONDOWN:
@@ -934,7 +1011,7 @@ vector<Event *> SDLDisplayEngine::pollEvents()
                 break;
             case SDL_QUIT:
                 {
-                    Event * pEvent = new Event(Event::QUIT, Event::NONE);
+                    EventPtr pEvent(new Event(Event::QUIT, Event::NONE));
                     Events.push_back(pEvent);
                 }
                 break;
@@ -948,21 +1025,29 @@ vector<Event *> SDLDisplayEngine::pollEvents()
     return Events;
 }
 
-Event * SDLDisplayEngine::createMouseMotionEvent
-        (Event::Type Type, const SDL_Event & SDLEvent)
+EventPtr SDLDisplayEngine::createMouseEvent
+        (Event::Type Type, const SDL_Event & SDLEvent, long Button)
 {
-    int x = int((SDLEvent.motion.x*m_Width)/m_WindowWidth);
-    int y = int((SDLEvent.motion.y*m_Height)/m_WindowHeight);
-    MouseEvent * pEvent = new MouseEvent (Type, 
-            SDLEvent.motion.state & SDL_BUTTON(1),
-            SDLEvent.motion.state & SDL_BUTTON(3), 
-            SDLEvent.motion.state & SDL_BUTTON(2),
-            IntPoint(x, y), 
-            MouseEvent::NO_BUTTON);
-    return pEvent;
+    int x,y;
+    Uint8 buttonState = SDL_GetMouseState(&x, &y);
+    x = int((x*m_Width)/m_WindowWidth);
+    y = int((y*m_Height)/m_WindowHeight);
+    DPoint speed;
+    if (m_LastMousePos.x == -1) {
+        speed = DPoint(0,0);
+    } else {
+        double lastFrameTime = 1000/getFramerate();
+        speed = DPoint(x-m_LastMousePos.x, y-m_LastMousePos.y)/lastFrameTime;
+    }
+    MouseEventPtr pEvent(new MouseEvent(Type, bool(buttonState & SDL_BUTTON(1)),
+            bool(buttonState & SDL_BUTTON(2)), bool(buttonState & SDL_BUTTON(3)),
+            IntPoint(x, y), Button, speed));
+    m_LastMousePos = IntPoint(x,y);
+    return pEvent; 
+
 }
 
-Event * SDLDisplayEngine::createMouseButtonEvent
+EventPtr SDLDisplayEngine::createMouseButtonEvent
         (Event::Type Type, const SDL_Event & SDLEvent) 
 {
     long Button = 0;
@@ -976,28 +1061,26 @@ Event * SDLDisplayEngine::createMouseButtonEvent
         case SDL_BUTTON_RIGHT:
             Button = MouseEvent::RIGHT_BUTTON;
             break;
+        case SDL_BUTTON_WHEELUP:
+            Button = MouseEvent::WHEELUP_BUTTON;
+            break;
+        case SDL_BUTTON_WHEELDOWN:
+            Button = MouseEvent::WHEELDOWN_BUTTON;
+            break;
     }
-    int x,y;
-    SDL_GetMouseState(&x, &y);
-    x = int((x*m_Width)/m_WindowWidth);
-    y = int((y*m_Height)/m_WindowHeight);
-    MouseEvent * pEvent = new MouseEvent(Type, 
-            SDLEvent.button.button == SDL_BUTTON_LEFT,
-            SDLEvent.button.button == SDL_BUTTON_MIDDLE, 
-            SDLEvent.button.button == SDL_BUTTON_RIGHT,
-            IntPoint(x, y), Button);
-    return pEvent; 
+    return createMouseEvent(Type, SDLEvent, Button);
+ 
 }
 
 /*
-Event * SDLDisplayEngine::createAxisEvent(const SDL_Event & SDLEvent)
+EventPtr SDLDisplayEngine::createAxisEvent(const SDL_Event & SDLEvent)
 {
     return new AxisEvent(SDLEvent.jaxis.which, SDLEvent.jaxis.axis,
                 SDLEvent.jaxis.value);
 }
 
 
-Event * SDLDisplayEngine::createButtonEvent
+EventPtr SDLDisplayEngine::createButtonEvent
         (Event::Type Type, const SDL_Event & SDLEvent) 
 {
     return new ButtonEvent(Type, SDLEvent.jbutton.which,
@@ -1005,7 +1088,7 @@ Event * SDLDisplayEngine::createButtonEvent
 }
 */
 
-Event * SDLDisplayEngine::createKeyEvent
+EventPtr SDLDisplayEngine::createKeyEvent
         (Event::Type Type, const SDL_Event & SDLEvent)
 {
     long KeyCode = KeyCodeTranslationTable[SDLEvent.key.keysym.sym];
@@ -1036,9 +1119,9 @@ Event * SDLDisplayEngine::createKeyEvent
     if (SDLEvent.key.keysym.mod & KMOD_RESERVED) 
         { Modifiers |= key::KEYMOD_RESERVED; }
 
-    KeyEvent * pEvent = new KeyEvent(Type, 
+    KeyEventPtr pEvent(new KeyEvent(Type,
             SDLEvent.key.keysym.scancode, KeyCode,
-            SDL_GetKeyName(SDLEvent.key.keysym.sym), Modifiers);
+            SDL_GetKeyName(SDLEvent.key.keysym.sym), SDLEvent.key.keysym.unicode, Modifiers));
     return pEvent;
 }
 
@@ -1413,7 +1496,7 @@ int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 #endif 
         case B8G8R8X8:
         case B8G8R8A8:
-            return GL_UNSIGNED_INT_8_8_8_8_REV;
+//            return GL_UNSIGNED_INT_8_8_8_8_REV;
         default:
             return GL_UNSIGNED_BYTE;
     }
@@ -1421,10 +1504,13 @@ int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 
 OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
 {
+    bool bIsParallels = 
+            (string((char*)glGetString(GL_VENDOR)).find("Parallels") != string::npos);
     if (!m_bCheckedMemoryMode) {
         if ((queryOGLExtension("GL_ARB_pixel_buffer_object") || 
              queryOGLExtension("GL_EXT_pixel_buffer_object")) &&
-            m_bShouldUsePixelBuffers)
+            m_bShouldUsePixelBuffers &&
+            !bIsParallels)
         {
             m_MemoryMode = PBO;
         } else {
@@ -1453,5 +1539,30 @@ void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCb
     m_DesiredVSyncMode = DesiredVSyncMode;
 }
 
+long long SDLDisplayEngine::getGPUMemoryUsage()
+{
+    vector<OGLSurface *>::iterator it;
+
+    long long lAmount = 0;
+    for(it = m_pSurfaces.begin() ; it != m_pSurfaces.end() ; ++it) {
+        lAmount += (*it)->getTotalTexMemory();
+    }
+    
+    return lAmount;
 }
 
+void SDLDisplayEngine::deregisterSurface(OGLSurface *pOGLSurface)
+{
+    vector<OGLSurface *>::iterator it;
+
+    for(it = m_pSurfaces.begin() ; it != m_pSurfaces.end() ; ++it) {
+        if (pOGLSurface == *it) {
+            m_pSurfaces.erase(it);
+            return;
+        }
+    }
+
+    assert(false);
+}
+
+}
