@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include "TestHelper.h"
 #include "Player.h"
 #include "MouseEvent.h"
+#include "KeyEvent.h"
 
 #include "../base/ObjectCounter.h"
 
@@ -40,11 +41,6 @@ TestHelper::~TestHelper()
 {
 }
 
-int TestHelper::getNumDifferentPixels(Bitmap* pBmp1, Bitmap* pBmp2)
-{
-    return pBmp1->getNumDifferentPixels(*pBmp2);
-}
-
 void TestHelper::useFakeCamera(bool bFake)
 {
     m_pPlayer->useFakeCamera(bFake);
@@ -55,8 +51,18 @@ void TestHelper::fakeMouseEvent(Event::Type eventType,
         bool rightButtonState,
         int xPosition, int yPosition, int button)
 {
-    MouseEvent * pEvent = new MouseEvent(eventType, leftButtonState, 
-            middleButtonState, rightButtonState, IntPoint(xPosition, yPosition), button);
+    MouseEventPtr pEvent(new MouseEvent(eventType, leftButtonState, 
+            middleButtonState, rightButtonState, IntPoint(xPosition, yPosition), button,
+            DPoint(0,0)));
+    m_Events.push_back(pEvent);
+}
+
+void TestHelper::fakeKeyEvent(Event::Type eventType,
+        unsigned char scanCode, int keyCode, 
+        const string& keyString, int unicode, int modifiers)
+{
+    KeyEventPtr pEvent(new KeyEvent(eventType, scanCode, keyCode, 
+        keyString, unicode, modifiers));
     m_Events.push_back(pEvent);
 }
 
@@ -66,9 +72,9 @@ void TestHelper::dumpObjects()
 }
 
 // From IEventSource
-std::vector<Event *> TestHelper::pollEvents()
+std::vector<EventPtr> TestHelper::pollEvents()
 {
-    vector<Event *> TempEvents = m_Events;
+    vector<EventPtr> TempEvents = m_Events;
     m_Events.clear();
     return TempEvents;
 }
