@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "../base/Exception.h"
 #include "../base/ObjectCounter.h"
 
-#include <boost/python.hpp>
+#include "BoostPython.h"
 
 #include <iostream>
 
@@ -66,16 +66,18 @@ bool Timeout::IsInterval() const
 
 void Timeout::Fire(long long CurTime)
 {
+    if (m_IsInterval) {
+        m_NextTimeout = m_Interval + CurTime;
+    }
     PyObject * arglist = Py_BuildValue("()");
     PyObject * result = PyEval_CallObject(m_PyFunc, arglist);
+    // XXX: After the call to python, this might have been deleted 
+    // by a call to clearTimeout()!
     Py_DECREF(arglist);    
     if (!result) {
         throw error_already_set();
     }
     Py_DECREF(result);
-    if (m_IsInterval) {
-        m_NextTimeout = m_Interval + CurTime;
-    }
 }
 
 int Timeout::GetID() const
