@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,17 +22,8 @@
 #ifndef _Timeout_H_
 #define _Timeout_H_
 
-#include <Python.h> // Python doc sais this should be included before 
-                              // any standard headers (!)
-#undef HAVE_STAT
-#undef PACKAGE
-#undef PACKAGE_BUGREPORT
-#undef PACKAGE_NAME
-#undef PACKAGE_STRING
-#undef PACKAGE_TARNAME
-#undef PACKAGE_VERSION
-
-#include <string>
+// Python docs say python.h should be included before any standard headers (!)
+#include "WrapPython.h" 
 
 namespace avg {
 
