@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -28,11 +28,26 @@
 #define EMULATE_INTTYPES
 #else
 // This is probably GCC-specific.
-#define INT64_C(c)    c ## L
+#if !defined INT64_C
+#if defined __WORDSIZE && __WORDSIZE == 64
+#define INT64_C(c) c ## L
+#else
+#define INT64_C(c) c ## LL
 #endif
+#endif
+#endif
+
 extern "C" {
+#ifdef _WIN32
+#pragma warning(push)
+#pragma warning(disable:4244)
+#endif
 #include <ffmpeg/avformat.h>
+#ifdef _WIN32
+#pragma warning(pop)
+#endif
 }
+
 #include <list>
 #include <map>
 
@@ -47,7 +62,7 @@ namespace avg {
            
             void enableStream(int StreamIndex);
             AVPacket * getPacket(int StreamIndex);
-            void seek(int DestFrame, int StartTimestamp, int StreamIndex);
+            void seek(long long DestTime);
             void dump();
             
         private:
