@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2006 Ulrich von Zadow
+//  Copyright (C) 2003-2008 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,8 +32,15 @@ namespace avg {
 
 class SeekDoneVideoMsg: public VideoMsg {
     public:
-        SeekDoneVideoMsg() {};
-        virtual ~SeekDoneVideoMsg() {};
+        SeekDoneVideoMsg(long long VideoFrameTime, long long AudioFrameTime);
+        virtual ~SeekDoneVideoMsg();
+        
+        long long getVideoFrameTime();
+        long long getAudioFrameTime();
+        
+    private:
+        long long m_VideoFrameTime;
+        long long m_AudioFrameTime;
 };
 
 typedef boost::shared_ptr<SeekDoneVideoMsg> SeekDoneVideoMsgPtr;
